/*
   FORM_MAI.CPP
   Main form handling functions.
   Copyright (c) Bachmann Software and Services, 1999-2002
   Author: Glenn Bachmann
*/

// system headers
#include <Palmos.h>
#include <SysEvtMgr.h>

// application-specific headers
#include "forms.h"
#include "form_res.h"

static void    MainFormInit (FormPtr formP);
static Boolean MainFormButtonHandler (FormPtr formP, EventPtr eventP);
static Boolean MainFormMenuHandler (FormPtr formP, EventPtr eventP);

void AboutFormShow ( void );
Err PalmMessageBox ( Int16 wAlertID, char * pMessage );
void ShoppingListShow ( void );
Boolean CreateShoppingList_EventHandler (EventPtr pEvent);

/*
   MainFormEventHandler:
   Parms:   pEvent   - event to be handled.
   Return:  true  - handled (successfully or not)
            false - not handled
*/
Boolean
MainFormEventHandler (EventPtr eventP)
{
   Boolean  handled = false;
   
   switch (eventP->eType)
   {
      case menuEvent:
      {
         FormPtr formP = FrmGetActiveForm ();
         handled = MainFormMenuHandler (formP, eventP);
         break;
      }
      
      case ctlSelectEvent:
      {
      // A control button was pressed.
         FormPtr formP = FrmGetActiveForm ();
         handled = MainFormButtonHandler (formP, eventP);
         break;
      }
      
      case frmOpenEvent:
      {
         // main form is opening

         FormPtr formP = FrmGetActiveForm ();

         MainFormInit (formP);

         FrmDrawForm (formP);

         handled = true;
         break;
      }
      
      default:
      {
         break;
      }
   }
   return handled;
}

/*
   MainFormInit:
   Initialize the main form.
   Parms:   formP - pointer to main form.
   Return:  none
*/
void
MainFormInit (FormPtr formP)
{
   // if we had any special setup to do upon form loading, we
   // would do it here
}

/*
   MainFormButtonHandler:
   Handle a command sent to the main form.
   Parms:   formP    - form handling event.
            command  - command to be handled.
   Return:  true  - handled (successfully or not)
            false - not handled
*/
Boolean
MainFormMenuHandler (FormPtr /*formP*/, EventPtr eventP)
{
   Boolean handled = false;
   
   switch (eventP->data.menu.itemID)
   {
      default:
      {
         handled = false;
         break;
      }
   }
   return handled;
}

/*
   MainFormButtonHandler:
   Handle a command sent to the main form.
   Parms:   formP    - form handling event.
            eventP   - event to be handled.
   Return:  true  - handled (successfully or not)
            false - not handled
*/
Boolean
MainFormButtonHandler (FormPtr formP, EventPtr eventP)
{
   Boolean handled = false;
   
   switch (eventP->data.ctlEnter.controlID)
   {
      // button handling

      case MainAlertButton:   
      {
         // simple alert
         Int16 wButton;

         wButton = FrmAlert (AbortRetryCancelAlert);

         handled = true;
         break;
      }

      case MainMessageBoxButton:   
      {
         // call our own MessageBox api

         PalmMessageBox (MessageBoxAlert, "You just pressed a button!");

         handled = true;
         break;
      }

      case MainAboutButton:   
      {
         // show an about box

         AboutFormShow ();

         handled = true;
         break;
      }

      case MainShoppingListButton:   
      {
         // show a shopping list form

         ShoppingListShow ();

         handled = true;
         break;
      }

   }
   return handled;
}

/*
   AboutFormShow:

   Displays an application about box

   Parms: none

   Returns: none
*/

void
AboutFormShow ( void )
{
   FormPtr pForm = FrmInitForm ( AboutForm );

   if (pForm)
   {
      FrmDoDialog (pForm);
      FrmDeleteForm (pForm);
   }
}

/*
   PalmMessageBox:

   Toolbox api providing a general purpose message box function

   Parms: wAlertID = id of alert to show
          pMessage = ptr to message text to display

   Returns: id of button chosen by user
*/

Err
PalmMessageBox ( Int16 wAlertID, char * pMessage )
{
   Err err;

   err = FrmCustomAlert (wAlertID, pMessage, "", "" );

   return err;
}

/*
   ShoppingListShow:
   Displays the Create Shopping List dialog, allowing the user to select items to add to the shopping list.
   Parms:   none
   Return:  none
*/
void ShoppingListShow ( void )
{
   Boolean bGetMilk;
   Boolean bGetCookies;

   // initialize the form
   FormPtr  pForm = FrmInitForm (CreateShoppingListForm);

   // preset the checkboxes for all items to be unchecked 
   // get the index of the checkbox form object
   Int16 wIDMilk = FrmGetObjectIndex (pForm,CreateShoppingListMilkCheckbox );   
   Int16 wIDCookies = FrmGetObjectIndex (pForm,CreateShoppingListCookiesCheckbox );   

   // given the object index, get a pointer to the control object for the checkbox
   ControlPtr pCtlMilk = (ControlPtr) FrmGetObjectPtr (pForm, wIDMilk);
   ControlPtr pCtlCookies = (ControlPtr) FrmGetObjectPtr (pForm, wIDCookies);

   // use the object pointer, set the checkbox value to 0 (False)
   CtlSetValue ( pCtlMilk, 0 );
   CtlSetValue ( pCtlCookies, 0 );

	// we set an event handler for the form
	FrmSetEventHandler (pForm, CreateShoppingList_EventHandler );
   
   // display the dialog
   FrmDoDialog (pForm);

   // get the values from the checkbox
   bGetMilk = CtlGetValue ( pCtlMilk );

   bGetCookies = CtlGetValue (pCtlCookies);

   // destroy the form
   FrmDeleteForm (pForm);
}

/*
   CreateShoppingList_EventHandler:
	Handles form events for Shopping List form
   Parms:   pEvent = ptr to event handler
   Return:  TRUE if we handled the event
*/

Boolean CreateShoppingList_EventHandler (EventPtr pEvent)
{
   Boolean  bHandled = false;
   switch (pEvent->eType)
   {
      case ctlSelectEvent:
      {
         // A control button was pressed.
         switch (pEvent->data.ctlEnter.controlID)
         {
         	case CreateShoppingListDateButton:
            {
            	Int16 mth, day, year;

					mth = 1;
					day = 31;
					year = 1999;

              	if ( SelectDay ( selectDayByDay, &mth, &day, &year, "Select a Shopping Day") )
             	{
               	FormPtr pForm = FrmGetActiveForm ();
                 	Int16 wIDDate = FrmGetObjectIndex (pForm, CreateShoppingListDateButton );   

                	// given the object index, get a pointer to the control object
                	ControlPtr pDate = (ControlPtr) FrmGetObjectPtr (pForm, wIDDate);
               	char szDate[20];
               	StrPrintF ( szDate, "%d/%d/%d", mth, day, year );
               	CtlSetLabel ( pDate, szDate );
		         }

      		   bHandled = true;
				}
	         break;
	      }
		}
	}
   return bHandled;
}
