/*
   LIST_MAI.CPP
   Main form handling functions.
   Copyright (c) Bachmann Software and Services, 1999-2002
   Author: Glenn Bachmann
*/

// system headers
#include <PalmOS.h>
#include <SysEvtMgr.h>

// application-specific headers
#include "lists.h"
#include "list_res.h"

static void    MainFormInit (FormPtr pForm);
static Boolean MainFormButtonHandler (FormPtr pForm, EventPtr eventP);
static Boolean MainFormMenuHandler (FormPtr pForm, EventPtr eventP);

// description of a single shopping item
typedef struct
{
   char     szItem[50];
} ShoppingItem;

// pre-defined arrays of available shopping items
static ShoppingItem FruitsAndVegetables[] =
{
   { "Apples" },
   { "Lettuce" },
   { "Oranges" },
   { "Strawberries" },
   { 0 }
};

static ShoppingItem Dairy[] =
{
   { "Cheese" },
   { "Milk" },
   { "Orange Juice" },
   { "Yogurt"},
   { 0 }
};

static ShoppingItem Meats[] =
{
   { "Beef" },
   { "Chicken" },
   { "Fish"  },
   { "Turkey"},
   { 0 }
};

// array to hold selected items
static ShoppingItem s_Selected[13];

// count of how many items are selected thus far
static int s_iNumSelected;

// local helper functions
void MainFillMainList (FormPtr pForm);
void MainListDrawFunction (Int16 itemNum, RectanglePtr bounds, char **pUnused);

void MainFillSelectedList (FormPtr pForm);
void SelectedListDrawFunction (Int16 itemNum, RectanglePtr bounds, char **pUnused);

char * MainGetAisleSelection (FormPtr pForm);
char * MainGetMainSelection (FormPtr pForm);

char * MainGetMainText (FormPtr pFrom, Int16 wSelect);

/*
   MainFormEventHandler:
   Parms:   pEvent   - event to be handled.
   Return:  true  - handled (successfully or not)
            false - not handled
*/
Boolean
MainFormEventHandler (EventPtr eventP)
{
   Boolean  handled = false;
   
   switch (eventP->eType)
   {
      case menuEvent:
      {
         FormPtr pForm = FrmGetActiveForm ();
         handled = MainFormMenuHandler (pForm, eventP);
         break;
      }

      // note we grap popSelectEvent here to trap the popup trigger
      case popSelectEvent:
      case ctlSelectEvent:
      {
         // A control button was tapped and released
         FormPtr pForm = FrmGetActiveForm ();
         handled = MainFormButtonHandler (pForm, eventP);
         break;
      }
      
      case frmOpenEvent:
      {
         // main form is opening

         FormPtr pForm = FrmGetActiveForm ();

         MainFormInit (pForm);

         FrmDrawForm (pForm);

         handled = true;
         break;
      }
      
      default:
      {
         break;
      }
   }
   return handled;
}

/*
   MainFormInit:
   Initialize the main form.
   Parms:   pForm - pointer to main form.
   Return:  none
*/
void
MainFormInit (FormPtr pForm)
{
   // get a ptr to the main list
   UInt16 wIDMainList = FrmGetObjectIndex (pForm, MainMainList );
   ListPtr pCtlMainList = (ListPtr) FrmGetObjectPtr (pForm, wIDMainList);

   // get a ptr to the selected list
   UInt16 wIDSelectedList = FrmGetObjectIndex (pForm, MainSelectedList);
   ListPtr pCtlSelectedList = (ListPtr) FrmGetObjectPtr (pForm, wIDSelectedList);

   // get a ptr to the aisle (popup) list
   UInt16 wIDAisleList = FrmGetObjectIndex (pForm, MainAisleList );
   ListPtr pCtlAisleList = (ListPtr) FrmGetObjectPtr (pForm, wIDAisleList);

   // set the combo box to select the first item
   LstSetSelection ( pCtlAisleList, 0 );

   // fill the main list based on the current aisle
   MainFillMainList (pForm);

   // set the drawing function for the main list
   LstSetDrawFunction ( pCtlMainList, MainListDrawFunction );

   // fill the selected list
   MainFillSelectedList (pForm);

   // set the drawing function for the selected list
   LstSetDrawFunction ( pCtlSelectedList, SelectedListDrawFunction );
}

/*
   MainFillMainList:
   Fills the main list based upon the currently selected aisle
   Parms: pForm - pointer to main form
   Returns: none
*/
void
MainFillMainList (FormPtr pForm)
{
   // get a ptr to the main list
   UInt16 wIDMainList = FrmGetObjectIndex (pForm, MainMainList );
   ListPtr pCtlMainList = (ListPtr) FrmGetObjectPtr (pForm, wIDMainList);

   int iNumMainChoices = 0;

   // determine the current aisle
   char * pAisle = MainGetAisleSelection (pForm);

   // get the count of available main list items from the selected aisle
   if (0 == StrCompare (pAisle, "Fruits and Vegetables"))
   {
      iNumMainChoices = sizeof ( FruitsAndVegetables ) / sizeof ( ShoppingItem ) - 1;
   }
   else if (0 == StrCompare (pAisle, "Dairy"))
   {
      iNumMainChoices = sizeof ( Dairy ) / sizeof ( ShoppingItem ) - 1;
   }
   else if (0 == StrCompare (pAisle, "Meats"))
   {
      iNumMainChoices = sizeof ( Meats ) / sizeof ( ShoppingItem ) - 1;
   }

   // fill the main list based on the combo selection
   LstSetListChoices ( pCtlMainList, NULL, iNumMainChoices );

   // redraw the list
   LstDrawList (pCtlMainList);

   return;
}

/*
   MainFillSelectedList:
   Fills the shopping selection list 
   Parms: pForm - pointer to main form
   Returns: none
*/
void
MainFillSelectedList (FormPtr pForm)
{
   // get ptr to selected list
   UInt16 wID = FrmGetObjectIndex (pForm, MainSelectedList);
   ListPtr pList = (ListPtr) FrmGetObjectPtr (pForm, wID);

   // fill the selected list based on how many have been selected so far
   LstSetListChoices ( pList, NULL, s_iNumSelected );

   LstDrawList (pList);

   return;
}

/*
   MainListDrawFunction:
   Draws a single item in the main list
   Parms: itemNum = 0 based index of item to draw
          bound = ptr to rectangle providing drawing bounds
          pUnused = ptr to char string for item to draw. In this app, we
          never give the actual item data to the list, so we ignore this parm
   Returns: none
*/
void 
MainListDrawFunction (Int16 itemNum, RectanglePtr bounds, char **pUnused)
{
   FormPtr pForm = FrmGetActiveForm ();

   // get the text to be drawn by looking it up in the appropriate
   // aisle
   char * pText = MainGetMainText (pForm, itemNum);

   if ( pText )
   {
      // draw it
      WinDrawChars (pText, StrLen (pText), bounds->topLeft.x, bounds->topLeft.y);
   }   
   return;
}

/*
   SelectedListDrawFunction:
   Draws a single item in the selected list
   Parms: itemNum = 0 based index of item to draw
          bound = ptr to rectangle providing drawing bounds
          pUnused = ptr to char string for item to draw. In this app, we
          never give the actual item data to the list, so we ignore this parm
   Returns: none
*/

void 
SelectedListDrawFunction (Int16 itemNum, RectanglePtr bounds, char **pUnused)
{
   FormPtr pForm = FrmGetActiveForm ();
   char * pText = NULL;

   // get the text to be drawn by looking it up in our selected list
   pText = s_Selected[itemNum].szItem;

   if ( pText )
   {
      // draw it
      WinDrawChars (pText, StrLen (pText), bounds->topLeft.x, bounds->topLeft.y);
   }   
   return;
}

/*
   MainGetAisleSelection:
   Gets a ptr to the text representing the currently selected aisle
   Parms: pForm
   Returns: ptr to selected item (can be null)
*/

char *
MainGetAisleSelection (FormPtr pForm)
{
   UInt16 wID = FrmGetObjectIndex (pForm, MainAisleList);

   ListPtr pList = (ListPtr) FrmGetObjectPtr (pForm, wID);

   UInt16 wSelect = LstGetSelection (pList);

   char * pszSelect = LstGetSelectionText (pList, wSelect);

   return pszSelect;
}

/*
   MainGetMainSelection:
   Gets a ptr to the text representing the currently selected main list
   shopping item
   Parms: pForm
   Returns: ptr to selected item (can be null)
*/
char *
MainGetMainSelection (FormPtr pForm)
{
   char * pText = NULL;

   // get the selection index from the main list
   UInt16 wID = FrmGetObjectIndex (pForm, MainMainList);
   ListPtr pList = (ListPtr) FrmGetObjectPtr (pForm, wID);
   UInt16 wSelect = LstGetSelection (pList);

   // get the text at that index in the currently selected aisle
   pText = MainGetMainText (pForm, wSelect);

   return pText;
}

/*
   MainGetMainText:
   Gets a ptr to the text representing an item at index wSelect
   in the main list
   Parms: pForm, wSelect
   Returns: ptr to item (can be null)
*/

char *
MainGetMainText (FormPtr pForm, Int16 wSelect)
{
   char * pText = NULL;

   // get the selected aisle
   char * pAisle = MainGetAisleSelection (pForm);

   // get the selected text based upon the aisle and index
   if (0 == StrCompare (pAisle, "Fruits and Vegetables"))
   {
      pText = FruitsAndVegetables[wSelect].szItem;
   }
   else if (0 == StrCompare (pAisle, "Dairy"))
   {
      pText = Dairy[wSelect].szItem;
   }
   else if (0 == StrCompare (pAisle, "Meats"))
   {
      pText = Meats[wSelect].szItem;
   }
   return pText;
}

/*
   MainFormMenuHandler:
   Handle a command sent to the main form.
   Parms:   pForm    - form handling event.
            command  - command to be handled.
   Return:  true  - handled (successfully or not)
            false - not handled
*/
Boolean
MainFormMenuHandler (FormPtr /*pForm*/, EventPtr eventP)
{
   Boolean handled = false;
   
   switch (eventP->data.menu.itemID)
   {
      default:
      {
         handled = false;
         break;
      }
   }
   return handled;
}

/*
   MainFormButtonHandler:
   Handle a command sent to the main form.
   Parms:   pForm    - form handling event.
            eventP   - event to be handled.
   Return:  true  - handled (successfully or not)
            false - not handled
*/
Boolean
MainFormButtonHandler (FormPtr pForm, EventPtr eventP)
{
   Boolean handled = false;
   
   switch (eventP->data.ctlEnter.controlID)
   {
      case MainAislePopTrigger:
      {
         // the aisle might have changed - refill the list
         MainFillMainList (pForm);

         break;
      }

      case MainAddButton:   
      {
         // get a ptr to the main list
         UInt16 wIDMainList = FrmGetObjectIndex (pForm, MainMainList );
         ListPtr pCtlMainList = (ListPtr) FrmGetObjectPtr (pForm, wIDMainList);

         // get the text from the main list
         char * pszSelect = MainGetMainSelection (pForm);

         if (pszSelect)
         {
            // add the item to the selected list
            StrCopy ( s_Selected[s_iNumSelected].szItem, pszSelect);

            // increment the number of selections
            s_iNumSelected++;
         }

         // refill selected list
         MainFillSelectedList (pForm);

         handled = true;
         break;
      }

   }
   return handled;
}

