/*
   TBLS_DB.CPP

   Records application db functions

   Copyright (c) 1999 Bachmann Software and Services, LLC
   Author: Glenn Bachmann
*/

// system headers
#include <PalmOS.h>
#include <SysEvtMgr.h>

// application-specific headers
#include "tables.h"
#include "tbls_res.h"

#define TABLES_DB_TYPE  ('Data')
#define TABLES_DB_NAME  ("FishDB")


typedef struct 
{
   char szName[FISH_NAME_LEN + 1];
   char szDesc[FISH_DESC_LEN + 1];
} FishRecord;

static DmOpenRef		s_dbFish;
static MemHandle        s_hRec;
static FishRecord *  s_pFish;
static UInt16			   s_recordNum = 0;

/*
   FishGetCount:
   Returns the count of records in the fish database
   Parms: none
   Returns: count
   
*/
UInt16 
FishGetCount ( void )
{
   UInt16 uFishCount = 0;
   if ( s_dbFish )
   {
      uFishCount = DmNumRecords ( s_dbFish );
   }
   return uFishCount;
}

/*
   FishNew:
   Creates a new Fish record
   Parms: none
   Returns: true if record created
*/
Boolean 
FishNew ( void )
{
	MemHandle hFish;
	Err		err = 0;
	Char		zero = 0;
   UInt16     uIndex = 0;

	// Create a new record in the Fish database
	hFish = DmNewRecord ( s_dbFish, &uIndex, sizeof ( FishRecord ) );

   if ( hFish )
   {
   	char *		p;

	   // get a pointer to the mem block associated with the new record
	   p = (char *)MemHandleLock (hFish);

	   // Init the record by writing a zero
	   err = DmWrite ( p, 0, &zero, sizeof ( FishRecord ) );
	
	   // Unlock the block of the new record.
	   MemPtrUnlock ( p );

	   // Release the record to the database manager.  

	   DmReleaseRecord ( s_dbFish, uIndex, true);

	   // Remember the index of the current record.
	   s_recordNum = 0;

      return true;
   }
   else 
   {
      // handle unexpected err
		ErrFatalDisplayIf ( err, "Could not create new record.");

      return false;
   }
}

/*
   FishGetFish:
   Gets a handle to a Fish record at index uIndex
   Parms: uIndex = 0 based index of record in db
   Returns: NULL or a valid record handle
*/
MemHandle
FishGetFish ( UInt16 uIndex )
{
	Err			err;

   s_recordNum = 0;

   if ( s_dbFish )
   {
      err = DmSeekRecordInCategory ( s_dbFish, 
                                    &s_recordNum,    // start at 0
                                    uIndex,
                                    dmSeekForward,   // seek forward
                                    dmAllCategories);   // all categories

      // get the record
	   s_hRec = (MemHandle)DmGetRecord ( s_dbFish, s_recordNum );
      if ( s_hRec )
      {
         // lock it down and get a ptr
   	   s_pFish = (FishRecord *)MemHandleLock ( s_hRec );
      }
   }

   return s_hRec;
}

/*
   FishReleaseFish:
   Releases the currently edited fish back to the database
   Parms: none
   Returns: none
*/
void
FishReleaseFish ( void )
{
   // unlock the record
   if ( s_hRec )
   {
	   MemHandleUnlock ( s_hRec );
   }

	// Release the record, not dirty.
	DmReleaseRecord ( s_dbFish, s_recordNum, false);

   // reset the current record number
   s_recordNum = 0;

   return;
}


/*
   FishGetName:
   Gets the ptr to the fish name field. 
   Parms: none
   Returns: ptr to name
*/

char *
FishGetName ( void )
{
   return s_pFish->szName;
}

/*
   FishGetDesc:
   Gets the ptr to the fish desc field. 
   Parms: none
   Returns: ptr to name
*/

char *
FishGetDesc ( void )
{
   return s_pFish->szDesc;
}

/*
   FishSetName:
   Sets the name of the currently edited fish record
   Parms: pName = ptr to new name
   Returns: none
*/
void
FishSetName ( char * pName )
{
	// Lock down the block containing the record
   if ( s_hRec )
   {
	   char *		   p;

	   p = (char *)MemHandleLock ( s_hRec );

      // write the name value. Add one to strlen to make sure its null termed
	   Err err = DmWrite ( p, 0, pName, StrLen ( pName ) + 1 );

   	MemPtrUnlock ( p );
   }

   return;
}

/*
   FishSetDesc:
   Sets the desc of the currently edited fish record
   Parms: pDesc = ptr to new desc
   Returns: none
*/
void
FishSetDesc ( char * pDesc )
{
	// Lock down the block containing the record
   if ( s_hRec )
   {
	   char *		   p;

	   p = (char *)MemHandleLock ( s_hRec );

      UInt16 offset = FISH_NAME_LEN + 1;

      // write the name value. Add one to strlen to make sure its null termed
	   Err err = DmWrite ( p, offset, pDesc, StrLen ( pDesc ) + 1 );

   	MemPtrUnlock ( p );
   }

   return;
}

/*
   FishOpen:
   Opens the Fish database, retaining a ref number to the db for further
   operations
   Parms: none
   Returns: none
*/

void
FishOpen ( void )
{   
   // open the db
	s_dbFish = DmOpenDatabaseByTypeCreator ( TABLES_DB_TYPE, 
                                          TABLES_FILE_CREATOR, 
                                          dmModeReadWrite);

	if ( 0 == s_dbFish )
	{
      // not there - create it now
		Err err = DmCreateDatabase ( 0,
                               TABLES_DB_NAME, 
                               TABLES_FILE_CREATOR, 
                               TABLES_DB_TYPE, 
                               false);

      // handle unexpected err
		ErrFatalDisplayIf ( err, "Could not create new database.");
		
		// lets try to open it again...
   	s_dbFish = DmOpenDatabaseByTypeCreator ( TABLES_DB_TYPE, 
                                             TABLES_FILE_CREATOR, 
                                             dmModeReadWrite);

      // handle unexpected err
      if ( 0 == s_dbFish )
      {
		   ErrFatalDisplayIf ( true, "Could not open new database.");
      }
   }

   return;
}

/*
   FishClose:
   Closes the fish db
   Parms: none
   Returns: none
*/

void
FishClose ( void )
{
   if ( s_dbFish )
   {
   	DmCloseDatabase ( s_dbFish );
      s_dbFish = 0;
   }
   return;
}

UInt16
FishGetNextRecordNum ( UInt16 uRecordNumStart, Int16 wRowsToSkip )
{
   UInt16 uRecordNumNext = uRecordNumStart;
   
   if ( s_dbFish )
   {
      Err err = DmSeekRecordInCategory ( s_dbFish, 
                                        &uRecordNumNext, 
                                        wRowsToSkip,
                                        dmSeekForward, 
                                        dmAllCategories);
   }
   return uRecordNumNext;
}      

UInt16
FishGetPrevRecordNum ( UInt16 uRecordNumStart, Int16 wRowsToSkip )
{
   UInt16 uRecordNumPrev = uRecordNumStart;
   
   if ( s_dbFish )
   {
      DmSeekRecordInCategory ( s_dbFish, 
                              &uRecordNumPrev, 
                              wRowsToSkip,
                              dmSeekBackward, 
                              dmAllCategories);
   }
   return uRecordNumPrev;
}      

MemHandle
FishQueryFish ( UInt16 uRecordNum )
{
   UInt16 uRecordNumTemp = uRecordNum;

   MemHandle hRecord = DmQueryNextInCategory ( s_dbFish, 
                                                &uRecordNumTemp, 
                                                dmAllCategories );
   return hRecord;
}
