/*
   TBLS_MAI.CPP
   Main form handling functions.
   Copyright (c) Bachmann Software and Services, 1999-2002
   Author: Glenn Bachmann
*/

// system headers
#include <PalmOS.h>
#include <SysEvtMgr.h>

// application-specific headers
#include "tables.h"
#include "tbls_res.h"

static void    MainFormInit (FormPtr formP);
static Boolean MainFormButtonHandler (FormPtr formP, EventPtr eventP);
static Boolean MainFormMenuHandler (FormPtr formP, EventPtr eventP);

// local helper functions
void MainListFill (FormPtr pForm);
void MainListDrawFunction (void * pTable, Int16 row, Int16 column, RectanglePtr bounds);
void MainListCalcTopVisible (FormPtr pForm);

Boolean AddFish ( void );
Boolean AddFish_EventHandler (EventPtr pEvent);

// current record in the table 
static Int16 s_wCurrentRecord;
static Int16 s_wTopVisibleRecord;

#define COL_FISH_NAME   0
#define COL_FISH_DESC   1

/*
   MainFormEventHandler:
   Parms:   pEvent   - event to be handled.
   Return:  true  - handled (successfully or not)
            false - not handled
*/
Boolean
MainFormEventHandler (EventPtr pEvent)
{
   Boolean  handled = false;
   
   switch (pEvent->eType)
   {
      case menuEvent:
      {
         FormPtr pForm = FrmGetActiveForm ();
         handled = MainFormMenuHandler (pForm, pEvent);
         break;
      }
      

      case ctlSelectEvent:
      {
         // A control button was tapped and released
         FormPtr pForm = FrmGetActiveForm ();
         handled = MainFormButtonHandler (pForm, pEvent);
         break;
      }

      case tblSelectEvent:
      {
         // set the current record based on user selection

         s_wCurrentRecord = TblGetItemInt ((TableType *)pEvent->data.tblSelect.pTable,
                                         pEvent->data.tblSelect.row,
                                         pEvent->data.tblSelect.column );
         break;
      }

      // physical scroll buttons
   	case keyDownEvent:		
      {
	   	if (pEvent->data.keyDown.chr == pageUpChr)
		  	{
            FormPtr pForm = FrmGetActiveForm ();
			   s_wTopVisibleRecord = FishGetPrevRecordNum ( s_wTopVisibleRecord, 1 );
            MainListFill ( pForm );
            FrmDrawForm ( pForm );
   	    	handled = true;
		  	}
	   	else if (pEvent->data.keyDown.chr == pageDownChr)
	    	{
            FormPtr pForm = FrmGetActiveForm ();
			   s_wTopVisibleRecord = FishGetNextRecordNum ( s_wTopVisibleRecord, 1 );
            MainListFill ( pForm );
            FrmDrawForm ( pForm );
   	    	handled = true;
         }
         break;
      }
      
      case frmOpenEvent:
      {
         // main form is opening

         FormPtr pForm = FrmGetActiveForm ();

         MainFormInit (pForm);

         FrmDrawForm (pForm);

         handled = true;
         break;
      }
      
      default:
      {
         break;
      }
   }
   return handled;
}

/*
   MainFormInit:
   Initialize the main form.
   Parms:   pForm - pointer to main form.
   Return:  none
*/
void
MainFormInit (FormPtr pForm)
{
   // set the controls to an initial state
   s_wCurrentRecord = -1;

   // walk through fish records and fill list
   MainListFill ( pForm );
}

/*
   MainFormMenuHandler:
   Handle a command sent to the main form.
   Parms:   pForm    - form handling event.
            command  - command to be handled.
   Return:  true  - handled (successfully or not)
            false - not handled
*/
Boolean
MainFormMenuHandler (FormPtr /*pForm*/, EventPtr eventP)
{
   Boolean handled = false;
   
   switch (eventP->data.menu.itemID)
   {
      default:
      {
         handled = false;
         break;
      }
   }
   return handled;
}

/*
   MainFormButtonHandler:
   Handle a command sent to the main form.
   Parms:   pForm    - form handling event.
            eventP   - event to be handled.
   Return:  true  - handled (successfully or not)
            false - not handled
*/

Boolean
MainFormButtonHandler (FormPtr pForm, EventPtr eventP)
{
   Boolean handled = false;
   
   switch (eventP->data.ctlEnter.controlID)
   {
      case MainNewButton:
      {
         if ( AddFish () )
         {
            MainListFill ( pForm );
            FrmDrawForm ( pForm );
         }

         handled = true;
         break;
      }

	   case MainScrollUpRepeating:
      {
			s_wTopVisibleRecord = FishGetPrevRecordNum ( s_wTopVisibleRecord, 1 );
         MainListFill ( pForm );
         FrmDrawForm ( pForm );
         break;
	   }
	   case MainScrollDownRepeating:
      {
			s_wTopVisibleRecord = FishGetNextRecordNum ( s_wTopVisibleRecord, 1 );
         MainListFill ( pForm );
         FrmDrawForm ( pForm );
         break;
	   }


      default: 
      {
         handled = false;
         break;
      }
   }
   return handled;
}

/*
   MainListFill:
   Fills the fish list 
   Parms: pForm - pointer to main form
   Returns: none
*/
void
MainListFill (FormPtr pForm)
{
   // get a ptr to the fish table
   UInt16 wIDFishTable = FrmGetObjectIndex (pForm, MainFishTable );
   TablePtr pTable = (TablePtr) FrmGetObjectPtr (pForm, wIDFishTable);

   // init the table

   // get the number of rows defined in Constructor for the table
   // this the number that will be visible
   UInt16 wRows = TblGetNumberOfRows (pTable);
   UInt16 i;

   for (i = 0; i < wRows; i++ )
   {
      // set the cells to have custom drawing
      TblSetItemStyle  (pTable, i, COL_FISH_NAME, customTableItem );
      TblSetItemStyle  (pTable, i, COL_FISH_DESC, customTableItem );

      // set the row to be unusable to start
      TblSetRowUsable (pTable, i, false);
   }

   // set the columns to be usable
   TblSetColumnUsable  (pTable, COL_FISH_NAME, true );
   TblSetColumnUsable  (pTable, COL_FISH_DESC, true );

   // set our draw procedure callback for each column
   TblSetCustomDrawProcedure (pTable, COL_FISH_NAME, MainListDrawFunction );
   TblSetCustomDrawProcedure (pTable, COL_FISH_DESC, MainListDrawFunction );

   // reset the top visible row based on size of table and current record
   MainListCalcTopVisible (pForm);

   UInt16 uRecordNum = 0;

   // load records into the table, starting at top visible
   // get the record number of the top visible record
   uRecordNum = s_wTopVisibleRecord;

   for (i = 0; i < wRows; i++, uRecordNum++)
   {
      UInt16 uNextRecordNum = 0;

      if (FishQueryFish ( uRecordNum ))
      {
         // remember the associated record number for this row
         TblSetItemInt (pTable, i, COL_FISH_NAME, uRecordNum);
         TblSetItemInt (pTable, i, COL_FISH_DESC, uRecordNum);
         TblSetRowUsable (pTable, i, true);
      }
      else
      {
         // if we are out of records, mark row as unusable
         TblSetRowUsable (pTable, i, false);
      }
      // force a row redraw
      TblMarkRowInvalid (pTable, i );
   }

   return;
}

/*
   MainListCalcTopVisible:
   Sets the top visible item in the table
   Parms: none
   Returns: none
*/
void
MainListCalcTopVisible (FormPtr pForm)
{
   // get a ptr to the fish table
   UInt16 wIDFishTable = FrmGetObjectIndex (pForm, MainFishTable );
   TablePtr pFishTable = (TablePtr) FrmGetObjectPtr (pForm, wIDFishTable);

   // get the number of rows defined in Constructor for the table
   // this the number that will be visible
   UInt16 wRows = TblGetNumberOfRows (pFishTable);

   // this all sets up what the top visible record
   // should be
   UInt16 uRecordNum = 0;

   if ( s_wCurrentRecord != -1)
   {
      if ( s_wTopVisibleRecord > s_wCurrentRecord )
      {
         // the current record is before the first visible record
         // "scroll up"
         s_wTopVisibleRecord = s_wCurrentRecord;
      }
      else 
      {
         // this tries to seek from the top visible record to the
         // last visible record.
         uRecordNum = FishGetNextRecordNum ( s_wTopVisibleRecord, 
                                             wRows - 1 );

         if ( uRecordNum < s_wCurrentRecord)
         {
            // the last visible record in the table is less than the 
            // current record (so the current record is after the visible
            // portion of the table). We need to reset the top 
            // visible record such that it is the current record.
            // this is a "scroll down"
            s_wTopVisibleRecord = s_wCurrentRecord;
         }
      }
   }

   // now adjust to make sure that we have a full number of records even
   // if its the last screen. If there's less than a screenful left,
   // we have to push TopVisible backwards until there is.
   uRecordNum = FishGetPrevRecordNum ( dmMaxRecordIndex, wRows - 1);

   s_wTopVisibleRecord = (s_wTopVisibleRecord < uRecordNum) ? s_wTopVisibleRecord : uRecordNum;
   return;
}

/*
   MainListDrawFunction:
   Draws a single cell in the main table
   Parms: pTable = table ptr
          row = row of item
          column = cell
          bounds = bounding rectangle for cell
   Returns: none
*/
void 
MainListDrawFunction (void * pTable, Int16 row, Int16 column, RectanglePtr bounds)
{
   FormPtr pForm = FrmGetActiveForm ();

	char *		pText;
   MemHandle      hFish = NULL;
   UInt16 recordNum = 0;

   // get the record num we are drawing
   recordNum = TblGetItemInt ((TableType *)pTable,
                              row,
                              column );
	
   // find the record at this index
   FishGetFish ( recordNum );

   if ( COL_FISH_NAME == column)
   {
      // get the fish name
      pText = FishGetName ();
   }
   else
   {
      // get the fish desc
      pText = FishGetDesc ();
   }

   // draw it
	if ( pText )
   {
   	WinDrawChars ( pText, StrLen (pText), bounds->topLeft.x, bounds->topLeft.y);
   }
	
   // release it
   FishReleaseFish ();
   return;
}


/*
   AddFish:
   Invokes the Add Fish popup form
   Parms:   none
   Return:  none
*/
Boolean AddFish ( void )
{
   // initialize the form
   FormPtr  pForm = FrmInitForm (AddFishForm);

   // create a new fish record
	if ( FishNew () )
   {
      // edit it!
      MemHandle hRecord = FishGetFish ( 0 );

      // create a mem buffer for the new fish name
      MemHandle hName = MemHandleNew ( FISH_NAME_LEN + 1 );
      char * pName = (char *)MemHandleLock (hName);
      // default the name of the fish
      StrCopy ( pName, "<new fish>" );
   	MemPtrUnlock ( pName );

      // create a mem buffer for the new fish description
      MemHandle hDesc = MemHandleNew ( FISH_DESC_LEN + 1 );
      char * pDesc = (char *)MemHandleLock (hDesc);
      // default the desc of the fish
      StrCopy ( pDesc, "" );
   	MemPtrUnlock ( pDesc );

      UInt16 wIDName = FrmGetObjectIndex ( pForm, AddFishNameField );
      FieldPtr pNameField = (FieldPtr) FrmGetObjectPtr ( pForm, wIDName);
      // copy the text into the edit field
		FldSetTextHandle ( pNameField, (MemHandle)hName);

      UInt16 wIDDesc = FrmGetObjectIndex ( pForm, AddFishDescField );
      FieldPtr pDescField = (FieldPtr) FrmGetObjectPtr ( pForm, wIDDesc);
      // copy the text into the edit field
		FldSetTextHandle ( pDescField, (MemHandle)hDesc);

      // display the dialog
      if ( FrmDoDialog (pForm) )
      {
         // get the name from the dialog
         hName = FldGetTextHandle (pNameField);
         char * pName = (char *)MemHandleLock (hName);
         hDesc = FldGetTextHandle (pDescField);
         char * pDesc = (char *)MemHandleLock (hDesc);

         // set it in the record
         FishSetName ( pName );
         FishSetDesc ( pDesc );

      	MemPtrUnlock ( pName);
      	MemPtrUnlock ( pDesc);
      }

      // un-edit the fish record, saving changes
      FishReleaseFish ();

      // destroy the form
      FrmDeleteForm (pForm);

      return true;
   }
   return false;
}
