/*
   DRAW_MAI.CPP
   Main form handling functions.
   Copyright (c) Bachmann Software and Services, 1999-2002
   Author: Glenn Bachmann
*/

// system headers
#include <PalmOS.h>
#include <SysEvtMgr.h>

// application-specific headers
#include "draw.h"
#include "draw_res.h"

static void    MainFormInit (FormPtr formP);
static Boolean MainFormButtonHandler (FormPtr formP, EventPtr eventP);
static Boolean MainFormMenuHandler (FormPtr formP, EventPtr eventP);

static void  DrawLines ( void );
static void  DrawRectangles ( void );
static void  DrawBitmap ( void );
static void  DrawText ( void );

/*
   MainFormEventHandler:
   Parms:   pEvent   - event to be handled.
   Return:  true  - handled (successfully or not)
            false - not handled
*/
Boolean
MainFormEventHandler (EventPtr eventP)
{
   Boolean  handled = false;
   
   switch (eventP->eType)
   {
      case menuEvent:
      {
         FormPtr formP = FrmGetActiveForm ();
         handled = MainFormMenuHandler (formP, eventP);
         break;
      }
      
      case ctlSelectEvent:
      {
         // A control button was tapped and released
         FormPtr formP = FrmGetActiveForm ();
         handled = MainFormButtonHandler (formP, eventP);
         break;
      }
      
      case frmOpenEvent:
      {
         // main form is opening

         FormPtr formP = FrmGetActiveForm ();

         MainFormInit (formP);

         FrmDrawForm (formP);

         handled = true;
         break;
      }
      
      default:
      {
         break;
      }
   }
   return handled;
}

/*
   MainFormInit:
   Initialize the main form.
   Parms:   formP - pointer to main form.
   Return:  none
*/
void
MainFormInit (FormPtr formP)
{
   // set the controls to an initial state
   // create the windows: two on top, one below
   RectangleType rect;
   Err err;
   WinHandle whUpperLeft, whUpperRight, whBottomLeft, whBottomRight, whOrig;
   Int16 iWidth = 160;
   Int16 iHeight = 160;

   WinGetDisplayExtent ( &iWidth, &iHeight );

   // Upper left
   rect.topLeft.x = 1;
   rect.extent.x = ( iWidth / 2 ) - 1;
   rect.topLeft.y = 20;
   rect.extent.y = 60;  
   whUpperLeft = WinCreateWindow ( &rect, simpleFrame, false, false, &err );

   whOrig = WinSetDrawWindow ( whUpperLeft );
   WinDrawWindowFrame ( );
   WinSetDrawWindow ( whOrig );

   // Upper right
   rect.topLeft.x = iWidth / 2;
   rect.extent.x = ( iWidth / 2 ) - 1;
   rect.topLeft.y = 20;
   rect.extent.y = 60;  
   whUpperRight = WinCreateWindow ( &rect, simpleFrame, false, false, &err );
   WinSetDrawWindow ( whUpperRight );
   WinDrawWindowFrame ( );
   WinSetDrawWindow ( whOrig );

   // bottom left
   rect.topLeft.x = 1;
   rect.extent.x = ( iWidth / 2 ) - 1;
   rect.topLeft.y = 80; 
   rect.extent.y = 60;  
   whBottomLeft = WinCreateWindow ( &rect, simpleFrame, false, false, &err );
   WinSetDrawWindow ( whBottomLeft );
   WinDrawWindowFrame ( );
   WinSetDrawWindow ( whOrig );

   // bottom right
   rect.topLeft.x = iWidth / 2;
   rect.extent.x = ( iWidth / 2 ) - 1;
   rect.topLeft.y = 80; 
   rect.extent.y = 60;  
   whBottomRight = WinCreateWindow ( &rect, simpleFrame, false, false, &err );
   WinSetDrawWindow ( whBottomRight );
   WinDrawWindowFrame ( );
   WinSetDrawWindow ( whOrig );

   WinSetDrawWindow ( whUpperLeft );
   DrawLines ();

   WinSetDrawWindow ( whUpperRight );
   DrawRectangles ();

   WinSetDrawWindow ( whBottomLeft );
   DrawText ();

   WinSetDrawWindow ( whBottomRight );
   DrawBitmap ();

   WinSetDrawWindow ( whOrig );
}

/*
   DrawLines:
   Draws example lines
   Parms: none
   Returns: none
*/
void 
DrawLines ( void )
{
   Int16 wX, wY;
   
   // get the bounds of the current draw window
   WinGetWindowExtent ( &wX, &wY );

   // single solid line
   // draw from one point to another
   WinDrawLine ( 1, 10, wX -2, 10 );

   // double solid line
   WinDrawLine ( 1, 20, wX -2, 20 );
   WinDrawLine ( 1, 21, wX -2, 21 );

   // single gray line
   WinDrawGrayLine ( 1, 30, wX -2, 30 );

   // patterned lines
   CustomPatternType patternOld;
   CustomPatternType patternDotted = { 0xAAAA, 0xAAAA, 0xAAAA, 0xAAAA };
   CustomPatternType patternDashed = { 0xCCCC, 0xCCCC, 0xCCCC, 0xCCCC };

   WinGetPattern ( &patternOld );

   // single dotted line
   WinSetPattern ( &patternDotted );
   WinFillLine ( 1, 40, wX -2, 40 );
   WinSetPattern ( &patternOld );

   // single dashed line
   WinSetPattern ( &patternDashed );
   WinFillLine ( 1, 50, wX -2, 50 );
   WinSetPattern ( &patternOld );

   return;
}

/*
   DrawRectangles:
   Draws example rectangles
   Parms: none
   Returns: none
*/
void 
DrawRectangles ( void )
{
   Int16 wX, wY;
   RectangleType rect;
   
   // get the bounds of the current draw window
   WinGetWindowExtent ( &wX, &wY );

   // Black Rectangle
   rect.topLeft.x = 1;
   rect.extent.x = wX - 2;

   rect.topLeft.y = 10;
   rect.extent.y = 20;
   WinDrawRectangle ( &rect, 0 );

   // Filled pattern rectangle
   CustomPatternType patternOld;
   CustomPatternType patternDotted = { 0xAA55, 0xAA55, 0xAA55, 0xAA55 };

   rect.topLeft.y = 35;
   rect.extent.y = 20;
   WinGetPattern ( &patternOld );
   WinSetPattern ( &patternDotted );
   WinFillRectangle ( &rect, 0 );
   WinSetPattern ( &patternOld );

   return;
}

/*
   DrawBitmap:
   Draws example bitmaps
   Parms: none
   Returns: none
*/
void 
DrawBitmap ( void )
{
   MemHandle hBitmap;

   hBitmap = DmGetResource ( bitmapRsc, HappyBitmap );

   if ( hBitmap )
   {
      BitmapPtr pBitmap;
      Int16 wX, wY;
   
      // get the bounds of the current draw window
      WinGetWindowExtent ( &wX, &wY );

      pBitmap = (BitmapPtr)MemHandleLock ( hBitmap );
      WinDrawBitmap ( pBitmap, wX / 2 - 8, 22 );
      MemHandleUnlock ( hBitmap );

      DmReleaseResource ( hBitmap );
   }

   return;
}

/*
   DrawText:
   Draws example text
   Parms: none
   Returns: none
*/
void 
DrawText ( void )
{
   WinDrawChars ( "Hello Palm!", StrLen ( "Hello Palm!" ), 10, 30 );

   return;
}

/*
   MainFormMenuHandler:
   Handle a command sent to the main form.
   Parms:   formP    - form handling event.
            command  - command to be handled.
   Return:  true  - handled (successfully or not)
            false - not handled
*/
Boolean
MainFormMenuHandler (FormPtr /*formP*/, EventPtr eventP)
{
   Boolean handled = false;
   
   switch (eventP->data.menu.itemID)
   {
      default:
      {
         handled = false;
         break;
      }
   }
   return handled;
}

/*
   MainFormButtonHandler:
   Handle a command sent to the main form.
   Parms:   formP    - form handling event.
            eventP   - event to be handled.
   Return:  true  - handled (successfully or not)
            false - not handled
*/
Boolean
MainFormButtonHandler (FormPtr formP, EventPtr eventP)
{
   Boolean handled = false;
   
   switch (eventP->data.ctlEnter.controlID)
   {
      default: 
      {
         handled = false;
         break;
      }
   }
   return handled;
}

