/*
   RECO_MAI.CPP
   Main form handling functions.
   Copyright (c) Bachmann Software and Services, 1999-2002
   Author: Glenn Bachmann
*/

// system headers
#include <Palmos.h>
#include <SysEvtMgr.h>

// application-specific headers
#include "record.h"
#include "reco_res.h"

static void    MainFormInit (FormPtr formP);
static Boolean MainFormButtonHandler (FormPtr formP, EventPtr eventP);
static Boolean MainFormMenuHandler (FormPtr formP, EventPtr eventP);

// local helper functions
void MainListFill (FormPtr pForm);
void MainListDrawFunction (Int16 itemNum, RectanglePtr bounds, char **pUnused);

Boolean AddFish ( void );
Boolean AddFish_EventHandler (EventPtr pEvent);

/*
   MainFormEventHandler:
   Parms:   pEvent   - event to be handled.
   Return:  true  - handled (successfully or not)
            false - not handled
*/
Boolean
MainFormEventHandler (EventPtr eventP)
{
   Boolean  handled = false;
   
   switch (eventP->eType)
   {
      case menuEvent:
      {
         FormPtr formP = FrmGetActiveForm ();
         handled = MainFormMenuHandler (formP, eventP);
         break;
      }
      

      case ctlSelectEvent:
      {
         // A control button was tapped and released
         FormPtr formP = FrmGetActiveForm ();
         handled = MainFormButtonHandler (formP, eventP);
         break;
      }
      
      case frmOpenEvent:
      {
         // main form is opening

         FormPtr formP = FrmGetActiveForm ();

         MainFormInit (formP);

         FrmDrawForm (formP);

         handled = true;
         break;
      }
      
      default:
      {
         break;
      }
   }
   return handled;
}

/*
   MainFormInit:
   Initialize the main form.
   Parms:   formP - pointer to main form.
   Return:  none
*/
void
MainFormInit (FormPtr pForm)
{
   // set the controls to an initial state

   // walk through fish records and fill list
   MainListFill ( pForm );

   // set the drawing function for the main list
   UInt16 wIDMainList = FrmGetObjectIndex (pForm, MainFishList );
   ListPtr pCtlMainList = (ListPtr) FrmGetObjectPtr (pForm, wIDMainList);
   LstSetDrawFunction ( pCtlMainList, MainListDrawFunction );
}

/*
   MainFormMenuHandler:
   Handle a command sent to the main form.
   Parms:   formP    - form handling event.
            command  - command to be handled.
   Return:  true  - handled (successfully or not)
            false - not handled
*/
Boolean
MainFormMenuHandler (FormPtr /*formP*/, EventPtr eventP)
{
   Boolean handled = false;
   
   switch (eventP->data.menu.itemID)
   {
      default:
      {
         handled = false;
         break;
      }
   }
   return handled;
}

/*
   MainFormButtonHandler:
   Handle a command sent to the main form.
   Parms:   formP    - form handling event.
            eventP   - event to be handled.
   Return:  true  - handled (successfully or not)
            false - not handled
*/

Boolean
MainFormButtonHandler (FormPtr pForm, EventPtr eventP)
{
   Boolean handled = false;
   
   switch (eventP->data.ctlEnter.controlID)
   {
      case MainNewButton:
      {
         if ( AddFish () )
         {
            MainListFill ( pForm );
         }

         handled = true;
         break;
      }

      default: 
      {
         handled = false;
         break;
      }
   }
   return handled;
}

/*
   MainListFill:
   Fills the fish list 
   Parms: pForm - pointer to main form
   Returns: none
*/
void
MainListFill (FormPtr pForm)
{
   // get a ptr to the fish list
   UInt16 wIDFishList = FrmGetObjectIndex (pForm, MainFishList );
   ListPtr pCtlFishList = (ListPtr) FrmGetObjectPtr (pForm, wIDFishList);

   // get the count of available main list items 
   Int16 iFishCount = FishGetCount ();

   // fill the main list 
   LstSetListChoices ( pCtlFishList, NULL, iFishCount );

   // redraw the list
   LstDrawList (pCtlFishList);

   return;
}

/*
   MainListDrawFunction:
   Draws a single item in the main list
   Parms: itemNum = 0 based index of item to draw
          bound = ptr to rectangle providing drawing bounds
          pUnused = ptr to char string for item to draw. In this app, we
          never give the actual item data to the list, so we ignore this parm
   Returns: none
*/
void 
MainListDrawFunction (Int16 itemNum, RectanglePtr bounds, char **pUnused)
{
   FormPtr pForm = FrmGetActiveForm ();

	char *		pText;
   MemHandle      hFish = NULL;
	
   // find the record at this index
   FishGetFish ( itemNum );

   // get the fish name
   pText = FishGetName ();

   // draw it
	if ( pText )
   {
   	WinDrawChars ( pText, StrLen (pText), bounds->topLeft.x, bounds->topLeft.y);
   }
	
   // release it
   FishReleaseFish ();
   return;
}


/*
   AddFish:
   Invokes the Add Fish popup form
   Parms:   none
   Return:  none
*/
Boolean AddFish ( void )
{
   // initialize the form
   FormPtr  pForm = FrmInitForm (AddFishForm);

   // create a new fish record
	if ( FishNew () )
   {
      // edit it!
      MemHandle hRecord = FishGetFish ( 0 );

      // create a mem buffer for the new fish name
      MemHandle hText = MemHandleNew ( FISH_LENGTH );
      char * pText = (char *)MemHandleLock (hText);

      // default the name of the fish
      StrCopy ( pText, "<new fish>" );
   	MemPtrUnlock ( pText );


      UInt16 wIDField = FrmGetObjectIndex ( pForm, AddFishNameField );
      FieldPtr pField = (FieldPtr) FrmGetObjectPtr ( pForm, wIDField);

      // copy the text into the edit field
		FldSetTextHandle ( pField, (MemHandle)hText);

      // display the dialog
      if ( FrmDoDialog (pForm) )
      {
         // get the name from the dialog
         hText = FldGetTextHandle (pField);
         char * pText = (char *)MemHandleLock (hText);

         // set it in the record
         FishSetName ( pText );

      	MemPtrUnlock ( pText );
      }

      // un-edit the fish record, saving changes
      FishReleaseFish ();

      // destroy the form
      FrmDeleteForm (pForm);

      return true;
   }
   return false;
}
