/*
   CATE_DB.CPP

   Categories application db functions

   Copyright (c) 1999-2002 Bachmann Software and Services, LLC
   Author: Glenn Bachmann
*/

// system headers
#include <PalmOS.h>
#include <SysEvtMgr.h>

// application-specific headers
#include "category.h"
#include "cate_res.h"

#define CATEGORY_DB_TYPE  ('Data')
#define CATEGORY_DB_NAME  ("FishDB")

static UInt16			s_wCurrentCategory = dmAllCategories;	
static char				s_szCategoryName[dmCategoryLength];		
static DmOpenRef		s_dbFish;
static MemHandle        s_hRec;
static char *	      s_pText;
static UInt16		   s_recordNum = 0;

static Err     FishInitAppInfo (DmOpenRef pDB);

/*
   FishGetCount:
   Returns the count of records in the fish database
   Parms: none
   Returns: count
   
*/

UInt16 
FishGetCount ( void )
{
   UInt16 uFishCount = 0;
   if ( s_dbFish )
   {
      uFishCount = DmNumRecordsInCategory ( s_dbFish, s_wCurrentCategory );
   }
   return uFishCount;
}

/*
   FishNew:
   Creates a new Fish record
   Parms: none
   Returns: true if record created
*/

Boolean 
FishNew ( void )
{
	void *		p;
	MemHandle hFish;
	UInt16		uIndex = 0;
	Err		err;
	Char		zero = 0;
	UInt16		attr;			

	// Create a new record in the Fish database
	hFish = DmNewRecord ( s_dbFish, &uIndex, FISH_LENGTH);

   if (hFish)
   {
	   // get a pointer to the mem block associated with the new record
	   p = (void *)MemHandleLock (hFish);

	   // Init the record by writing a zero
	   err = DmWrite ( p, 0, &zero, 1 );
	
	   // Unlock the block of the new record.
	   MemPtrUnlock ( p );

      // obtain the record's attribute info, and set it to reflect the 
      // currently selected category
	   DmRecordInfo ( s_dbFish, uIndex, &attr, NULL, NULL);

	   attr &= ~dmRecAttrCategoryMask;				// Remove all category bits

	   if ( s_wCurrentCategory == dmAllCategories)
	   {
		   attr |= dmUnfiledCategory;
	   }
	   else
	   {
		   attr |= s_wCurrentCategory;
	   }

	   DmSetRecordInfo ( s_dbFish, uIndex, &attr, NULL);


	   // Release the record to the database manager.  
	   DmReleaseRecord ( s_dbFish, uIndex, true);

	   // Remember the index of the current record.
	   s_recordNum = 0;
      return true;
   }
   else
   {
      return false;
   }

}

/*
   FishGetFish:
   Gets a handle to a Fish record at index uIndex
   Parms: uIndex = 0 based index of record in db
   Returns: NULL or a valid record handle
*/

MemHandle
FishGetFish ( UInt16 uIndex )
{
	Err			err;

   s_recordNum = 0;

   if ( s_dbFish )
   {
      err = DmSeekRecordInCategory ( s_dbFish, 
                                    &s_recordNum, 
                                    uIndex,
                                    dmSeekForward, 
                                    s_wCurrentCategory);

      // get the record
	   s_hRec = (MemHandle)DmGetRecord ( s_dbFish, s_recordNum );
      if ( s_hRec )
      {
         // lock it down and get a ptr
   	   s_pText = (Char *)MemHandleLock ( s_hRec );
      }

   }

   return s_hRec;
}

/*
   FishReleaseFish:
   Releases the currently edited fish back to the database
   Parms: none
   Returns: none
*/

void
FishReleaseFish ( void )
{
   if ( s_hRec )
   {
	   MemHandleUnlock ( s_hRec );
   }

	// Release the record, not dirty.
	DmReleaseRecord ( s_dbFish, s_recordNum, false);

   s_recordNum = 0;

   return;
}

/*
   FishUpdate:
   Commits changes to the category information back to the record
   Parms: none
   Returns: none
*/

void
FishUpdate ( void )
{
	UInt16		attr;	

	attr &= ~dmRecAttrCategoryMask;				

	if ( s_wCurrentCategory == dmAllCategories)
	{
		attr |= dmUnfiledCategory;
	}
	else
	{
		attr |= s_wCurrentCategory;
	}

   // update the attributes
	DmSetRecordInfo ( s_dbFish, s_recordNum, &attr, NULL);
   return;
}

/*
   FishSelectType:
   Performs Category Selection in order to choose the fish type
   If the category is changed, updates the current category
   Parms: pForm = parent form
          pTypeID = out param for new type
          wTriggerID = id of popup trigger control
          wListID = id of listbox
   Returns: true if selection made
*/

Boolean
FishSelectType ( FormPtr pForm, UInt16 *pTypeID, UInt16 wTriggerID, UInt16 wListID )
{
   Boolean bChanged = false;

	UInt16 category;
	Boolean bEdited;

	// Process the category popup list
   // save the current one in a temp
	category = s_wCurrentCategory;


	bEdited = CategorySelect ( s_dbFish, 
                                       pForm, 
                                       wTriggerID,
                              			wListID, 
                                       true, 
                                       &category, 
                                       s_szCategoryName,
                                       1,
                                       0);

	if (bEdited || (category != s_wCurrentCategory))
	{
      // category is new - save it and refill the fish list!
		s_wCurrentCategory = category;

      *pTypeID = category;
      return true;
   }
   return false;
}

/*
   FishGetCurrentType:
   Obtains the name of the currently selected category/fish type
   Parms: pType= buffer to receive name of type
   Returns: none
*/

void
FishGetCurrentType ( char * pType )
{
	CategoryGetName ( s_dbFish, 
                     s_wCurrentCategory, 
                     s_szCategoryName);

   StrCopy ( pType, s_szCategoryName );
   return;
}

/*
   FishGetName:
   Gets the ptr to the fish name field. Since the fish record is simply one
   field, just returns the whole record
   Parms: none
   Returns: ptr to name
*/

CharPtr
FishGetName ( void )
{
   return s_pText;
}

/*
   FishSetName:
   Sets the name of the currently edited fish record
   Parms: pName = ptr to new name
   Returns: none
*/

void
FishSetName ( CharPtr pName )
{
	// Lock down the block containing the record
   if ( s_hRec )
   {
	   Ptr		p;

	   p = (Ptr)MemHandleLock ( s_hRec );

      // write the name value. Add one to strlen to make sure its null termed
	   Err err = DmWrite ( p, 0, pName, StrLen ( pName ) + 1 );

   	MemPtrUnlock ( p );
   }

   return;
}

/*
   FishOpen:
   Opens the Fish database, retaining a ref number to the db for further
   operations
   Parms: none
   Returns: none
*/

void
FishOpen ( void )
{   
   // open the db
	s_dbFish = DmOpenDatabaseByTypeCreator ( CATEGORY_DB_TYPE, 
                                          CATEGORY_FILE_CREATOR, 
                                          dmModeReadWrite);

	if ( 0 == s_dbFish )
	{
	   UInt		cardNo;			
	   LocalID	dbID;				
	   UInt		dbAttrs;			

      // not there - create it now
		Err err = DmCreateDatabase ( 0,
                               CATEGORY_DB_NAME, 
                               CATEGORY_FILE_CREATOR, 
                               CATEGORY_DB_TYPE, 
                               false);

      // handle unexpected err
		ErrFatalDisplayIf ( err, "Could not create new database.");
		
		// lets try to open it again...
   	s_dbFish = DmOpenDatabaseByTypeCreator ( CATEGORY_DB_TYPE, 
                                               CATEGORY_FILE_CREATOR, 
                                             dmModeReadWrite);

		// Store category info in the application's information block.
		FishInitAppInfo(s_dbFish);
   }

	// Get the name of the current category from the app info block.
   if ( s_dbFish )
   {
   	CategoryGetName ( s_dbFish, s_wCurrentCategory, s_szCategoryName);
   }
   return;
}

/*
   FishClose:
   Closes the fish db
   Parms: none
   Returns: none
*/

void
FishClose ( void )
{
   if ( s_dbFish )
   {
   	DmCloseDatabase ( s_dbFish );
   }
   return;
}

/*
   FishInitAppInfo:
   Initializes the App Info portion of the database header, and sets the category
   area
   Parms: pDB = ptr to fish db
   Returns: 0 = success

*/

Err
FishInitAppInfo (DmOpenRef pDB)
{
   UInt cardNo;
   VoidHand handle;
   LocalID dbID;
   LocalID appInfoID;
   FishAppInfo * pAppInfo;
   FishAppInfo * pNull = 0;

   if ( DmOpenDatabaseInfo ( pDB, &dbID, NULL, NULL, &cardNo, NULL ))
   {
      return dmErrInvalidParam;
   }
   if ( DmDatabaseInfo ( cardNo, dbID, 
                         NULL, 
                         NULL, 
                         NULL,
                         NULL,
                         NULL,
                         NULL,
                         NULL,
                         &appInfoID, 
                         NULL,
                         NULL,
                         NULL ))
   {
      return dmErrInvalidParam;
   }

   if ( NULL == appInfoID )
   {
      handle = DmNewHandle ( pDB, sizeof (FishAppInfo) );
      if ( NULL == handle )
      {
         return dmErrMemError;
      }

      appInfoID = MemHandleToLocalID ( handle );
      DmSetDatabaseInfo ( cardNo, dbID, 
                         NULL, 
                         NULL, 
                         NULL,
                         NULL,
                         NULL,
                         NULL,
                         NULL,
                         &appInfoID, 
                         NULL,
                         NULL,
                         NULL );
   
      pAppInfo = (FishAppInfo *)MemLocalIDToLockedPtr ( appInfoID, cardNo );

      DmSet ( pAppInfo, 0, sizeof ( FishAppInfo ), 0 );

      CategoryInitialize ( (AppInfoPtr)pAppInfo, FishTypesAppInfoStr );

      MemPtrUnlock ( pAppInfo );

   }
   return 0;
}   

