/*
   VFS_MAI.CPP
   Main form handling functions.
   Copyright (c) Bachmann Software and Services, 1999
   Author: Glenn Bachmann
*/

// system headers
#include <palmos.h>
#include <palmutils.h>
#include <SysEvtMgr.h>
#include "expansionmgr.h"
#include "vfsmgr.h"


// application-specific headers
#include "vfsexplorer.h"
#include "vfs_res.h"

static void    MainFormInit (FormPtr formP);
static Boolean MainFormButtonHandler (FormPtr formP, EventPtr eventP);
static Boolean MainFormMenuHandler (FormPtr formP, EventPtr eventP);

// local helper functions
void MainListFill (FormPtr pForm);
void MainListDrawFunction (void * pTable, Int16 row, Int16 column, RectanglePtr bounds);
void MainListCalcTopVisible (FormPtr pForm);

// current record in the table 
static Int16 s_wCurrentRecord;
static Int16 s_wTopVisibleRecord;

#define COL_FILE_NAME   0

static UInt16 s_aryVolumes[10];
static UInt16 s_uiNumVolumes = 0;
static char s_szCurrentFolder[256];
static Char * s_aryFileNames[11];

Err EnumVolumes ( void );
Err EnumFolder ( UInt16 volRefNum, Char *pszFolder );

/*
   MainFormEventHandler:
   Parms:   pEvent   - event to be handled.
   Return:  true  - handled (successfully or not)
            false - not handled
*/
Boolean
MainFormEventHandler (EventPtr pEvent)
{
   Boolean  handled = false;
   
   switch (pEvent->eType)
   {
      case menuEvent:
      {
         FormPtr pForm = FrmGetActiveForm ();
         handled = MainFormMenuHandler (pForm, pEvent);
         break;
      }
      

      case ctlSelectEvent:
      {
         // A control button was tapped and released
         FormPtr pForm = FrmGetActiveForm ();
         handled = MainFormButtonHandler (pForm, pEvent);
         break;
      }

      case tblSelectEvent:
      {
         // set the current record based on user selection
         FormPtr pForm = FrmGetActiveForm ();
         s_wCurrentRecord = TblGetItemInt ((TableType *)pEvent->data.tblSelect.pTable,
                                         pEvent->data.tblSelect.row,
                                         pEvent->data.tblSelect.column );
                                         
			// if its a folder,                                          
			// switch the current folder and refill the list           
			StrCopy (s_szCurrentFolder, s_aryFileNames[pEvent->data.tblSelect.row]);
			
			MainListFill (pForm);                              
         FrmDrawForm ( pForm );
			
         break;
      }

      // physical scroll buttons
   	case keyDownEvent:		
      {
	   	if (pEvent->data.keyDown.chr == pageUpChr)
		  	{
            FormPtr pForm = FrmGetActiveForm ();
			   s_wTopVisibleRecord --;
            MainListFill ( pForm );
            FrmDrawForm ( pForm );
   	    	handled = true;
		  	}
	   	else if (pEvent->data.keyDown.chr == pageDownChr)
	    	{
            FormPtr pForm = FrmGetActiveForm ();
			   s_wTopVisibleRecord ++;
            MainListFill ( pForm );
            FrmDrawForm ( pForm );
   	    	handled = true;
         }
         break;
      }
      
      case frmOpenEvent:
      {
         // main form is opening

         FormPtr pForm = FrmGetActiveForm ();

         MainFormInit (pForm);

         FrmDrawForm (pForm);

         handled = true;
         break;
      }
      
      default:
      {
         break;
      }
   }
   return handled;
}

/*
   MainFormInit:
   Initialize the main form.
   Parms:   pForm - pointer to main form.
   Return:  none
*/
void
MainFormInit (FormPtr pForm)
{
   // set the controls to an initial state
   s_wCurrentRecord = -1;

	// get a list of volumes
	EnumVolumes ();
	
	for (UInt16 i = 0; i < 11; i++)
	{
		s_aryFileNames[i] = (Char *)MemPtrNew (256);
		StrCopy (s_aryFileNames[i], "");
	}
	
	if (s_uiNumVolumes > 0)
	{
		StrCopy (s_szCurrentFolder, "/");
		
		// walk through the list of files in the current directory
		EnumFolder ( s_aryVolumes[0], s_szCurrentFolder);
	}
	
   // walk through files in current folder and fill list
   MainListFill ( pForm );
}

/*
   MainFormMenuHandler:
   Handle a command sent to the main form.
   Parms:   pForm    - form handling event.
            command  - command to be handled.
   Return:  true  - handled (successfully or not)
            false - not handled
*/
Boolean
MainFormMenuHandler (FormPtr /*pForm*/, EventPtr eventP)
{
   Boolean handled = false;
   
   switch (eventP->data.menu.itemID)
   {
      default:
      {
         handled = false;
         break;
      }
   }
   return handled;
}

/*
   MainFormButtonHandler:
   Handle a command sent to the main form.
   Parms:   pForm    - form handling event.
            eventP   - event to be handled.
   Return:  true  - handled (successfully or not)
            false - not handled
*/

Boolean
MainFormButtonHandler (FormPtr pForm, EventPtr eventP)
{
   Boolean handled = false;
   
   switch (eventP->data.ctlEnter.controlID)
   {

		case MainBackButton:
		{
			Char *fNameP;

			if (StrLen ( s_szCurrentFolder ) > 1 )
			{
				// move pointer to end and move backward looking for first '/' character
				fNameP = s_szCurrentFolder + StrLen( s_szCurrentFolder ) - 1;

				// skip past the trailing /
				fNameP--;
				
				while( *fNameP != '/' )
				    fNameP--;
				
				// add a null terminator after that
				fNameP++;
				
				*fNameP = 0;
				
				FrmCustomAlert (ErrorAlert, s_szCurrentFolder, NULL, NULL);     
				MainListFill (pForm);                              
	         FrmDrawForm ( pForm );
			}
			break;
		}
      default: 
      {
         handled = false;
         break;
      }
   }
   return handled;
}

/*
   MainListFill:
   Fills the Table 
   Parms: pForm - pointer to main form
   Returns: none
*/
void
MainListFill (FormPtr pForm)
{
   // get a ptr to the table
   Int16 wIDTable = FrmGetObjectIndex (pForm, MainFilesTable );
   TablePtr pTable = (TablePtr) FrmGetObjectPtr (pForm, wIDTable);

   // init the table

   // get the number of rows defined in Constructor for the table
   // this the number that will be visible
   Int16 wRows = TblGetNumberOfRows (pTable);
   Int16 i;

   for (i = 0; i < wRows; i++ )
   {
      // set the cells to have custom drawing
      TblSetItemStyle  (pTable, i, COL_FILE_NAME, customTableItem );

      // set the row to be unusable to start
      TblSetRowUsable (pTable, i, false);
   }

   // set the columns to be usable
   TblSetColumnUsable  (pTable, COL_FILE_NAME, true );

   // set our draw procedure callback for each column
   TblSetCustomDrawProcedure (pTable, COL_FILE_NAME, MainListDrawFunction );

   // reset the top visible row based on size of table and current record
   MainListCalcTopVisible (pForm);

   UInt16 uRecordNum = 0;

   // load records into the table, starting at top visible
   // get the record number of the top visible record
   uRecordNum = s_wTopVisibleRecord;

	for (UInt16 i = 0; i < 11; i++)
	{
		s_aryFileNames[i] = (Char *)MemPtrNew (256);
		StrCopy (s_aryFileNames[i], "");
	}

	EnumFolder ( s_aryVolumes[0], s_szCurrentFolder );

   for (i = 0; i < wRows; i++, uRecordNum++)
   {
      UInt16 uNextRecordNum = 0;

      if (StrCompare (s_aryFileNames[i], "") != 0)
      {
         // remember the associated record number for this row
         TblSetItemInt (pTable, i, COL_FILE_NAME, uRecordNum);
         TblSetRowUsable (pTable, i, true);
      }
      else
      {
         // if we are out of records, mark row as unusable
         TblSetRowUsable (pTable, i, false);
      }
      // force a row redraw
      TblMarkRowInvalid (pTable, i );
   }

   return;
}

/*
   MainListCalcTopVisible:
   Sets the top visible item in the table
   Parms: none
   Returns: none
*/
void
MainListCalcTopVisible (FormPtr pForm)
{
   // get a ptr to the table
   Int16 wIDTable = FrmGetObjectIndex (pForm, MainFilesTable );
   TablePtr pTable = (TablePtr) FrmGetObjectPtr (pForm, wIDTable);

   // get the number of rows defined in Constructor for the table
   // this the number that will be visible
   Int16 wRows = TblGetNumberOfRows (pTable);

   // this all sets up what the top visible record
   // should be
   UInt16 uRecordNum = 0;

   if ( s_wCurrentRecord != -1)
   {
      if ( s_wTopVisibleRecord > s_wCurrentRecord )
      {
         // the current record is before the first visible record
         // "scroll up"
         s_wTopVisibleRecord = s_wCurrentRecord;
      }
      else 
      {
         if ( uRecordNum < s_wCurrentRecord)
         {
            // the last visible record in the table is less than the 
            // current record (so the current record is after the visible
            // portion of the table). We need to reset the top 
            // visible record such that it is the current record.
            // this is a "scroll down"
            s_wTopVisibleRecord = s_wCurrentRecord;
         }
      }
   }

   // now adjust to make sure that we have a full number of records even
   // if its the last screen. If there's less than a screenful left,
   // we have to push TopVisible backwards until there is.
   uRecordNum = 0;

   s_wTopVisibleRecord = min (s_wTopVisibleRecord, uRecordNum);
   return;
}

/*
   MainListDrawFunction:
   Draws a single cell in the main table
   Parms: pTable = table ptr
          row = row of item
          column = cell
          bounds = bounding rectangle for cell
   Returns: none
*/
void 
MainListDrawFunction (void * pTable, Int16 row, Int16 column, RectanglePtr bounds)
{
   FormPtr pForm = FrmGetActiveForm ();

	Char *		pText;

	
	// get the file name
	pText = s_aryFileNames[row];

   // draw it
	if ( pText )
   {
   	WinDrawChars ( pText, StrLen (pText), bounds->topLeft.x, bounds->topLeft.y);
   }
	
   return;
}



/*
	Enumerates all volumes on all cards
	Parms: none
	Returns: none
*/

Err 
EnumVolumes ( void )
{
	Err             err;
	UInt32          volIterator = vfsIteratorStart;
	UInt16          volRefNum;
	UInt16          uiCardNo;
	UInt16          uiCardsFound = 0;

   for (UInt16 i = 0; i < 10; i++)
   {
   	s_aryVolumes[i] = vfsInvalidVolRef;
	}
	   
   volRefNum = vfsInvalidVolRef;
    
	while (volIterator != vfsIteratorStop && s_uiNumVolumes < 10) 
	{
	   err = VFSVolumeEnumerate(&volRefNum, &volIterator);
	   if (err == errNone) 
	   {
	   	s_aryVolumes[s_uiNumVolumes] = volRefNum;
			s_uiNumVolumes++;
	   } 
	} 
   return errNone;	
} 

/*
	
*/
Err 
EnumFolder ( UInt16 volRefNum, Char *pszFolder )
{
	Err             err;
	Char            dirName[256], fileName[256];
	UInt32          fileIterator;
	FileInfoType    fileInfo;
	FileRef         dirRef, fileRef;
	
	UInt32          volIterator = vfsIteratorStart;
	if ( volRefNum == vfsInvalidVolRef )
	{
		return vfsErrVolumeBadRef;
	}
	
	fileInfo.nameP = fileName; // point to local buffer
	fileInfo.nameBufLen = 256;
	
	Char        tempName[256];

	StrCopy ( dirName, pszFolder );

	err = VFSFileOpen( volRefNum, dirName, vfsModeRead, &dirRef );
	
	if ( err != errNone )
	{
		Char    errStr[25];
		StrPrintF ( errStr, "Error opening dir" );
		FrmCustomAlert ( ErrorAlert, errStr, NULL, NULL );
		return err;
	}
	
	fileIterator = vfsIteratorStart;
        
   UInt16 uiFilePos = 0;
   
	while ( fileIterator != vfsIteratorStop )
	{
		err = VFSDirEntryEnumerate( dirRef, &fileIterator, &fileInfo );

		if ( (err != errNone) && (err != expErrEnumerationEmpty) )
		{
		    FrmCustomAlert( ErrorAlert, "Error in VFSDirEntryEnumerate!", NULL, NULL );
		    return err;
		}

		if (uiFilePos >= s_wTopVisibleRecord)
		{			
			StrCat (s_aryFileNames[uiFilePos - s_wTopVisibleRecord], dirName);
			StrCat (s_aryFileNames[uiFilePos - s_wTopVisibleRecord], fileInfo.nameP);
		   if ( (fileInfo.attributes & vfsFileAttrDirectory) && !(fileInfo.attributes & vfsFileAttrVolumeLabel) )
			{
				StrCat (s_aryFileNames[uiFilePos - s_wTopVisibleRecord], "/");
			}				

		}
		
		uiFilePos++;
		
	} 
	
	VFSFileClose( dirRef );
	return errNone;
        
} 

