/*
   PalmTalkmain.cpp
   Serial Manager Example Program
   Copyright (c) Bachmann Software and Services, 1999-2002
   Author: W.F. Ableson
*/

#include <PalmOS.h>
#include <SysEvtMgr.h>
#include <SerialMgrOld.h>
#include "PalmTalkmain.h"		// MainFormEventHandler ()
#include "PalmTalk_res.h"


static void    MainFormInit (FormPtr formP);
static Boolean MainFormButtonHandler (FormPtr formP, EventPtr eventP);
static Boolean MainFormMenuHandler (FormPtr formP, EventPtr eventP);
void    MainFormOpenSerial (FormPtr formP);
void    MainFormWriteSerial (FormPtr formP);
static void    MainFormVersion (FormPtr formP);


static UInt16 SerialRefNum;
static int Connected = 0;


/*
   MainFormEnableControl:
   Parms:   formP form pointer,
            object id of button to enable/disable, 
            true for enable | false to disable
   Return:  none
*/
static void
MainFormEnableControl(FormPtr formP, UInt16 objectId, Boolean enabled)
{
  // Get object index from form
  UInt16 objectIndex = FrmGetObjectIndex(formP,objectId);

  // get ptr to control
  ControlPtr controlP = (ControlPtr) FrmGetObjectPtr (formP, objectIndex);

  // set enabled
  CtlSetEnabled(controlP, enabled);

  if (enabled) 
  {
    CtlShowControl (controlP);
  }
  else
  {
    CtlHideControl (controlP);
  }
}



/*
   MainFormEventHandler:
   Parms:   pEvent   - event to be handled.
   Return:  true  - handled (successfully or not)
            false - not handled
*/
Boolean
MainFormEventHandler (EventPtr eventP)
{
   Boolean  handled = false;
   
   switch (eventP->eType)
   {
      case menuEvent:
      {
         FormPtr formP = FrmGetActiveForm ();
         handled = MainFormMenuHandler (formP, eventP);
         break;
      }
      
      case ctlSelectEvent:
      {
      // A control button was pressed.
         FormPtr formP = FrmGetActiveForm ();
         handled = MainFormButtonHandler (formP, eventP);
         break;
      }
      
      case frmOpenEvent:
      {
         FormPtr formP = FrmGetActiveForm ();
         MainFormInit (formP);
         FrmDrawForm (formP);
         handled = true;
         break;
      }
      
      default:
      {
         break;
      }
   }
   return handled;
}

/*
   MainFormInit:
   Initialize the main form.
   Parms:   formP - pointer to main form.
   Return:  none
*/
void
MainFormInit (FormPtr /*formP*/)
{
}

/*
   MainFormButtonHandler:
   Handle a command sent to the main form.
   Parms:   formP    - form handling event.
            command  - command to be handled.
   Return:  true  - handled (successfully or not)
            false - not handled
*/
Boolean
MainFormMenuHandler (FormPtr formP, EventPtr eventP)
{
   Boolean handled = false;
/*   
   switch (eventP->data.menu.itemID)
   {
      
   }
*/
   return handled;
}

/*
   MainFormButtonHandler:
   Handle a command sent to the main form.
   Parms:   formP    - form handling event.
            eventP   - event to be handled.
   Return:  true  - handled (successfully or not)
            false - not handled
*/
Boolean
MainFormButtonHandler (FormPtr formP, EventPtr eventP)
{
   Boolean handled = false;
   
   switch (eventP->data.ctlEnter.controlID)
   {
      case MainBtnOpenButton:   // User selected Open Serial button.
      {
         MainFormOpenSerial (formP);
         handled = true;
         break;
      }

      case MainBtnSendButton:   // User selected Write Serial button.
      {
         MainFormWriteSerial (formP);
         handled = true;
         break;
      }

      case MainBtnCloseButton:   // User selected Close Serial button.
      {
         MainFormCloseSerial();
         // modify user interface
         MainFormEnableControl(formP,MainBtnCloseButton,false);
         MainFormEnableControl(formP,MainBtnSendButton,false);
         MainFormEnableControl(formP,MainBtnOpenButton,true);

         handled = true;
         break;
      }


  }
   return handled;
}


/*
   MainFormOpenSerial:
   Handle the main form's OpenSerial command.
   Parms:   formP    - form handling event.
   Return:  none
*/
void
MainFormOpenSerial (FormPtr formP)
{
   Err e;
   SerSettingsType sstSetup;
   
   // load the serial library
   e = SysLibFind("Serial Library",&SerialRefNum);
   if (e) 
   {
      FrmCustomAlert(ErrorAlert,"Failed To Load Library!","","");
      return;
   }
   
   // now, let's open the port itself.  We will request an initial
   // baud rate of 9600.
   e = SerOpen(SerialRefNum,0,9600);
   if (e) 
   {
      FrmCustomAlert(ErrorAlert,"Failed To Open Port 0!","","");
      return;
   }

   // let's keep track of when the port is open
   Connected = 1;

   // before we can set the parameters on the port, it is a good
   // practice to retrieve them first.
   SerGetSettings( SerialRefNum, &sstSetup );

   // let's set the baud rate and other serial comms properties
   sstSetup.baudRate = 9600;
   sstSetup.flags =    serSettingsFlagBitsPerChar7 | 
                        serSettingsFlagParityOnM |    
                        serSettingsFlagParityEvenM |
                        serSettingsFlagStopBitsM |
                        serSettingsFlagStopBits1;
   sstSetup.ctsTimeout = serSettingsFlagRTSAutoM | serSettingsFlagCTSAutoM;

   // Ask Serial Manager to update the port settings
   SerSetSettings( SerialRefNum, &sstSetup );

   // let's clear the port, just in case some garbage is sitting there
   SerReceiveFlush (SerialRefNum, 100 );

   // change the state of some buttons on our main form
   MainFormEnableControl(formP,MainBtnCloseButton,true);
   MainFormEnableControl(formP,MainBtnSendButton,true);
   MainFormEnableControl(formP,MainBtnOpenButton,false);
}
   

/*
   MainFormWriteSerial:
   Handle the main form's WriteSerial command.
   Parms:   formP    - form handling event.
   Return:  none
*/
void
MainFormWriteSerial (FormPtr formP)
{
   unsigned short usRegister;
   int i;
   Err e;
   UInt16 wIDField; 
   FieldPtr pCtlField;
   MemHandle hText;
   char * pText;


   // check connected .. just in case
   if (Connected == 0) return;

   // get message from our field
   wIDField = FrmGetObjectIndex(formP,MainMessageField);
   pCtlField = (FieldPtr) FrmGetObjectPtr(formP,wIDField);
   hText = FldGetTextHandle (pCtlField);
   pText = (char *) MemHandleLock(hText);

   // let's send our message!
   SerSend(SerialRefNum,(unsigned char *) pText,StrLen(pText),&e);
   if (e)
   {
      FrmCustomAlert(ErrorAlert,"Error Sending Message!","","");
   }
   MemHandleUnlock(hText);
   // let's send along a newline, so if we can use PalmTalk on two devices
   // connected together with a null modem cable!
   SerSend(SerialRefNum,(unsigned char *) "\n",1,&e);
   if (e)
   {
      FrmCustomAlert(ErrorAlert,"Error Sending Message!","","");
   }


}


/*
   MainFormReadSerial:
   Handle the main form's ReadSerial command.
   Parms:   none
   Return:  none
*/
void
MainFormReadSerial ()
{
   static unsigned char szBuf[1024];
   static int index = 0;
   Err e;
   UInt32 ulBytes;


   if (Connected == 0) return;
   // see if there is anything in the queue ... "peek"
   e = SerReceiveCheck(SerialRefNum,&ulBytes);

   // ensure there were no errors
   if (e) 
   {
      FrmCustomAlert(ErrorAlert, "Error Checking Serial Port!","","");
      return;
   }

   // is there something waiting for us ?
   if (ulBytes) 
   {
     // let's retrieve the info!

     // first, make sure the amount to read is not too large
     if ((ulBytes + index) > sizeof(szBuf))
     {
        ulBytes = sizeof(szBuf) - index - 1;
     }

     // retrieve one byte at a time, looking for our end of message indicator
     while (ulBytes)
     {
        SerReceive(SerialRefNum,&szBuf[index],1,0,&e);
        // check for read error
        if (e)
        {
           FrmCustomAlert(ErrorAlert, "Error Reading Serial Port!","","");
           SerReceiveFlush(SerialRefNum,100);
           index = 0;
           return;
        }
        switch (szBuf[index])
        {
            case 0x0a:
              szBuf[index] = 0x00;
              // we have our new line indicator .. that will mark the end
              // of this "message" so let's display it!
              FrmCustomAlert(MessageAlert,(char *) szBuf,"","");
              index = 0;
              break;
            default:
              index++;
              break;
        }
        ulBytes--;
      } // while
   }

}

/*
   MainFormCloseSerial:
   Handle the main form's CloseSerial command.
   Parms:   none
   Return:  none
*/
void
MainFormCloseSerial()
{
   Err e;

   // close port
   e = SerClose(SerialRefNum);
   if (e) 
   {
      FrmCustomAlert(ErrorAlert,"Error during Close!","","");
   }
   
   // keep track of our state
   Connected = 0;
   
}



/*
   MainFormVersion:
   Handle the main form's version command.
   Parms:   formP    - form handling event (not used).
   Return:  true     - handled (successfully or not)
            false    - not handled
*/
void
MainFormVersion (FormPtr /*formP*/)
{
}
   
