//	File:	basemon.cpp
//
//	Module: 
//
//	Description:	Source code for the ConduitMonitor object 
//
//     CBaseConduitMonitor   - base monitor object 
//     CBaseDTLinkConverter  - base class for HH<->PC data conversion
/////////////////////////////////////////////////////////////////////////////
#include "string.h"
#include "memory.h"
#include <tchar.h>

#include "afx.h"    // All of these includes are necessary for table stuff
#include "bfields.h"
#include "basetabl.h"

#include "syncmgr.h"
#include "condapi.h"
#include "basemon.h"
#include "hslog.h"
#include "updcatid.h"
#include "catmgr.h"
#include "logstrng.h"   // for Resource strings


#define	CATEGORY_LENGTH		16
#define	APPINFOBLOCK_SIZE	800
#define MAX_LOG_STRING		128
#define HH_NAME				"Palm organizer"
#define MAX_PROD_ID_TEXT	255
#define	ROM_VER_2			2

#define DOS_RESERVED_COUNT		12
char *ReservedFileNames[DOS_RESERVED_COUNT] = { "aux",  "con",    "prn",  "lpt1", "lpt2", 
                                "lpt3", "clock$", "com1", "com2", "com3", 
                                "com4", "nul" };
///////////////////////////////////////////////////////////////////////////
// CBaseConduitMonitor 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		CBaseConduitMonitor()      Public 
//
//	Description:	Constructor	
//
//	Parameters:		PROGRESSFN			Used for callback to send progress
//					CSyncProperties		Contains syncType and filenames
//
//	Returns:		n/a
//
///////////////////////////////////////////////////////////////////////////
CBaseConduitMonitor::CBaseConduitMonitor(PROGRESSFN pFn,
                                         CSyncProperties& rProps, 
                                         HINSTANCE hInst)
                    : m_LocRealTable(NULL),  m_LocArchTable(NULL),
                      m_RemRealTable(NULL),  m_BackupTable(NULL),
                      m_RemCategory(NULL),   m_LocCategory(NULL),
                      m_TotRemoteDBs(1),     m_CurrRemoteDB(0),
                      m_DllInstance(hInst),	 m_wRawRecSize(0)  
{
	m_pfnProgress = pFn;
	m_pDTConvert  = NULL;
	memcpy(&m_rSyncProperties, &rProps, sizeof(m_rSyncProperties)); 
	memset(&m_DbGenInfo, 0, sizeof(m_DbGenInfo));

	// Allocate memory used by System Info call
	memset(&m_SystemInfo, 0, sizeof(m_SystemInfo));
	m_SystemInfo.m_ProductIdText = (BYTE*) new char [MAX_PROD_ID_TEXT];
	m_SystemInfo.m_AllocedLen = (BYTE) MAX_PROD_ID_TEXT; 

	// added for file link
	m_lFilelinkSupported = -1;
}

///////////////////////////////////////////////////////////////////////////
//
//	Function:		CBaseConduitMonitor()      Public 
//
//	Description:	Destructor	
//
//	Parameters:		None
//
//	Returns:		n/a
//
///////////////////////////////////////////////////////////////////////////
CBaseConduitMonitor::~CBaseConduitMonitor()  
{
	if (m_pDTConvert) 
		delete m_pDTConvert; 

	if (m_LocArchTable)  
		delete m_LocArchTable;

	if (m_LocRealTable)  
		delete m_LocRealTable;

	if (m_BackupTable)  
		delete m_BackupTable;

	if (m_RemRealTable) 
		delete m_RemRealTable;

	if (m_RemCategory)
		delete m_RemCategory; 

	if (m_DbGenInfo.m_pBytes) 
		delete m_DbGenInfo.m_pBytes; 

	// Free System Info memory
	delete m_SystemInfo.m_ProductIdText;
}

///////////////////////////////////////////////////////////////////////////
//
//	Function:		SetDllIntance()      Public 
//
//	Description:	Place the Dll instance handle for use in obtaining
//					string resources.
//
//	Parameters:		HINSTANCE	Dll instance handle
//
//	Returns:		n/a
//
///////////////////////////////////////////////////////////////////////////
void CBaseConduitMonitor::SetDllInstance(HINSTANCE hInst)
{
	m_DllInstance = hInst;   
}

///////////////////////////////////////////////////////////////////////////
//
//	Function:		Engage()      Public 
//
//	Description:	This method offers the public a means of getting the
//					conduit churning onto it's main duty of synchronization.
//                  This routine will offload the real work once it detects
//                  the type of sync from the properties object.
//
//	Parameters:		None
//
//	Returns:		long	error code
//
///////////////////////////////////////////////////////////////////////////
long CBaseConduitMonitor::Engage(void)  
{
	long retval = 0;

	// Obtain System Information
	retval = SyncReadSystemInfo(m_SystemInfo);

	if(retval == 0)
	{
		retval = CONDERR_BAD_SYNC_TYPE;

		switch (m_rSyncProperties.m_SyncType)
		{
			case eHHtoPC:
			case ePCtoHH:
			case eFast:
			case eSlow:
				retval = EngageStandard();  
				break;

			case eInstall:
			case eProfileInstall:
				retval = EngageInstall();  
				break;

			case eBackup:
				retval = EngageBackup();  
				break;

			case eDoNothing:
				retval = EngageDoNothing();
				break;

			default:
				break;
		}
	}

	return(retval);
}


///////////////////////////////////////////////////////////////////////////
// CBaseConduitMonitor 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		EngageInstall()     ** virtual Protected ** 
//
//	Description:    This virtual routine is meant to be overridden in a 
//                  derivative and is in place only to define an 'interface'
//                  for the main engine.
//
//	Parameters:		None
//
//	Returns:		long	error code
//
///////////////////////////////////////////////////////////////////////////
long CBaseConduitMonitor::EngageInstall(void)  
{
	return(-1);
}

///////////////////////////////////////////////////////////////////////////
// CBaseConduitMonitor 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		EngageDoNothing()     ** virtual Protected ** 
//
//	Description:    This virtual routine may be overridden in a 
//                  derivative if something needs to be done in a DoNothing case
//
//	Parameters:		None
//
//	Returns:		long	error code
//
///////////////////////////////////////////////////////////////////////////
long CBaseConduitMonitor::EngageDoNothing(void)  
{
	LogDoNothing();
	return(0);
}

///////////////////////////////////////////////////////////////////////////
// CBaseConduitMonitor 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		EngageBackup()     ** virtual Protected ** 
//
//	Description:    This virtual routine is meant to be overridden in a 
//                  derivative and is in place only to define an 'interface'
//                  for the main engine.
//				
//	Parameters:		None
//
//	Returns:		long	error code
//
///////////////////////////////////////////////////////////////////////////
long CBaseConduitMonitor::EngageBackup(void)  
{
	return(-1);
}


///////////////////////////////////////////////////////////////////////////
// CBaseConduitMonitor 
///////////////////////////////////////////////////////////////////////////
//
//	Function:     EngageStandard()        ** Protected ** 
//
//	Description:  This routine handles the 4 main brans of synchronization
//                  	
//	The Progress Callback is not currently used, but here is an example of
//	how you could use it:
//		char probString[64];
//		memset(probString,0,sizeof(probString));
//		strcpy(probString, "Joe");
//		(*m_pfnProgress)(probString);
//
//	Parameters:		None
//
//	Returns:		error code
//
///////////////////////////////////////////////////////////////////////////
long CBaseConduitMonitor::EngageStandard(void)  
{
	CONDHANDLE	conduitHandle = (CONDHANDLE)0;
	long 		retval = 0; 
	char		appName[40];
	long		pcCount = 0;
	WORD		hhCount = 0;
	Activity    syncFinishCode = slSyncFinished;


	// Register this conduit with SyncMgr.DLL for communication to HH
	if (retval = SyncRegisterConduit(conduitHandle))
		return(retval);

	// Notify the log that a sync is about to begin
	LogAddEntry("", slSyncStarted,FALSE);	         

	memset(&m_DbGenInfo, 0, sizeof(m_DbGenInfo));

	//  Loop through all possible 'remote' db's
	for (; m_CurrRemoteDB < m_TotRemoteDBs && !retval; m_CurrRemoteDB++)
	{
		// Open the Remote Database
		retval = ObtainRemoteTables();
	
		// Open PC tables and load local records && local categories.   
		if (!retval && !(retval = ObtainLocalTables()))
		{
			if( !(retval) )
			{
				FlushPCRecordIDs();

				if (!(retval = ObtainRemoteCategories()))
				{
					// Synchronize the AppInfoBlock Info excluding the categories
					m_pDTConvert->SynchronizeAppInfoBlock(m_DbGenInfo, 
														 *m_LocRealTable, 
												  m_rSyncProperties.m_SyncType,
												  m_rSyncProperties.m_FirstDevice);

					// Synchronize the categories
					retval = SynchronizeCategories();
				}
			}
	
		}

		// Synchronize the records
		if (!retval)
		{

			if (m_rSyncProperties.m_SyncType == eHHtoPC)
				retval = CopyRecordsHHtoPC();
			else if (m_rSyncProperties.m_SyncType == ePCtoHH)
				retval = CopyRecordsPCtoHH();
			else if (m_rSyncProperties.m_SyncType == eFast)
				retval = FastSyncRecords();
			else if (m_rSyncProperties.m_SyncType == eSlow)
				retval = SlowSyncRecords();

		}

		// If the number of records are not equal after a FastSync or
		// SlowSync: If the PC has more records, then do a PCtoHH Sync.
		// If the HH has more records, then do a HHtoPC Sync.
		if (!retval && ((m_rSyncProperties.m_SyncType == eFast) || 
							(m_rSyncProperties.m_SyncType == eSlow)))
		{
			// Get the record counts
			pcCount = GetLocalRecordCount(); 
			if (!(retval = SyncGetDBRecordCount(m_RemHandle, hhCount)))
			{
				if (pcCount > (long)hhCount)
					retval = CopyRecordsPCtoHH();

				else if (pcCount < (long)hhCount)
				{
					m_LocRealTable->PurgeAllRecords();
					retval = CopyRecordsHHtoPC();
				}
			}
		}
		if (!retval || !IsCommsError(retval)) 
		{
			// Re-check the record counts, only if we've obtained rem tables
			pcCount = GetLocalRecordCount(); 
			hhCount = 0;
			retval  = SyncGetDBRecordCount(m_RemHandle, hhCount);
			
			// If the record counts are not equal, send message to the log.
			if (pcCount < (long)hhCount)
			{
				LogRecCountMismatch(pcCount, (long)hhCount);			
				syncFinishCode = slSyncAborted;
			}
			else if (pcCount > (long)hhCount)
			{
				LogPilotFull(pcCount, (long)hhCount);			
				syncFinishCode = slSyncAborted;
			}
		}


		// This allows exact display order matching with the remote device.
		if (!retval || !IsCommsError(retval))
			if (ApplyRemotePositionMap())
				LogBadXMap();

		if (!retval || IsRemoteMemError(retval)) 
		{
			// Save all records to be archived to their appropriate files
			if (ArchiveRecords())
				LogBadArchiveErr();
				
			// Copy PC file to Backup PC file
			CString backFile(m_rSyncProperties.m_PathName);
			CString dataFile(m_rSyncProperties.m_PathName);
			backFile += m_rSyncProperties.m_LocalName;
			dataFile += m_rSyncProperties.m_LocalName;
			int nIndex = backFile.ReverseFind(_T('.'));
			if (nIndex != -1)
				backFile = backFile.Left(nIndex);
			backFile += BACK_EXT;

			// Save synced records to PC file
			if (!SaveLocalTables((const char*)dataFile))
			{
				// Clear HH status flags
				if (SyncResetSyncFlags(m_RemHandle))
				{
					LogBadResetSyncFlags();
					syncFinishCode = slSyncAborted;
				}
				remove(backFile);
				CopyFile(dataFile, backFile, FALSE);
			}
			else
				syncFinishCode = slSyncAborted;
		}
		if (!IsCommsError(retval))
			SyncCloseDB(m_RemHandle);
	}

	if (retval)
		syncFinishCode = slSyncAborted;

	
	// Get the application name
	memset(appName, 0, sizeof(appName));
	LogApplicationName(appName, sizeof(appName)); 

	LogAddEntry(appName, syncFinishCode,FALSE);
		
	if (!IsCommsError(retval))
		SyncUnRegisterConduit(conduitHandle);

	return(retval);
}


///////////////////////////////////////////////////////////////////////////
// CBaseConduitMonitor 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		CopyRecordsHHtoPC()     ** virtual Protected ** 
//
//	Description:	This is a virtual routine that *must* be overridden
//					by derived classes.
//
//					This routine copies all of the records from the handheld
//					(HH) to the PC excluding the records marked for archive
//					ordeletion.  Records marked for archive are added to
//					the archive table and later added to their appropriate
//					archive files.	
//
//	Parameters:		None
//
//	Returns:		long	error code
//
///////////////////////////////////////////////////////////////////////////
long CBaseConduitMonitor::CopyRecordsHHtoPC(void)  
{
	long			convErr = 0, retval = 0, err = 0;
	CRawRecordInfo	rawRecord;
	WORD            recIx = 0;

	memset(&rawRecord, 0, sizeof(CRawRecordInfo));
	rawRecord.m_FileHandle = m_RemHandle;   // remote file handle           
	rawRecord.m_RecIndex = recIx ;        

	if (!m_RemRealTable)
		return(CONDERR_BAD_REMOTE_TABLES);

	// Create a holding place for converted remote field values.
	// We need at least one valid record (with valid fields) in the table.
	
	// declare a pointer to the base class which needs to point to 
	// the appropriate derived class object 
	CBaseRecord* pRemRecord = NULL;
	if (retval = ConstructRecord(pRemRecord, *m_RemRealTable, 0))
		return (retval);
	
	if (m_RemRealTable->AppendBlankRecord(*pRemRecord))
	{	
		delete pRemRecord;
		return(CONDERR_BAD_REMOTE_TABLES);
	}

	// Allocate memory for rawRecord.m_pBytes, return if Bad!
	if (retval = AllocateRawRecordMemory(rawRecord, m_wRawRecSize))
	{
		delete pRemRecord;
		return(retval);
	}

	// Read in each Remote (HH) record one at a time
	while (!retval && !err)
	{
		rawRecord.m_RecIndex = recIx ;        
		if (!(err = SyncReadRecordByIndex(rawRecord)))
		{
			// Convert from raw record format to corresponding 
			// conduit record format
            convErr = m_pDTConvert->ConvertFromRemote(*pRemRecord, rawRecord);
			if (!convErr || !IsFatalConduitError(convErr, pRemRecord))
			{
				if (pRemRecord->IsArchived())
					// Add HH record to Archive table
						retval = ClearStatusAddRecord(*pRemRecord, *m_LocArchTable);
				else if (pRemRecord->IsDeleted() == FALSE)
				{
					LogModifiedSubscRec(pRemRecord, FALSE);
					// Add HH record to PC table
					retval = ClearStatusAddRecord(*pRemRecord, *m_LocRealTable);
				}
			}
			else
				retval = CONDERR_CONVERT_FROM_REMOTE_REC; 
		}
		recIx++;
	}
	if (err != SYNCERR_FILE_NOT_FOUND)
			LogBadReadRecord(err);

	// Free memory for rawRecord data
	if (rawRecord.m_TotalBytes > 0 && rawRecord.m_pBytes)
		delete rawRecord.m_pBytes;

	if (pRemRecord)
		delete pRemRecord;

	// Delete all records marked for deletion on the handheld
	if (!retval && SyncPurgeDeletedRecs(m_RemHandle))
		LogBadPurge(CONDERR_REMOTE_RECS_NOT_PURGED);
	
	return(retval);
}


///////////////////////////////////////////////////////////////////////////
// CBaseConduitMonitor 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		CopyRecordsPCtoHH()     ** virtual Protected ** 
//
//	Description:	This is a virtual routine that *must* be overridden
//					by derived classes.	
//
//					This routine copies all of the records from the PC to
//					the handheld (HH) excluding the records marked for
//					archive or deletion.  Records marked for archive are
//					added to the archive table and later added to their
//					appropriate archive files.
//
//	Parameters:		None	
//
//	Returns:		long	error code
//
///////////////////////////////////////////////////////////////////////////
long CBaseConduitMonitor::CopyRecordsPCtoHH(void)  
{
	long			retval = 0, err = 0;
	CBaseRecord*	plocRecord = NULL;
	if (retval = ConstructRecord(plocRecord, *m_LocRealTable, 0))
		return (retval);
	CBaseIterator	locIterator(*m_LocRealTable);

	// Delete all Remote (Handheld) records
	if (SyncPurgeAllRecs(m_RemHandle))
	{
		retval = CONDERR_REMOTE_RECS_NOT_PURGED;
		delete plocRecord;
		return(retval);
	}
	
	// For each PC record ...
	err = locIterator.FindFirst(*plocRecord, FALSE);
	while (!err && !retval)
	{
		if (plocRecord->IsArchived())
		{
			// Add PC record to Archive table
			retval = ClearStatusAddRecord(*plocRecord, *m_LocArchTable);
			// Mark for deletion
			plocRecord->SetStatus(fldStatusDELETE);
		}
		else if (!plocRecord->IsDeleted()) // record not deleted
		{
			if (!plocRecord->IsNone())  // either modified or added
			{
				LogModifiedSubscRec( plocRecord, TRUE);
			}
			// Add the record to the Handheld by virtual worker function.
			plocRecord->SetStatus(fldStatusNONE);
			if (retval = AddRemoteRecord(*plocRecord))
				LogBadAddRecord(*plocRecord); 
		}
		err = locIterator.FindNext(*plocRecord, FALSE);
	}

	// Purge all deleted records from the PC table
	if (!retval)
		retval = m_LocRealTable->PurgeDeletedRecords();

	if (plocRecord)
		delete plocRecord;

	return(retval);
}



///////////////////////////////////////////////////////////////////////////
// CBaseConduitMonitor 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		FastSyncRecords()     ** virtual Protected ** 
//
//	Description:	This is a virtual routine that *must* be overridden
//					by derived classes.	
//
//		FastSync reads and writes only the modified records to and from the
//		Handheld.  It relies on the statusFlags for each record to be accurate.
//
//	Procedure: 
//		All of the PC records have already been read into memory (into the PC
//		(m_LocRealTable) table).  Each modified HH record will be read in one
//		at a time.  Each modified HH record will be compared against the PC
//		table (which contains the PC records along with the newly merged HH 
//		records) to determine if the HH record should be added to PC table,
//		replace the current PC record, cause the PC record to be deleted from
//		the PC table, or be added to the Archive file.  After each HH record
//		has been read with its sync action performed to the PC table, then a
//		second pass is made to the PC table.  For each record in the PC table
//		that is marked as modified (statusFlag != 0), a message will be sent
//		to the HH to update that HH record.  After all the appropriate records
//		are updated on the HH and the statusFlags for each PC record have been
//		cleared, the PC table is ready to be written to the PC as the new file.
//
//	Parameters:		None
//
//	Returns:		long	error code
//
///////////////////////////////////////////////////////////////////////////
long CBaseConduitMonitor::FastSyncRecords(void)  
{
	long            convErr = 0, retval = 0, err = 0;
	CRawRecordInfo  rawRecord;
	memset(&rawRecord, 0, sizeof(rawRecord));
	rawRecord.m_FileHandle = m_RemHandle;  // remote file handle
	rawRecord.m_RecId      = 0;            // remote HH will assign RecId

	if (!m_RemRealTable)
		return(CONDERR_BAD_REMOTE_TABLES);

	// Create a holding place for converted remote field values.
	// We need at least one valid record (with valid fields) in the table.
	CBaseRecord* pRemRecord = NULL;
	if (retval = ConstructRecord(pRemRecord, *m_RemRealTable, 0))
		return (retval);
	if (m_RemRealTable->AppendBlankRecord(*pRemRecord))
		retval = CONDERR_BAD_REMOTE_TABLES;
	
	CBaseRecord* plocRecord = NULL;
	if (!retval)
		retval = ConstructRecord(plocRecord, *m_LocRealTable, 0);
	
	CBaseRecord* pbackRecord = NULL;
	if (!retval)
		retval = ConstructRecord(pbackRecord, *m_BackupTable, 0);
	
	// Allocate memory for raw record
	if (!retval)
		retval = AllocateRawRecordMemory(rawRecord, m_wRawRecSize);

	// Read in each modified Remote record one at a time
	while (!err && !retval)
	{
		if (!(err = SyncReadNextModifiedRec(rawRecord)))
		{
			// Convert from raw record format to CAddressRecord
            convErr = m_pDTConvert->ConvertFromRemote(*pRemRecord, rawRecord);
			if (!convErr || !IsFatalConduitError(convErr, pRemRecord))
			{
				// Synchronize the record obtained from the handheld
				retval = SynchronizeRecord(*pRemRecord, *plocRecord, *pbackRecord);
			}
			else
				retval = CONDERR_CONVERT_FROM_REMOTE_REC; 
		} 
		memset(rawRecord.m_pBytes, 0, rawRecord.m_TotalBytes);
	}
	if (err && err != SYNCERR_FILE_NOT_FOUND)
		LogBadReadRecord(err);

	// Free the memory allocated for the records
	if (rawRecord.m_TotalBytes > 0 && rawRecord.m_pBytes)
		delete rawRecord.m_pBytes;

	if (pRemRecord)
		delete pRemRecord;
	if (plocRecord)
		delete plocRecord;
	if (pbackRecord)
		delete pbackRecord;

	// Send all modified records to the HH. Supply a specific record object 
	// to the base class member function.
	if (!retval)
	{
		CBaseRecord* ploc2Rec = NULL;
		if (retval = ConstructRecord(ploc2Rec, *m_LocRealTable, 0))
			return (retval);
		retval = SendRemoteChanges(*ploc2Rec);
		if (ploc2Rec)
			delete ploc2Rec;
	}
	return(retval);
}


///////////////////////////////////////////////////////////////////////////
// CBaseConduitMonitor 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		SlowSyncRecords()     ** virtual Protected ** 
//
//	Description:  This is a virtual routine that *must* be overridden
//                by derived classes.	
//
//		SlowSync is used when the statusFlags can not be relied on for
//		accuracy.  If the HH is HotSynced with another PC, then the
//		statusFlags are wiped out.  SlowSync uses the Backup file, which
//		is a copy of the file after the last HotSync, to determine which
//		records have been added, changed, or deleted on the HH since the 
//		last HotSync.  To perform a SlowSync, every record must be read in
//		from the HH, as opposed to the FastSync which reads only the
//		modified records.
//
//	Procedure:
//		All of the PC records have already been read into memory (into the
//		PC (m_LocRealTable) table).  For each PC record, if the statusFlag
//		= None, it will be set to Pending.  Each HH record will be read in
//		one at a time.  Since the HH statusFlags may not be accurate, if the
//		HH statusFlag = None, then that record will be compared against the
//		Backup file record to determine if the HH record has been added,
//		changed, or deleted.  Then, each HH record will be compared against
//		the PC table (which contains the PC records along with the newly
//		merged HH records) to determine if the HH record should be added to
//		the PC table, replace the current PC record, cause the PC record to 
//		be deleted from the PC table, or be added to the Archive file.  If
//		the record exists on both the HH and the PC and the PC record has a
//		Pending statusFlag, the statusFlag is changed to its appropriate
//		value.  This is important because in the second pass, if the
//		statusFlag = Pending, that means that the record does not exist on
//		the HH and it does exist on the PC with no changes, therefore the
//		record was deleted on the HH so it needs to be deleted from the PC.
//		After each HH record has been read with its sync action performed to
//		the PC table, then a second pass is made to the PC table.  For each
//		PC record that is marked as modified (statusFlag != None), a message
//		will be send to the HH to update that HH record.  After all the
//		appropriate records are updated on the HH and the statusFlags for
//		each PC record have been cleared, then the PC table is ready to be
//		written to the PC as the new PC file.
//
//	Parameters:		None
//
//	Returns:		long	returns error code
//
///////////////////////////////////////////////////////////////////////////
long CBaseConduitMonitor::SlowSyncRecords(void)  
{
	long            convErr = 0, retval   = 0, tErr, err = 0;
	WORD            rawRecIx = 0;
	CRawRecordInfo	rawRecord;
	CBaseIterator	locIterator(*m_LocRealTable);

	CString backFile(m_rSyncProperties.m_PathName);

	// replace local file name extension with BACK_EXT
	backFile += m_rSyncProperties.m_LocalName;
	int nIndex = backFile.ReverseFind(_T('.'));
	if (nIndex != -1)
		backFile = backFile.Left(nIndex);
	backFile += BACK_EXT;

	memset(&rawRecord, 0, sizeof(rawRecord));
	rawRecord.m_FileHandle = m_RemHandle;            // Remote File Handle     

	// Read in PC Backup file (file after last sync) 
	tErr = m_BackupTable->OpenFrom(backFile, 0);

	if (!m_RemRealTable)
		return(CONDERR_BAD_REMOTE_TABLES);

	// Create a holding place for converted remote field values.
	// We need at least one valid record (with valid fields) in the table.
	
	CBaseRecord* pRemRecord = NULL;
	if (retval = ConstructRecord(pRemRecord, *m_RemRealTable, 0))
		return (retval);
	
	if (m_RemRealTable->AppendBlankRecord(*pRemRecord))
		return(CONDERR_BAD_REMOTE_TABLES);
	else (retval = AllocateRawRecordMemory(rawRecord, m_wRawRecSize));
		
	CBaseRecord* plocRecord = NULL;
	if (!retval)
		retval = ConstructRecord(plocRecord, *m_LocRealTable, 0);
	
	CBaseRecord* pbackRecord = NULL;
	if (!retval)
		retval = ConstructRecord(pbackRecord, *m_BackupTable, 0);

	// Set each PC record with statusFlag = None to Pending
	if (!retval)
		err = locIterator.FindFirst(*plocRecord, TRUE);
	else err = retval;
	while (!err)
	{
		if ((!plocRecord->IsArchived()) && plocRecord->IsNone())
			plocRecord->SetStatus(fldStatusPENDING);

		err = locIterator.FindNext(*plocRecord, TRUE);
	}

	err = 0;
	rawRecIx = 0;     
	while (!err && !retval)
	{
		rawRecord.m_RecIndex = rawRecIx;     

		// Read && Convert each remote record from raw format to CAddressRecord
		if (!(err = SyncReadRecordByIndex(rawRecord)))
		{
			// Convert from raw record format to CAddressRecord
            convErr = m_pDTConvert->ConvertFromRemote(*pRemRecord, rawRecord);
			if (!convErr || !IsFatalConduitError(convErr, pRemRecord))
			{
				// Synchronize the record obtained from the handheld
				retval = SynchronizeRecord(*pRemRecord, *plocRecord, *pbackRecord);
			}
			else
				retval = CONDERR_CONVERT_FROM_REMOTE_REC; 
		}
		rawRecIx++;
	}
	if (err != SYNCERR_FILE_NOT_FOUND)
			LogBadReadRecord(err);

	// Free the memory allocated for the records
	if (rawRecord.m_TotalBytes > 0 && rawRecord.m_pBytes)
		delete rawRecord.m_pBytes;

	if (pRemRecord)
		delete pRemRecord;
	if (plocRecord)
		delete plocRecord;
	if (pbackRecord)
		delete pbackRecord;

	// Send all modified records to the HH. Give in a specific record object. 
	if (!retval)
	{
		CBaseRecord* ploc2Rec = NULL;
		if (retval = ConstructRecord(ploc2Rec, *m_LocRealTable, 0))
			return (retval);
		retval = SendRemoteChanges(*ploc2Rec);
		if (ploc2Rec)
			delete ploc2Rec;
	}
	return(retval);
}


///////////////////////////////////////////////////////////////////////////
// CBaseConduitMonitor 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		ApplyRemotePositionMap()     ** virtual Protected ** 
//
//	Description:	Issues a SyncManager command to retrieve a list of the
//					record ids from the remote in the order that they are
//					sorted on the device.  This is needed because the sort
//					order on the device is not always predictable.			
//
//	Parameters:		None
//
//	Returns:		long	returns error code
//
///////////////////////////////////////////////////////////////////////////
long CBaseConduitMonitor::ApplyRemotePositionMap(void)
{
	return(0);
/*
	long 			retval=0, lErr, i=0;	
	CPositionInfo	posInfo;
	long 			*pLongs;
	long            numRecs = 0;

	memset(&posInfo, 0, sizeof(posInfo));

	// Alloc enough memory for all the record Id's
	if (m_LocRealTable) 
		numRecs = m_LocRealTable->GetRecordCount();

	if (numRecs > 0)
	{
		posInfo.m_TotalBytes = numRecs * sizeof(long);
		posInfo.m_pBytes     = (BYTE*) new char [posInfo.m_TotalBytes];
		posInfo.m_FileHandle = m_RemHandle;
		posInfo.m_MaxEntries = (WORD)numRecs;  

		// Read in the remote array of record Id's (motorola format)
		retval = SyncReadPositionXMap(posInfo);

		if (!retval)
		{
			if (!m_pDTConvert->ConvertPositionMap(posInfo))
			{
				pLongs = (long*)posInfo.m_pBytes;
	
				m_LocRealTable->ClearPlacementField();
	
				CBaseRecord     locRec(*m_LocRealTable);
				CBaseIterator   locIt(*m_LocRealTable);
				locIt.SortByRecordId();
		
				lErr = locIt.FindFirst(locRec, FALSE);
				while (!lErr && i < posInfo.m_NumReadIn)
				{
					if (!locIt.FindByRecordId(pLongs[i], locRec, FALSE))
					{
						locRec.SetPlacementPos(i);
					}
					i++;
				}
			}
		}
		if (posInfo.m_pBytes) 
			delete posInfo.m_pBytes;     

	}
	return(retval);
	*/
}


///////////////////////////////////////////////////////////////////////////
// CBaseConduitMonitor 
///////////////////////////////////////////////////////////////////////////
//
//	Function:      AuditAddToPC()     ** virtual Protected ** 
//
//	Description:   Optionally overridden by a derived monitor. Serves
//                 as a notification from the core synchronization logic
//                 that it is adding a new record from the HH to the PC.
//
//
//
//
//	Parameters:
//
//	Returns:		
//
///////////////////////////////////////////////////////////////////////////
long CBaseConduitMonitor::AuditAddToPC(CBaseRecord&, long) 
{
	return(0);
}


///////////////////////////////////////////////////////////////////////////
// CBaseConduitMonitor 
///////////////////////////////////////////////////////////////////////////
//
//	Function:      AuditUpdateToPC()     ** virtual Protected ** 
//
//	Description:   Optionally overridden by a derived monitor. Serves
//                 as a notification from the core synchronization logic
//                 that it is changing an existing record on the PC with
//                 the contents of the matching record from the HH. 
//
//
//
//
//	Parameters:
//
//	Returns:		
//
///////////////////////////////////////////////////////////////////////////
long CBaseConduitMonitor::AuditUpdateToPC(CBaseRecord&, long) 
{
	return(0);
}


///////////////////////////////////////////////////////////////////////////
// CBaseConduitMonitor 
///////////////////////////////////////////////////////////////////////////
//
//	Function:      AuditAddToHH()     ** virtual Protected ** 
//
//	Description:   Optionally overridden by a derived monitor. Serves
//                 as a notification from the core synchronization logic
//                 that it is adding a new PC record to the HH device. This
//                 may be interesting from the standpoint that the remote
//                 HH device will assign the new unique record number to 
//                 this PC record, which up untill it is sent over to the
//                 device had a value of zero. 
//
//
//
//	Parameters:
//
//	Returns:		
//
///////////////////////////////////////////////////////////////////////////
long CBaseConduitMonitor::AuditAddToHH(CBaseRecord&, long) 
{
	return(0);
}


///////////////////////////////////////////////////////////////////////////
// CBaseConduitMonitor 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		SynchronizeRecord()     ** Protected ** 
//
//	Description:	This routine takes a given HH record and synchronizes
//					it with the records in the m_LocRealTable.  It is used by
//					both FastSync and SlowSync.  The following describes the
//					actions taken during synchronization:
//
//	HH					PC					ACTION												
//  --------------------------------------------------------------------------------
//  Add					No Record		Add the HH record to the PC						
//	No Record			Add				Add the PC record to the HH		
//	Delete				No Change		Delete record on HH and PC											
//	No Change			Delete			Delete record on HH and PC
//	Delete				Change			Instead of deleting the HH record,							
//										replace the HH record with the PC record
//	Change				Delete			Instead of deleting the PC record,
//										replace the PC record with the HH record
//	Change				No Change		Replace the PC record with the HH record
//	No Change			Change			Replace the HH record with the PC record
//	Change				Change			If changes are identical, no action
//	Change				Change			If different, add HH record to the PC
//										and add PC record to the HH
//	Archive				No Record		Archive HH record
//	Archive				No Change		Archive HH record; Delete both records
//	Archive				Delete			Archive HH record; Delete both records
//	Archive&No Change	Change			Instead of archiving the HH record,
//										replace the HH record with the PC record
//	Archive&Change		Change			If changes are identical, archive the HH
//										record and delete both records
//	Archive&Change		Change			If different, then HH record will not
//										be archived and the PC record will be
//										added to the HH and the HH record will 
//										be added to the PC
//
//	No Record			Archive			Archive PC record
//	No Change			Archive			Archive PC record; Delete both records
//	Delete				Archive			Archive PC record; Delete both records
//	Change				Archive&No ChangeInstead of archiving the PC record,
//										replace the PC record with the HH record
//	Change				Archive&Change	If changes are identical, archive the HH
//										record and delete both records
//	Change				Archive&Change	If different, then PC record will not
//									    be archived and the PC record will be
//									    added to the HH, the HH record will be
//									    added to the PC
//
//	Parameters:		CBaseRecord& rRemRecord	- the remote (HH) record
//					CBaseRecord& rLocRecord	- the local (PC) record
//					CBaseRecord& rBackRecord- the backup (PC) record
//
//	Returns:		long	error code
//
///////////////////////////////////////////////////////////////////////////
long CBaseConduitMonitor::SynchronizeRecord(CBaseRecord& rRemRecord, 
                                            CBaseRecord& rLocRecord, 
                                            CBaseRecord& rBackRecord) 
{
	long 		recOffset, retval = 0;
	int			recId;
	CString		str1;

	CBaseIterator   locIterator(*m_LocRealTable);
	CBaseIterator   backupIterator(*m_BackupTable);

	// Sort tables by recordID
	if (m_LocRealTable->GetRecordCount())
		retval = locIterator.SortByRecordId();
	if (!retval && m_BackupTable->GetRecordCount())
		retval = backupIterator.SortByRecordId();
	
	if (retval)
		return(retval);

	if (rRemRecord.IsArchived())
	{
		rRemRecord.GetRecordId(recId);

		// if record does not exist on the PC
		if (locIterator.FindByRecordId(recId, rLocRecord, FALSE) != 0)
		{
			// Add the HH record to the archive table
			retval = ClearStatusAddRecord(rRemRecord, *m_LocArchTable);
		}

		else // record exists on the PC
		{
			if (!rLocRecord.IsModified()) // record has not changed				
			{
				// Add the HH record to the archive table
				retval = ClearStatusAddRecord(rRemRecord, *m_LocArchTable);

				// Mark as already deleted from the HH and needs to
				// be deleted from the PC
				rLocRecord.SetStatus(fldStatusPENDING);
				rLocRecord.SetArchiveBit(FALSE);
			}

			else // PC record has changed
			{
				// Only look at backup record during a slow sync
				if (rRemRecord.IsNone() && 	// HH rec = No change
					m_rSyncProperties.m_SyncType == eSlow) 
				{
					// Compare HH record to backup file and mark HH
					// record as modified if records are not identical
					rRemRecord.GetRecordId(recId);
								
					// if record does not exist in the backup table
					if (backupIterator.FindByRecordId(recId, rBackRecord, FALSE) != 0)
					{
						rRemRecord.SetStatus(fldStatusUPDATE);
						// subscription log , newly added record
						LogModifiedSubscRec( &rRemRecord, FALSE);
					}
							
					// if HH record is different from backup record
					else if (!(rRemRecord == rBackRecord))
					{
						rRemRecord.SetStatus(fldStatusUPDATE);
						// subscription log, changed since last sync with this PC
						LogModifiedSubscRec( &rRemRecord, FALSE);
					}
				}
					
				if (rRemRecord.IsModified())  // Both records have changed
				{
					if (rRemRecord == rLocRecord) // records are identical
					{
						// Add the HH record to the archive table
						retval = ClearStatusAddRecord(rRemRecord, *m_LocArchTable);

						// Mark as already deleted from the HH and needs
						// to be deleted from the PC
						rLocRecord.SetStatus(fldStatusPENDING);
						rLocRecord.SetArchiveBit(FALSE);
					}
					else // records are different
					{
						// Change the PC record to a new record so
						// that it gets added to the HH
						rLocRecord.SetRecordId(0);
#ifdef _FILELNK
						rLocRecord.SetStatus(fldStatusUPDATE |fldStatusADD);
#else
						rLocRecord.SetStatus(fldStatusADD);
#endif
						rLocRecord.SetArchiveBit(FALSE);

						// Disallow the original Archive action, keep HH record.
						rRemRecord.SetStatus(fldStatusUPDATE);
						rRemRecord.SetArchiveBit(FALSE);

						// Add the HH record to the PC table
						if (retval = AddRecord(rRemRecord, *m_LocRealTable))
							LogBadLocalAdd(rRemRecord); 

						// Double Modify - Report to log
						LogArchiveDoubleModify(rRemRecord);
						// subscription log  (case 9a)
						LogModifiedSubscRec(&rRemRecord, FALSE);
					}
				}
				else
				{
					// HH = Archive and PC = Modified causes HH record to be
					// updated not archived - Report to log
					LogReverseDelete(rLocRecord);
					// subscription log  (case 8)
					LogModifiedSubscRec( &rLocRecord, TRUE);
				}
			}
		}
	}

	else if (rRemRecord.IsDeleted())
	{
		// if record exists on the PC
		rRemRecord.GetRecordId(recId);
		if (locIterator.FindByRecordId(recId, rLocRecord, FALSE) == 0)
		{
			if (rLocRecord.IsArchived())
			{
				// Add the PC record to the archive table
				retval = ClearStatusAddRecord(rLocRecord, *m_LocArchTable);

				// Mark as already deleted from the HH and needs 
				// to be deleted from the PC
				rLocRecord.SetStatus(fldStatusPENDING);
			}
			else if (rLocRecord.IsModified())
			{
				// HH = Delete and PC = Modified causes HH record to be updated not deleted
				// Report to Log
				LogReverseDelete(rLocRecord);
				// subscription log  (case 3)
				LogModifiedSubscRec(&rLocRecord, TRUE);
			}
			else if (!rLocRecord.IsModified())
				// Mark as already deleted from the HH and needs 
				// to be deleted from the PC
				rLocRecord.SetStatus(fldStatusPENDING);
		}
	}

	else // record = Pending, Modified, or None
	{
		// if record does not exist on the PC
		rRemRecord.GetRecordId(recId);				
		if (locIterator.FindByRecordId(recId, rLocRecord, FALSE) != 0)
		{
			// Add the HH record to the PC table
			retval = ClearStatusAddRecord(rRemRecord, *m_LocRealTable);
			// subscription log  (case 1)
			LogModifiedSubscRec( &rRemRecord, FALSE);

			// Call the Audit trail notification  mja
			if (!retval)
			{
				recOffset  = m_LocRealTable->GetRecordCount();
				recOffset -= 1;
				AuditAddToPC(rRemRecord, recOffset); 
			}
		}
		else // Record exists on both HH and PC
		{
			// HH record status can only be None during SlowSync
			// because only the modified records are retrieved
			// during a FastSync
			if (rRemRecord.IsNone())
			{
				// Compare HH record to backup file and mark HH
				// record as modified if records are not identical
				rRemRecord.GetRecordId(recId);
								
				// if record does not exist in the backup table
				if (backupIterator.FindByRecordId(recId, rBackRecord, FALSE) != 0)
				{
					rRemRecord.SetStatus(fldStatusUPDATE);
					// subscription log  (??)
					LogModifiedSubscRec( &rRemRecord, FALSE);
				}
				
				// if HH record is different from backup record
				else if (!(rRemRecord == rBackRecord))
				{
					rRemRecord.SetStatus(fldStatusUPDATE);
					// subscription log  (??)
					LogModifiedSubscRec( &rRemRecord, FALSE);
				}
			}
				
			if (rRemRecord.IsModified()) // HH record has been changed
			{
				LogModifiedSubscRec( &rRemRecord, FALSE);
				if (!rLocRecord.IsModified()) // PC record has not changed
				{
					if (rLocRecord.IsDeleted() || rLocRecord.IsArchived())
					{
						// HH = Modified and PC = Deleted/Archived causes PC record to be 
						// updated not deleted - Report to Log
						LogReverseDelete(rRemRecord);
						// subscription log  (case 4, 5, 10)
						LogModifiedSubscRec( &rRemRecord, FALSE);
					}

					// Clear the HH status flag
					rRemRecord.SetStatus(fldStatusNONE);
				
					// Replace the PC record with the HH record
					retval = rLocRecord.Assign(rRemRecord);

					// Call the Audit trail notification  mja
					if (!retval)
					{
						if (!locIterator.GetCurrentRowPosition(recOffset)) 
							AuditUpdateToPC(rLocRecord, recOffset); 
					}
				}
				else // both records have changed
				{
					if (rRemRecord == rLocRecord) // records are identical
					{
						if (rLocRecord.IsArchived())
						{
							// Add the PC record to the archive table
							retval = ClearStatusAddRecord(rLocRecord, *m_LocArchTable);
					
							// Mark for deletion of HH record
							rLocRecord.SetStatus(fldStatusDELETE);
						}
						else if (!rLocRecord.IsDeleted())
						{
							rLocRecord.SetStatus(fldStatusNONE);
							// subscription log  (case 8)
							LogModifiedSubscRec( &rLocRecord, TRUE);
						}
					}
					else // records are different
					{
						// Change the PC record to a new record so
						// that it gets added to the HH
						rLocRecord.SetRecordId(0);
#ifdef _FILELNK
						rLocRecord.SetStatus(fldStatusUPDATE |fldStatusADD);
#else
						rLocRecord.SetStatus(fldStatusADD);
#endif
						rLocRecord.SetArchiveBit(FALSE);

						// Add the HH record to the PC table
						retval = ClearStatusAddRecord(rRemRecord, *m_LocRealTable);
						// subscription log  (case 7a, 11a)
						LogModifiedSubscRec( &rRemRecord, FALSE);
						
						// Call the Audit trail notification  mja
						if (!retval)
						{
							recOffset  = m_LocRealTable->GetRecordCount();
							recOffset -= 1;
							AuditAddToPC(rRemRecord, recOffset); 
						}
						// Double Modify - Report to log
						LogDoubleModify(rRemRecord);
					}
				}
			}
						
			else if (rRemRecord.IsNone())
			{
				if (rLocRecord.IsArchived())
				{
					// Add the PC record to the archive table
					retval = ClearStatusAddRecord(rLocRecord, *m_LocArchTable);
							
					// Set status to delete, so HH record will be deleted
					rLocRecord.SetStatus(fldStatusDELETE);
				}
				else if (rLocRecord.IsPending())
					rLocRecord.SetStatus(fldStatusNONE);
			}
		}
	}
	return(retval);
}


///////////////////////////////////////////////////////////////////////////
// CBaseConduitMonitor 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		SendRemoteChanges()      ** Protected **     
//
//	Description:	Searches through the PC table and sends all records that
//					are marked as being modified (added, changed, or deleted),
//					to the HH so they can up updated on the remote handheld.
//					All records marked for deletion and archive are also
//					purged from the PC table and from the HH.
//
//	Parameters:		None
//
//	Returns:		long	error code
//
///////////////////////////////////////////////////////////////////////////
long CBaseConduitMonitor::SendRemoteChanges(CBaseRecord& rLocRecord)
{
	long			recOffset, remErr = 0;
	int				err = 0, recId = 0, oldStatus;
	CBaseIterator	locIterator(*m_LocRealTable);
	CRawRecordInfo  rawRecInfo;
	long            retval = 0;

	#ifdef _FILELNK
	int CatId;
	CCategory* pCat;
	#endif
	CString csCatName;
	CCategoryMgr* pCatMgr;
	pCatMgr = m_LocRealTable->GetCategoryManager();
	BOOL bUserAdded = FALSE;

	memset(&rawRecInfo, 0, sizeof(rawRecInfo));
	rawRecInfo.m_FileHandle = m_RemHandle;

	// Sort records by status field (sort order = Archive, Pending,
	// Delete, Change, Add, None)
	if (m_LocRealTable->GetRecordCount() > 0){
		if (locIterator.SortByRecordStatus())
			return(CONDERR_REMOTE_CHANGES_NOT_SENT);
	}

	err = locIterator.FindFirst(rLocRecord, FALSE);
	while (!err && !retval)	{
		rawRecInfo.m_RecId = recId;
		if (rLocRecord.IsArchived()) {
			// Add the record to the archive table
			ClearStatusAddRecord(rLocRecord, *m_LocArchTable); // took retval

			// Delete the record from the HH
			rLocRecord.GetRecordId(recId);
			rawRecInfo.m_RecId = recId;
            if (recId != 0 && (remErr = SyncDeleteRec(rawRecInfo))) {
                if (IsCommsError(remErr)) {
				    LogBadDeleteRecord(rLocRecord, remErr); 
					retval = remErr;
                }
			}
			// Mark record for deletion from the PC table
			rLocRecord.SetStatus(fldStatusDELETE);
        } else if (rLocRecord.IsDeleted()) {
			// Delete the record from the HH
			rLocRecord.GetRecordId(recId);
			rawRecInfo.m_RecId = recId;
			if (recId != 0 && (remErr = SyncDeleteRec(rawRecInfo))) {
                if (IsCommsError(remErr)) {
				    LogBadDeleteRecord(rLocRecord, remErr); 
					retval = remErr;
                }
            }
        } else if (rLocRecord.IsPending()) {
			// Mark record for deletion from the PC table
			rLocRecord.SetStatus(fldStatusDELETE);
        } else // status flag != Pending, Archive, or Delete
			break;

		err   = locIterator.FindNext(rLocRecord, FALSE);
		recId = 0;
	}
	
	// Delete all records marked for deletion on the handheld
	if (!retval)
	{
		if (retval = SyncPurgeDeletedRecs(m_RemHandle))
			LogBadPurge(retval); 
	}

	while (!err && !retval)
	{
		if (rLocRecord.IsAdded())
		{	
			// Add the record to the HH by virtual worker function.
			// If successfull clear the status flag.
			rLocRecord.GetStatus(oldStatus);
			rLocRecord.SetStatus(fldStatusNONE);
			if ((remErr = AddRemoteRecord(rLocRecord)))
			{
				rLocRecord.SetStatus(oldStatus);
				LogBadAddRecord(rLocRecord, remErr); 
				retval = remErr;
			}
			else
			{
				// Call the Audit trail notification  mja
				if (!locIterator.GetCurrentRowPosition(recOffset)) 
					AuditAddToHH(rLocRecord, recOffset); 
				// subscription log  (case 2, 7b, 9b, 11b )
				if (bUserAdded)
					LogModifiedSubscRec( &rLocRecord, TRUE);
			}
		}
		else if (rLocRecord.IsModified())
		{
			// Change the record on the HH by virtual worker function.
			// Clear the status flag
			rLocRecord.GetStatus(oldStatus);
			rLocRecord.SetStatus(fldStatusNONE);
			if ((remErr = ChangeRemoteRecord(rLocRecord)))
			{
				rLocRecord.SetStatus(oldStatus);
				LogBadChangeRecord(rLocRecord, remErr); 
				if (remErr && !IsRemoteMemError(remErr))
					retval = remErr;
			}
			else
				// subscription log  (case 6)
				LogModifiedSubscRec( &rLocRecord, TRUE);
		}
		else // status flag == NONE, so done
			break;

		err = locIterator.FindNext(rLocRecord, FALSE);
	}

	// Remove all PC records marked for deletion
	// if (!retval)
	PurgeLocalDeletedRecs();

	return(retval);
}



///////////////////////////////////////////////////////////////////////////
// CBaseConduitMonitor 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		IsRemoteMemError()      ** Protected **     
//
//	Description:	
//				
//			
//		
//
//	Parameters:		None
//
//	Returns:
//    TRUE   - passed in error value represents error problem on HH
//   FALSE   - not an error problem
//
///////////////////////////////////////////////////////////////////////////
BOOL CBaseConduitMonitor::IsRemoteMemError(long lRemoteErr)  
{
	BOOL bRetval = FALSE;

	if (lRemoteErr == SYNCERR_REMOTE_MEM || 
	    lRemoteErr == SYNCERR_REMOTE_NO_SPACE)
	{
		bRetval = TRUE;
	}	

	return(bRetval);
}

///////////////////////////////////////////////////////////////////////////
// CBaseConduitMonitor 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		IsCommsError()      ** Protected **     
//
//	Description:	
//				
//			
//		
//
//	Parameters:		None
//
//	Returns:
//    TRUE   - passed in error value represents error problem on HH
//   FALSE   - not an error problem
//
///////////////////////////////////////////////////////////////////////////
BOOL CBaseConduitMonitor::IsCommsError(long lErr)  
{
	BOOL bRetval = FALSE;

	if ((lErr != -1) && (lErr & 0x00002000))
	{
		bRetval = TRUE;

		char buff[64];
		sprintf(buff, "::IsCommsError(0x%x)\n", lErr);
		TRACE(buff);

	}

	return(bRetval);
}


///////////////////////////////////////////////////////////////////////////
// CBaseConduitMonitor 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		ObtainLocalTables()      ** Protected **     
//
//	Description:	Creates a new local (PC) table and an archive table.
//					If the syncType (obtained from m_rSyncProperties) is
//					defined as ePCtoHH, eFastSync, or eSlowSync, then
//					the PC records are all read into m_LocRealTable.
//
//	Parameters:		None
//
//	Returns:		long	error code
//
///////////////////////////////////////////////////////////////////////////
long CBaseConduitMonitor::ObtainLocalTables(void) 
{
	long retval = 0;
	long lTblErr = 0;

	CString dataFile(m_rSyncProperties.m_PathName);
	dataFile += m_rSyncProperties.m_LocalName;

	// Create our local table object and open it's disk file. 

	if (!(lTblErr = CreateTable(m_LocRealTable))) 
	{
	 	if (m_rSyncProperties.m_SyncType != eHHtoPC)
		{
			lTblErr = m_LocRealTable->OpenFrom(dataFile, 0);
            
            // if you remove your local database and then do a sync.
            // you would expect the data from the hh to be copied to the
            // desktop. In the old code, the category and preferences would
            // be lost if it continued in slow/fast mode.
            if (lTblErr) {
                if (lTblErr == DERR_FILE_NOT_FOUND) {
                    if ((m_rSyncProperties.m_SyncType == eFast) ||
                        (m_rSyncProperties.m_SyncType == eSlow)) {
                        m_rSyncProperties.m_SyncType = eHHtoPC;
                    }

                } else {
                    retval = CONDERR_BAD_LOCAL_TABLES;
                }
            }
		}
	}

	// Create our local archive table object
	if (!retval)
		lTblErr = CreateTable(m_LocArchTable);

	if (!lTblErr)
		// Set Archive File Extention
		SetArchiveFileExt();
	else	
		retval = CONDERR_BAD_LOCAL_TABLES;

	if(!retval)
		// Create Backup table object
		if (m_rSyncProperties.m_SyncType == eFast || m_rSyncProperties.m_SyncType == eSlow)
		{
			lTblErr = CreateTable(m_BackupTable);
			if (lTblErr)
				retval = CONDERR_BAD_LOCAL_TABLES;
		}
	

	return(retval);
}



///////////////////////////////////////////////////////////////////////////
// CBaseConduitMonitor 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		SaveLocalTables()      ** Protected **     
//
//	Description:	Saves the local (PC) table.
//
//	Parameters:		None
//
//	Returns:		long	error code
//
///////////////////////////////////////////////////////////////////////////
long CBaseConduitMonitor::SaveLocalTables(const char* fileName) 
{
	CString destFile(fileName);

	long retval = m_LocRealTable->SaveTo(destFile);
	if (retval)
		LogBadLocalSave();

	return(retval);
}


///////////////////////////////////////////////////////////////////////////
// CBaseConduitMonitor 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		PurgeLocalDeletedRecs()      ** virtual Protected **     
//
//	Description:	Purges records marked for delection from the local
//					real table. This routine also offers the opportunity
//					for derived classes to perform their own post processing
//					on the local table if they desire to.
//
//	Parameters:		None
//
//	Returns:		long	error code
//
///////////////////////////////////////////////////////////////////////////
long CBaseConduitMonitor::PurgeLocalDeletedRecs()
{
	long retval = m_LocRealTable->PurgeDeletedRecords();
	return(retval);
}


///////////////////////////////////////////////////////////////////////////
// CBaseConduitMonitor 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		FlushPCRecordIDs()           ** virtual Protected **     
//
//	Description:    Detects if the Pilot HH has been rebooted between syncs
//                 then loops through the local table object and zeros
//                 out the record IDs. This is necessary becuase the HH
//                 does not in fact generate a unique record ID seed inside
//                 the dataBase headers. It generates the *same* seed on
//                 each re-boot. This could cause havok if the HH gets
//                 re-booted between synchronization sessions, as the HH
//                 will begin numbering it's records with a duplicate
//                 numeric sequence.  
//
//					** The above hack only applies to 1.0 Pilots **
//
//					This call will be ignored if we attempt it while
//					syncing with a version 2.x or above Pilot.
//					
//					
//
//	Parameters:		None
//
//	Returns:		long	error code
//
///////////////////////////////////////////////////////////////////////////
long CBaseConduitMonitor::FlushPCRecordIDs()
{
	long retval = 0;
	int	 vRomMajor;

	vRomMajor = ROMVMAJOR(m_SystemInfo.m_RomSoftVersion);

	if ((m_rSyncProperties.m_FirstDevice == eHH && m_LocRealTable)) 
	{
        CBaseRecord aRec(*m_LocRealTable, 0);  // Turn off auto modify
        CBaseIterator it(*m_LocRealTable);

        long lErr = it.FindFirst(aRec, FALSE);
        while (!lErr)
        {
			if(vRomMajor < ROM_VER_2)
	            aRec.SetRecordId(0);

            if (!aRec.IsDeleted()) 
				aRec.SetStatus(fldStatusADD);

            lErr = it.FindNext(aRec, FALSE);
        }
	}
	return(retval);
}





///////////////////////////////////////////////////////////////////////////
// CBaseConduitMonitor 
///////////////////////////////////////////////////////////////////////////
//
//	Function:      GetLocalRecordCount()      ** virtual Protected **     
// 
//
//	Description:  Ask the m_LocRealTable object for the number of records
//                contained in it that are *not* marked for deletion.
//			
//                Derivative objects will want to override this routine
//                if they have also overridden the PurgeLocalDeletedRecs()
//                member function to diffuse it. This is because they are
//                likely to want the deleted records around for some form
//                of post processing logic. Therefore records 'marked' for
//                deletion may still be living in the table object, and this
//                method wants the *valid* record count to compare it against
//                the remote hand held data base in order to make a decision
//                about automatically copying records to the machine with the
//                least amount of records.
//					
//			
//
//	Parameters:		None
//
//	Returns:
//     long - Number of valid (non-deleted) records residing in m_LocRealTable	
//
///////////////////////////////////////////////////////////////////////////
long CBaseConduitMonitor::GetLocalRecordCount(void)
{
	long validRecs = 0; 

	if (m_LocRealTable)
		validRecs = m_LocRealTable->GetRecordCount();

	return(validRecs);
}



///////////////////////////////////////////////////////////////////////////
// CBaseConduitMonitor 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		ObtainRemoteTables()      ** Protected **     
//
//	Description:	Open the remote Database.
//
//	Parameters:		None
//
//	Returns:		long	error code
//
///////////////////////////////////////////////////////////////////////////
long CBaseConduitMonitor::ObtainRemoteTables(void) 
{
	long retval = CONDERR_BAD_REMOTE_TABLES;

	// Call into SyncManager.DLL to open Remote data base
	retval = SyncOpenDB(m_rSyncProperties.m_RemoteName[m_CurrRemoteDB], 0, m_RemHandle);

	// Create the remote dataBase if it's not there (in 3 out of 4 syncs)
	if (retval == SYNCERR_FILE_NOT_FOUND && 
	    m_rSyncProperties.m_SyncType != eHHtoPC)
	{
		CDbCreateDB dbInfo;
		memset(&dbInfo, 0, sizeof(dbInfo));
		dbInfo.m_Creator  = m_rSyncProperties.m_Creator; 
		dbInfo.m_Flags    = eRecord; 
		dbInfo.m_CardNo   = (BYTE)m_rSyncProperties.m_CardNo; 
		dbInfo.m_Type     = m_rSyncProperties.m_DbType; 
		strcat(dbInfo.m_Name, m_rSyncProperties.m_RemoteName[m_CurrRemoteDB]);

		if (!(retval = SyncCreateDB(dbInfo)))
		{
			m_RemHandle = dbInfo.m_FileHandle; 
		}
	}

	// Need a martyr table to hold converted remote records
	if (!retval)
	{
		long lTblErr = CreateTable(m_RemRealTable);
		if (lTblErr)
		{
			SyncCloseDB(m_RemHandle);
			retval = CONDERR_BAD_REMOTE_TABLES;
		}
	}

	return(retval);
}



///////////////////////////////////////////////////////////////////////////
// CBaseConduitMonitor 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		ObtainRemoteCategories()      ** Protected **     
//
//	Description:	Obtains the categories from the HH, creates a category
//					object for each category and places them into the
//					remote category manager (m_RemCategory).
//
//					Call into the SyncManager DLL to read the DBAppInfoBlock, 
//					if successful call into the DTLinkConverter to convert 
//					the results into a known CCategory object.
//
//	Parameters:		None
//
//	Returns:		long	error code
//
///////////////////////////////////////////////////////////////////////////
long CBaseConduitMonitor::ObtainRemoteCategories(void)                 
{
	long 	retval = 0;

	if (m_RemCategory)
	{
		delete m_RemCategory;
		m_RemCategory = NULL;
	}
	m_RemCategory = new CCategoryMgr();

	//  This memory allocation is real and is needed to call into SyncManager.
	//  We just need to provide enough room for the largest appInfoBlock..
	if (m_DbGenInfo.m_pBytes)
	{
		delete m_DbGenInfo.m_pBytes;
		m_DbGenInfo.m_pBytes = NULL;
	}
	memset(&m_DbGenInfo, 0, sizeof(m_DbGenInfo));

	m_DbGenInfo.m_TotalBytes = APPINFOBLOCK_SIZE;
	m_DbGenInfo.m_pBytes     = (BYTE*) new char [m_DbGenInfo.m_TotalBytes];
	strcat(m_DbGenInfo.m_FileName, 
	       m_rSyncProperties.m_RemoteName[m_CurrRemoteDB]);

	if (!m_DbGenInfo.m_pBytes || !m_RemCategory) 
	{
		retval = CONDERR_LOCAL_MEMORY_ALLOC_FAILED;
	}
	else if (!(retval = SyncReadDBAppInfoBlock(m_RemHandle, m_DbGenInfo)))
	{
		if (m_RemCategory)
			retval = m_pDTConvert->ConvertFromRemoteCategories(m_DbGenInfo, 
			                                                   m_RemCategory);
	}
	return(retval);

}


///////////////////////////////////////////////////////////////////////////
// CBaseConduitMonitor 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		SaveRemoteCategories()      ** Protected **     
//
//	Description:	Saves the newly synced categories to the Handheld.
//                 
//	Parameters:		CCategoryMgr*	Object that contains the categories to
//									be saved on the HH.
//
//	Returns:		long	error code
//
///////////////////////////////////////////////////////////////////////////
long CBaseConduitMonitor::SaveRemoteCategories(CCategoryMgr* catMgr)                 
{
	long retval = 0;

	m_pDTConvert->ConvertToRemoteCategories(m_DbGenInfo, catMgr);
	 
	retval = SyncWriteDBAppInfoBlock(m_RemHandle, m_DbGenInfo);

	return(retval);
}


///////////////////////////////////////////////////////////////////////////
// CBaseConduitMonitor 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		SaveLocalCategories()      ** Protected **     
//
//	Description:	Saves the newly synced categories to the PC.
//
//	Parameters:		CCategoryMgr*	An object that contains the newly
//									synced categories to be saved to the PC.
//
//	Returns:		long	error code
//
///////////////////////////////////////////////////////////////////////////
long CBaseConduitMonitor::SaveLocalCategories(CCategoryMgr* catMgr)                 
{
	long		retval = 0;
	CCategory	*cat1;
	CCategory	*cat2;

	// Delete all local categories
	m_LocCategory->DeleteAllCategories();

	// Add synchronized categories
	catMgr->FindFirst(cat1);
	while (cat1 != NULL)
	{
		cat2 = new CCategory();
		cat2->SetID(cat1->GetID());
		cat2->SetIndex(cat1->GetIndex());
		cat2->SetName(cat1->GetName());
		cat2->SetNameDirty(FALSE);
		m_LocCategory->Add(cat2);
		catMgr->FindNext(cat1);
	}

	return(retval);
}

///////////////////////////////////////////////////////////////////////////
// CBaseConduitMonitor 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		SynchronizeSubscCategories()      ** Protected **     
//
//	Description:	Called after the HH and PC cats have been sync'd
//							Assumptions :  
//							Deleted on HH - CatName & CatID combination does not exist
//							Renamed on HH - CatName does not exist, CatID still exists
//							Merged on HH - CatName & CatID combination does not exist
//							Deletes subscriptions from the Subsc.dat
//							if the category has been deleted/ merged during 
//							category synchronization. 
//							Also handles renaming of categories on the HH.
//
//	Parameters:		CCategoryMgr*	An object that contains the newly
//									synced categories.
//
//	Returns:		long	error code
//
///////////////////////////////////////////////////////////////////////////
long CBaseConduitMonitor::SynchronizeSubscCategories(CCategoryMgr* pCatMgr)
{
	long errRtn = 0;
	return (errRtn);
}

///////////////////////////////////////////////////////////////////////////
// CBaseConduitMonitor 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		SynchronizeCategories()      ** Protected **     
//
//	Description:	Synchronizes the local and remote categories.
//					During a fast or slow sync, synchronizing the categories
//					works as follows: 
//
//						1) Each PC category is compared against
//						   the list of HH (and merged PC categories). 
//
//						2) If the PC catName exists in the HH list and the 
//						   catIndexes are the same, nothing is added/changed. 
//
//						3) If the PC catName exists in the HH list and the 
//						   catIndexes are different, all of the PC records 
//						   with the old PC catID are changed to the catID from
//						   the HH. 
//
//						4) If the catName does not exist but the catID does 
//						   exist and the dirty flag is set for the PC category 
//						   (the category was renamed), then the HH catName is
//					       replaced by the PC catName. 
//
//						5) If the catName does not exist and the catID does not 
//						   exist and the PC catIndex is not already being used, 
//						   then the PC category is added to the HH list (the 
//						   merged list). 
//
//						6) If the catName does not exist and the catID does not 
//						   exist and the PC catIndex is already being used, then 
//						   the PC category is added with the next free HH 
//						   catIndex and the PC records with the old	catIndex 
//						   are changed to the new catIndex.
//
//	Parameters:		none
//
//	Returns:		long	error code
//
///////////////////////////////////////////////////////////////////////////
long CBaseConduitMonitor::SynchronizeCategories(void)
{
	CCategory			*hhCat, *pcCat, *newCat, *oldhhCat;
	CUpdateCategoryId	*updCatId;
	int 				nextIndex;
	long				retval = 0;
	char				strData[MAX_LOG_STRING];

	// Set m_LocCategory to PC Categories
	if (m_LocRealTable)
		m_LocCategory = m_LocRealTable->GetCategoryManager();

	if (m_rSyncProperties.m_SyncType == eHHtoPC)
	{
		// Copy HH categories to PC
		retval = SaveLocalCategories(m_RemCategory); 
    } else if (m_rSyncProperties.m_SyncType == ePCtoHH) {
		// Copy the PC categories to the HH
		retval = SaveRemoteCategories(m_LocCategory);
    } else {// FastSync or SlowSync
	
		// If the PC is a virgin, then set all the HH category flags so
		// those HH categories get added and not deleted
        if (m_rSyncProperties.m_FirstDevice == ePC) {
			SetDirtyCategoryFlags(m_RemCategory);
		
		// Profile scenario.  The HH is has been profiled, and is treated
		// as a new device, but a slow sync was called for since the profile
		// acts as if the HH was synced with another PC.
        } else if ((m_rSyncProperties.m_FirstDevice == eHH) 
			  && (m_rSyncProperties.m_SyncType == eSlow) ) {
			SetDirtyCategoryFlags(m_LocCategory);
			SetDirtyCategoryFlags(m_RemCategory);
		// If the HH is a virgin, then set all the PC category flags so
		// those PC categories get added and not deleted
        } else if (m_rSyncProperties.m_FirstDevice == eHH) {
			SetDirtyCategoryFlags(m_LocCategory);
		
		// If this is a slow sync that means that the last sync was with
		// a different PC, so set the category dirty flags on the HH
		// because they were cleared in the last sync.
        } else if (m_rSyncProperties.m_SyncType == eSlow) {
			SetDirtyCategoryFlags(m_RemCategory);
        }
			
		// First search through the HH categories and delete the categories that
		// were deleted form the PC and need to be deleted from the HH.
		// A category needs to be deleted when the category exists on one side
		// and not the other and the dirty flag is off.
		m_RemCategory->FindFirst(hhCat);
		while (hhCat != NULL) {
			if (hhCat->GetNameDirty() == FALSE)	{
				m_LocCategory->FindName(hhCat->GetName(), pcCat);
				if ( (pcCat == NULL) && (m_rSyncProperties.m_FirstDevice != eHH) ) {	
					// HH catName does not exist on PC and the HH is not a new device
					if (m_LocCategory->FindID(hhCat->GetID(), pcCat) == catNotFound) {
						// The category has been deleted on the PC, so needs 
						// to be deleted on the HH.  Also, change all the 
						// records under that category to be Unfiled.
						retval = SyncChangeCategory(m_RemHandle, 
						                            hhCat->GetIndex(), 0);

						// Move records to unfiled - Report to Log
						if (! retval) {
							CString catName = hhCat->GetName();
							LogCategoryDeleted(catName.GetBuffer(catName.GetLength()), HH_NAME);
							catName.ReleaseBuffer(-1);
								// Data Subsc :  for subsc cats, sub info has already been removed
							m_RemCategory->Delete(hhCat->GetIndex());
                        } else {
							CString catName = hhCat->GetName();
							LogBadChangeCategory(catName.GetBuffer(catName.GetLength()));
						}
					}
				}				
			}

// ** We have no way of tracking a deleted category on the desktop side, so we
//	  are unable to tell whether a category on the handheld was deleted from the 
//	  desktop side under a different name.  This is only a problem if you rename
//	  a category on the handheld and delete it on the desktop. 
			//
			// Adding to the above comment... The Remote catmgr is used as the base
			// and the desktop categories are added/deleted from the categories in the 
			// RemCatMgr. This ensures that the renamed HH cat which was deleted 
			// from the desktop is retained. 
            // However all the records are moved to 'Unfiled' or lost, depending on the delete option
			//}
			m_RemCategory->FindNext(hhCat);
		}
		
		updCatId = new CUpdateCategoryId;

		// Merge the PC categories into the HH categories
		m_LocCategory->FindFirst(pcCat);
		while (pcCat != NULL) {
			m_RemCategory->FindName(pcCat->GetName(), hhCat);
            if (hhCat != NULL) {// PC catName exists on HH
			
				// catIndexes are different
				if (pcCat->GetIndex() != hhCat->GetIndex()) {
					// Check for an index on the HH in case this was a rename
                    if( m_RemCategory->FindID(pcCat->GetID(), oldhhCat) != catNotFound) {
						retval = SyncChangeCategory(m_RemHandle, 
											oldhhCat->GetIndex(), hhCat->GetIndex());

						// Move records - Report to Log
						if (! retval) {
							CString catName = oldhhCat->GetName();
							LogCategoryDeleted(catName.GetBuffer(catName.GetLength()), HH_NAME);
							catName.ReleaseBuffer(-1);

							// Delete the old category
							m_RemCategory->Delete(oldhhCat->GetIndex());
                        } else {
							CString catName = oldhhCat->GetName();
							LogBadChangeCategory(catName.GetBuffer(catName.GetLength()));
						}


						// Delete the old category
						m_RemCategory->Delete(oldhhCat->GetIndex());
					}

					// Change the PC catIndex
					updCatId->Add(pcCat->GetIndex(), hhCat->GetIndex());
				}
            } else {// PC catName does not exist on HH
				// if PC catID exists on HH
				if (m_RemCategory->FindID(pcCat->GetID(), hhCat) != catNotFound) {
					// If PC category was renamed, then replace the HH
					// catName with the PC catName
					if (pcCat->GetNameDirty())
						hhCat->SetName(pcCat->GetName());

					// If the indexes are different, change the PC catIndex
					else if (pcCat->GetIndex() != hhCat->GetIndex())
						updCatId->Add(pcCat->GetIndex(), hhCat->GetIndex());
                } else {
					// If the category exists on the PC but not on the HH and
					// the dirty flag is not on, the category must have been
					// deleted on the HH, therefore needs to be deleted on PC.
					if (pcCat->GetNameDirty() == FALSE) {
						// Move all PC records under current category to unfiled
						updCatId->Add(pcCat->GetIndex(), 0);
   		
						// Move records to unfiled - Report to Log
						CString catName = pcCat->GetName();
						::LoadString(m_DllInstance, IDSTR_DESKTOP, strData, sizeof(strData));
						LogCategoryDeleted(catName.GetBuffer(catName.GetLength()), strData);
						catName.ReleaseBuffer(-1);
                    } else {// New category from PC
					
						m_RemCategory->FindIndex(pcCat->GetIndex(), hhCat);
						// PC catIndex does not exist on HH, so add PC category
						// as is to the HH Category Manager					
						if (hhCat == NULL)  {
							newCat = new CCategory;
							newCat->SetIndex(pcCat->GetIndex());
							newCat->SetID(pcCat->GetID());
							newCat->SetName(pcCat->GetName());
							m_RemCategory->Add(newCat);
                        } else {// PC catIndex already exists
						
							// Find the next available catIndex on the HH
							nextIndex = m_RemCategory->GetFreeIndex();
                            if (nextIndex == -1) {// Category Manager is Full
							
								// Set PC catIndex to unfiled (0)
								updCatId->Add(pcCat->GetIndex(), 0);
   		
								// Too many categories - Report to Log
								CString catName = pcCat->GetName();
								LogTooManyCats(catName.GetBuffer(catName.GetLength()));
								catName.ReleaseBuffer(-1);
                            } else {
								// Change PC catIndex to next available HH catIndex
								updCatId->Add(pcCat->GetIndex(), nextIndex);
							
								// Add the PC Category
								newCat = new CCategory;
								newCat->SetIndex(nextIndex);
								newCat->SetID(pcCat->GetID());
								newCat->SetName(pcCat->GetName());
								m_RemCategory->Add(newCat);
							}
						}
					}
				}
			}
			m_LocCategory->FindNext(pcCat);
		}
		// Change the category ID in some of the PC records
		UpdatePCCategories(updCatId); 
		delete updCatId;

		// Save synced categories to PC file
		retval = SaveLocalCategories(m_RemCategory); 
		if (!retval)
			// Write the synced categories to the HH 
			retval = SaveRemoteCategories(m_RemCategory); 
	}
	if (m_lFilelinkSupported != (-1))
		SynchronizeSubscCategories(m_RemCategory);
	//subError = SubSaveInfo(csSubInfoPath);
	return (retval);
}                 


///////////////////////////////////////////////////////////////////////////
// CBaseConduitMonitor 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		SetDirtyCategoryFlags()      ** Protected **     
//
//	Description:	Turn on all of the dirty category flags in the given
//					category manager.
//
//	Parameters:		CCategoryMgr* catMgr
//
//	Returns:		Nothing
//
///////////////////////////////////////////////////////////////////////////
void CBaseConduitMonitor::SetDirtyCategoryFlags(CCategoryMgr* catMgr)
{
	CCategory	*category;

	catMgr->FindFirst(category);
	while (category != NULL)
	{
		category->SetNameDirty(TRUE);
		catMgr->FindNext(category);
	}
}


///////////////////////////////////////////////////////////////////////////
// CBaseConduitMonitor 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		UpdatePCCategories()      ** Protected **     
//
//	Description:	Since some of the categories from the PC did not match the
//					categories from the handheld, some of the PC records will
//					have their category IDs changed.
//
//	Parameters:		CUpdateCategoryId*		Table that contains the old category
//											IDs and the new category IDs
//
//	Returns:		n/a
//
///////////////////////////////////////////////////////////////////////////
void CBaseConduitMonitor::UpdatePCCategories (CUpdateCategoryId *updCatId)
{
	int	err, catId, newCatId;
	CString  csLogFilePath(m_rSyncProperties.m_PathName);
	CBaseIterator locIterator(*m_LocRealTable);
	CBaseRecord locRecord(*m_LocRealTable, 0);

	if (updCatId->GetCount() == 0)
		return;

	// Search through all of the PC records
	err = locIterator.FindFirst(locRecord, FALSE);
	while (!err)
	{
		locRecord.GetCategoryId(catId);
		updCatId->FindByOldCatId(catId, newCatId);
		if (newCatId != -1)
		{
			locRecord.SetCategoryId(newCatId);
		}
		
		err = locIterator.FindNext(locRecord, FALSE);
	}
}


///////////////////////////////////////////////////////////////////////////
// CBaseConduitMonitor 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		AddRecord()      ** Protected **     
//
//	Description:	Takes a given record and adds it to a given table object.
//                 
//	Parameters:		CBaseRecord&	contains the record to be added
//					CBaseTable&		the table that the record will be added to
//
//	Returns:		long	error code
//
///////////////////////////////////////////////////////////////////////////
long CBaseConduitMonitor::AddRecord(CBaseRecord& rFromRec, CBaseTable& rTable)
{
	long	retval=0;
	long	lRecErr;
	
	CBaseRecord& rFromRecord = rFromRec;

	CBaseRecord* ptoRec = NULL;
	if (!(lRecErr = ConstructRecord(ptoRec, rTable, 0)))
	{
		if (rTable.AppendDuplicateRecord(rFromRecord, *ptoRec))
			retval = CONDERR_ADD_LOCAL_RECORD;
	}
	else retval = lRecErr;
	if (ptoRec)
		delete ptoRec;
	return (retval);
}




///////////////////////////////////////////////////////////////////////////
// CBaseConduitMonitor 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		AddRemoteRecord()      ** Protected **     
//
//	Description:	Interface definition allowing derivatives to override
//					and provide record specific actions.
//
//	Parameters:		CBaseRecord&	contains the record to be added
//
//	Returns:		long	error code
//
///////////////////////////////////////////////////////////////////////////
long CBaseConduitMonitor::AddRemoteRecord(CBaseRecord& rRec) 
{
	CRawRecordInfo rawRec;
	CBaseRecord &rLocRec = rRec;
	long retval  = CONDERR_ADD_REMOTE_RECORD;

	memset(&rawRec, 0, sizeof(rawRec));
	rawRec.m_FileHandle = m_RemHandle;  // remote file handle
	rawRec.m_RecId      = 0 ;           // remote HH will assign new RecId 

	// Allocate memory for rawRecord.m_pBytes
	if (!AllocateRawRecordMemory(rawRec, m_wRawRecSize))
	{
		// Convert record data for remote, upon return grab new RecordId.
		if (!(retval = m_pDTConvert->ConvertToRemote(rLocRec, rawRec)))
		{
			if (!(retval = SyncWriteRec(rawRec)))		
				rRec.SetRecordId(rawRec.m_RecId);
		}
		else
			retval = CONDERR_CONVERT_TO_REMOTE_REC;

		if (rawRec.m_TotalBytes > 0 && rawRec.m_pBytes)
			delete rawRec.m_pBytes;
	}
	return(retval);
}

///////////////////////////////////////////////////////////////////////////
// CBaseConduitMonitor 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		ChangeRemoteRecord()      ** Protected **     
//
//	Description:	Interface definition allowing derivatives to override
//					and provide record specific actions.
//
//	Parameters:		CBaseRecord&	record to be updated
//
//	Returns:		long	error code
//
///////////////////////////////////////////////////////////////////////////
long CBaseConduitMonitor::ChangeRemoteRecord(CBaseRecord& rRec) 
{
	long			retval = CONDERR_CHANGE_REMOTE_RECORD;
	int				locRecId;
	CRawRecordInfo	rawRec;
	CBaseRecord		&rLocRec = rRec;

	memset(&rawRec, 0, sizeof(rawRec));
	rLocRec.GetRecordId(locRecId);

	rawRec.m_FileHandle = m_RemHandle;      // remote file handle
	rawRec.m_RecId      = (DWORD)locRecId;  // remote HH record already exists 

	// Allocate memory for rawRecord.m_pBytes
	if (!AllocateRawRecordMemory(rawRec, m_wRawRecSize))
	{
		// Prepare record data for remote, upon return grab new RecordId.
		if (!m_pDTConvert->ConvertToRemote(rLocRec, rawRec))
			retval = SyncWriteRec(rawRec);
		else
			retval = CONDERR_CONVERT_TO_REMOTE_REC;

		if (rawRec.m_TotalBytes > 0 && rawRec.m_pBytes)
			delete rawRec.m_pBytes;
	}
	return(retval);
}



///////////////////////////////////////////////////////////////////////////
// CBaseConduitMonitor 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		ClearStatusAddRecord()      ** Protected **     
//
//	Description:	Sets the statusFlag = NONE and the archiveFlag = FALSE.
//					Then, calls AddRecord to add the given record to the
//					given table.
//
//  ** Note **      All derivative Monitor Objects *must* supply a specific
//                  version of the virtual routine 'AddRecord()' which is 
//                  indirectly called by this member function.
//                 
//	Parameters:		
//    CBaseRecord&  contains the record to be added
//    CBaseTable&   the table that the record will be added to
//
//	Returns:		long 	error code
//
///////////////////////////////////////////////////////////////////////////
long CBaseConduitMonitor::ClearStatusAddRecord(CBaseRecord& rFromRec, 
                                               CBaseTable&  rTable)
{
	long  retval = 0;

	// Clear the status flag
	rFromRec.SetStatus(fldStatusNONE);
	rFromRec.SetArchiveBit(FALSE);

	// Add the new record to the table
	if (AddRecord(rFromRec, rTable))
	{
		retval = CONDERR_ADD_LOCAL_RECORD;
		LogBadLocalAdd(rFromRec); 
	}

	return(retval);
}


///////////////////////////////////////////////////////////////////////////
// CBaseConduitMonitor 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		ArchiveRecords()      ** Protected **     
//
//	Description:	After all of the records have been synchronized, this
//					routine is called.  During synchronization, the records
//					that needed to be archived were added to the table object
//					(m_LocArchTable).  This routine first sorts this table
//					by categoryID.  Archive records are stored in different
//					archive files based on their category names.  So, this
//					routine will open the appropriate archive files and add
//					the new archive records to those files.
//
//	Parameters:		None
//
//	Returns:		long 	error code
//
///////////////////////////////////////////////////////////////////////////
long CBaseConduitMonitor::ArchiveRecords(void)
{
	long			retval = 0, err = 0;
	
	if (!m_LocArchTable || m_LocArchTable->GetRecordCount() <= 0)
		return(retval); 

	int				catIndex, lastCatIndex = -1;
	CBaseIterator	archIterator(*m_LocArchTable);
	CBaseRecord		archRecord(*m_LocArchTable);
	CBaseTable		*newArchTable = NULL;
	CCategory		*category = NULL;
	CString			pathName(m_rSyncProperties.m_PathName);
	CString			arcName;

	// Sort archive records by categoryID
	archIterator.SortByCatId();

	err = archIterator.FindFirst(archRecord, FALSE);
	while(!err && !retval)
	{
		archRecord.GetCategoryId(catIndex);
		
		// Prevent gp_faults
		if (catIndex < 0 || catIndex >= MAX_CATEGORIES)
			catIndex = 0;

		// if the catIndex is different from the last archive record
		if (lastCatIndex != catIndex)
		{
			if (lastCatIndex != -1 && newArchTable) // Not the first record
				retval = newArchTable->SaveTo(arcName);	
			
			if (m_LocCategory)
				m_LocCategory->FindIndex(catIndex, category);

			// Create the Archive file name
			if (category != NULL)
				arcName = pathName + category->GetFileName() + (CString)m_ArchFileExt;
			else
				arcName = pathName + (CString)"Unfiled" + (CString)m_ArchFileExt;
			
			if (newArchTable)
			{
				delete newArchTable;
				newArchTable = NULL;
			}
			// Call virtual to do specific work.
			if (!CreateTable(newArchTable))
			{
				// Read in new archive file
				retval = newArchTable->OpenFrom(arcName, 0);
				if (retval == DERR_FILE_NOT_FOUND)
					retval = 0;
				else
					newArchTable->PurgeDeletedRecords();
				
			}
			lastCatIndex = catIndex;
		}

		// Set the categoryId to Unfiled (0)
		archRecord.SetCategoryId(0);

		// Add the archive record to the table
		if (!retval)
			retval = AddRecord(archRecord, *newArchTable);

		err = archIterator.FindNext(archRecord, FALSE);
	}
	if (newArchTable)
	{
		if (!retval)
			retval = newArchTable->SaveTo(arcName);	
		delete newArchTable;
	}
	return(retval);
}


///////////////////////////////////////////////////////////////////////////
// CBaseConduitMonitor 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		CreateTable()      ** Protected **     
//
//	Description:	Create a specific table for the base function.
//					
//	Parameters:		CBaseTable*& - gets (Table object) memory alloced onto it
//
//	Returns:		long 	error code
//
///////////////////////////////////////////////////////////////////////////
long CBaseConduitMonitor::CreateTable(CBaseTable*&)
{
	return(-1);
}

///////////////////////////////////////////////////////////////////////////
// CBaseConduitMonitor 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		ConstructRecord()      ** Protected **     
//
//	Description:	Create a derived class record object to be used in base
//					class functions 
//
//	Parameters:
//			CBaseRecord* pBase - pointer to base class record that will 
//								 point to derived class object
//			CBaseTable& rtable - table associated with the record, reqd 
//								 for constructing CAddressRecord
//			WORD wModAction	   - reqd for constructing CAddressRecord
//
//	Returns:		long	error code
//
///////////////////////////////////////////////////////////////////////////
long CBaseConduitMonitor::ConstructRecord(CBaseRecord*& pBase, 
										  CBaseTable& rTable,
										  WORD wModAction)
{
	return(-1);
}


///////////////////////////////////////////////////////////////////////////
// CBaseConduitMonitor 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		SetArchiveFileExt()      ** Protected **     
//
//	Description:	set file extension depending on app type
//
//	Parameters:		none
//			
//	Returns:		void
//
///////////////////////////////////////////////////////////////////////////
void  CBaseConduitMonitor::SetArchiveFileExt()
{
}

///////////////////////////////////////////////////////////////////////////
// CBaseConduitMonitor 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		AllocateRawRecordMemory()      ** Protected **     
//
//	Description:    Memory needs to be allocated for the raw record before
//					the conversion routines (ConvertToRemote and
//					ConvertFromRemote) are used.
//
//	Parameters:		
//		CRawRecordInfo&	- the raw record that needs memory allocated to it
//		WORD			- number of bytes to allocate
//
//	Returns:		error code
//
///////////////////////////////////////////////////////////////////////////
long CBaseConduitMonitor::AllocateRawRecordMemory(CRawRecordInfo& rawRecord, WORD wRawRecSize)
{
	long retval = CONDERR_RAW_RECORD_ALLOCATE;

	// Allocate memory for rawRecord data
	rawRecord.m_TotalBytes = 0;
	rawRecord.m_pBytes = (BYTE*) new char [wRawRecSize];
	if (rawRecord.m_pBytes)
	{
		rawRecord.m_TotalBytes = wRawRecSize;
		memset(rawRecord.m_pBytes, 0, wRawRecSize);
		retval = CONDERR_NONE;
	}
	return(retval);	
}

///////////////////////////////////////////////////////////////////////////
// CBaseConduitMonitor 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		LogDoNothingr()      ** Protected **     
//
//	Description:    logs "AppName - sync configured to Do Nothing"
//
//	Parameters:		None
//		
//	Returns:		Nothing
//
///////////////////////////////////////////////////////////////////////////
void CBaseConduitMonitor::LogDoNothing(void)
{
	char appName[MAX_LOG_STRING];
	char buffer[MAX_LOG_STRING];

	// Get the application name
	memset(appName, 0, sizeof(appName));
	LogApplicationName(appName, sizeof(appName)); 

	// get Do Nothing string
	memset(buffer, 0, sizeof(buffer));
	// Load string from the resource file.
	::LoadString(m_DllInstance, IDSTR_SYNC_DONOTHING, 
		                        buffer, sizeof(buffer));
	
	strcat(appName, " - ");
	strcat(appName, buffer);
	LogAddEntry((const char*)appName,slText,FALSE);	         
	
}

///////////////////////////////////////////////////////////////////////////
// CBaseConduitMonitor 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		LogBanner()      ** Protected **     
//
//	Description:    If this is the first message to be sent to the log, then
//					print th application name first.
//
//	Parameters:		None
//		
//	Returns:		Nothing
//
///////////////////////////////////////////////////////////////////////////
void CBaseConduitMonitor::LogBanner(void)
{
	char appName[MAX_LOG_STRING];
	char buffer[MAX_LOG_STRING];

	if (LogTestCounters() == 0)
	{
		// Get the application name
		memset(appName, 0, sizeof(appName));
		LogApplicationName(appName, sizeof(appName)); 
		
		strcpy(buffer, "-- ");
		strcat(buffer, appName);
		LogAddEntry((const char*)buffer,slText,FALSE);	         
	}
}


///////////////////////////////////////////////////////////////////////////
// CBaseConduitMonitor 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		LogBadAddRecord()      ** Protected **     
//
//	Description:    Adds message to the log when record fails to be added to
//					the viewer.
//
//	Parameters:		CBaseRecord& - Needed for LogRecordData
//					long - error code
//		
//	Returns:		Nothing
//
///////////////////////////////////////////////////////////////////////////
void CBaseConduitMonitor::LogBadAddRecord(CBaseRecord &rRec, long lErr)
{
	char  errBuff[MAX_LOG_STRING];

	LogBanner();

	// Load string from the resource file.
	::LoadString(m_DllInstance, IDSTR_BAD_ADD_REC, errBuff, sizeof(errBuff));     
	LogAddEntry((const char*)errBuff, slRemoteAddFailed,FALSE);

	// Ask derivative to add field info..
	memset(errBuff, 0, sizeof(errBuff));
	LogRecordData(rRec, errBuff); 
	LogAddEntry((const char*)errBuff,slText,FALSE);	         

	int				vRomMajor;

	vRomMajor = ROMVMAJOR(m_SystemInfo.m_RomSoftVersion);
#define	ROM_VER_3			3

	// Pilot may be full message
    if (vRomMajor < ROM_VER_3)
	    ::LoadString(m_DllInstance, IDSTR_DEVICE_FULL, errBuff, sizeof(errBuff));
    else
	    ::LoadString(m_DllInstance, IDS_DEVICE_FULL_30, errBuff, sizeof(errBuff));

	LogAddEntry((const char*)errBuff,slText,FALSE);

#ifdef _DEBUG
	if (lErr > 0)
	{
		sprintf(errBuff, "     Error Code = %lx", lErr);
		LogAddEntry((const char*)errBuff,slText,FALSE);
	}
#endif 
}


///////////////////////////////////////////////////////////////////////////
// CBaseConduitMonitor 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		LogBadPurge()      ** Protected **     
//
//	Description:    Adds message to the log when purging the deleted/archived
//					records on the viewer fails.
//
//	Parameter:		long - error code
//		
//	Returns:		Nothing
//
///////////////////////////////////////////////////////////////////////////
void CBaseConduitMonitor::LogBadPurge(long lErr)
{
	char  errBuff[MAX_LOG_STRING];

	LogBanner();

	// Load string from the resource file.
	::LoadString(m_DllInstance, IDSTR_BAD_PURGE, 
		                        errBuff, sizeof(errBuff));     
	LogAddEntry((const char*)errBuff, slRemotePurgeFailed,FALSE);	         

	// May have lost connection message
	::LoadString(m_DllInstance, IDSTR_CONNECTION, errBuff, sizeof(errBuff));
	LogAddEntry((const char*)errBuff,slText,FALSE);

#ifdef _DEBUG
	if (lErr > 0)
	{
		sprintf(errBuff, "     Error Code = %lx", lErr);
		LogAddEntry((const char*)errBuff,slText,FALSE);
	}
#endif 
}


///////////////////////////////////////////////////////////////////////////
// CBaseConduitMonitor 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		LogBadDeleteRecord()      ** Protected **     
//
//	Description:    Adds message to the log when record fails to be deleted from
//					the viewer.
//
//	Parameters:		CBaseRecord& - Needed for LogRecordData
//					long - error code
//		
//	Returns:		Nothing
//
///////////////////////////////////////////////////////////////////////////
void CBaseConduitMonitor::LogBadDeleteRecord(CBaseRecord &rRec, long lErr)
{
	char  errBuff[MAX_LOG_STRING];

	LogBanner();

	// Load string from the resource file.
	::LoadString(m_DllInstance, IDSTR_BAD_DEL_REC, 
		                        errBuff, sizeof(errBuff));     
	LogAddEntry((const char*)errBuff,slRemoteDeleteFailed,FALSE);	         

	// Ask derivative to add field info..
	memset(errBuff, 0, sizeof(errBuff));
	LogRecordData(rRec, errBuff); 
	LogAddEntry((const char*)errBuff,slText,FALSE);	         

	// May have lost connection message
	::LoadString(m_DllInstance, IDSTR_CONNECTION, errBuff, sizeof(errBuff));
	LogAddEntry((const char*)errBuff,slText,FALSE);

#ifdef _DEBUG 
	if (lErr > 0)
	{
		sprintf(errBuff, "     Error Code = %lx", lErr);
		LogAddEntry((const char*)errBuff,slText,FALSE);
	}
#endif
}


///////////////////////////////////////////////////////////////////////////
// CBaseConduitMonitor 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		LogBadChangeRecord()      ** Protected **     
//
//	Description:    Adds message to the log when record fails to be modified on
//					the viewer.
//
//	Parameters:		CBaseRecord& - Needed for LogRecordData
//					long - error code
//		
//	Returns:		Nothing
//
///////////////////////////////////////////////////////////////////////////
void CBaseConduitMonitor::LogBadChangeRecord(CBaseRecord &rRec, long lErr)
{
	char  errBuff[MAX_LOG_STRING];

	LogBanner();

	// Load string from the resource file.
	::LoadString(m_DllInstance, IDSTR_BAD_CHANGE_REC, 
		                        errBuff, sizeof(errBuff));     
	LogAddEntry((const char*)errBuff, slRemoteChangeFailed,FALSE);	         
	
	// Ask derivative to add field info..
	memset(errBuff, 0, sizeof(errBuff));
	LogRecordData(rRec, errBuff); 
	LogAddEntry((const char*)errBuff,slText,FALSE);	         

	// May have lost connection message
	::LoadString(m_DllInstance, IDSTR_CONNECTION, errBuff, sizeof(errBuff));
	LogAddEntry((const char*)errBuff,slText,FALSE);

#ifdef _DEBUG
	if (lErr > 0)
	{
		sprintf(errBuff, "     Error Code = %lx", lErr);
		LogAddEntry((const char*)errBuff,slText,FALSE);
	}
#endif
}


///////////////////////////////////////////////////////////////////////////
// CBaseConduitMonitor 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		LogBadLocalAdd()      ** Protected **     
//
//	Description:    Adds message to the log when record fails to be added to
//					the PC table.
//
//	Parameters:		CBaseRecord& - Needed for LogRecordData
//		
//	Returns:		Nothing
//
///////////////////////////////////////////////////////////////////////////
void CBaseConduitMonitor::LogBadLocalAdd(CBaseRecord &rRec)
{
	char  errBuff[MAX_LOG_STRING];

	LogBanner();

	// Load string from the resource file.
	::LoadString(m_DllInstance, IDSTR_BAD_LOCALADD_REC, 
		                        errBuff, sizeof(errBuff));     
	LogAddEntry((const char*)errBuff, slLocalAddFailed,FALSE);	         

	// Ask derivative to add field info..
	memset(errBuff, 0, sizeof(errBuff));
	LogRecordData(rRec, errBuff); 
	LogAddEntry((const char*)errBuff,slText,FALSE);	         
}


///////////////////////////////////////////////////////////////////////////
// CBaseConduitMonitor 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		LogBadChangeCategory()      ** Protected **     
//
//	Description:	If a category is deleted from the PC and a sync is
//					performed, it should be deleted from the HH and all of
//					the records under that category on the HH should be
//					moved to 'Unfiled'.  There is a SyncManager call to
//					accomplish this task called 'SyncChangeCategory'.  If
//					SyncChangeCategory fails, this log message is printed.
//					
//	Parameters:		char* - Category Name
//		
//	Returns:		Nothing
//
///////////////////////////////////////////////////////////////////////////
void CBaseConduitMonitor::LogBadChangeCategory(char *catName) 
{
    CString csError;

	LogBanner();

    csError.GetBuffer(1024);
    csError.Format(IDS_BAD_CHANGE_CAT, catName);
	LogAddEntry((const char*)csError.GetBuffer(0), slChangeCatFailed,FALSE);

    // May have lost connection message
	csError.LoadString(IDSTR_CONNECTION);
	LogAddEntry((const char*)csError.GetBuffer(0),slText,FALSE);
}


///////////////////////////////////////////////////////////////////////////
// CBaseConduitMonitor 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		LogBadReadRecord()      ** Protected **     
//
//	Description:	Failed while trying to read a Pilot record.
//					
//	Parameters:		long - error code
//		
//	Returns:		Nothing
//
///////////////////////////////////////////////////////////////////////////
void CBaseConduitMonitor::LogBadReadRecord(long lErr) 
{
	char  errBuff[MAX_LOG_STRING];

	LogBanner();

	// Load string from the resource file.
	::LoadString(m_DllInstance, IDSTR_BAD_READ_RECORD, 
		                        errBuff, sizeof(errBuff));
	LogAddEntry((const char*)errBuff, slRemoteReadFailed,FALSE);	         

	// May have lost connection message
	::LoadString(m_DllInstance, IDSTR_CONNECTION, errBuff, sizeof(errBuff));
	LogAddEntry((const char*)errBuff,slText,FALSE);

#ifdef _DEBUG 
	if (lErr > 0)
	{
		sprintf(errBuff, "     Error Code = %lx", lErr);
		LogAddEntry((const char*)errBuff,slText,FALSE);
	}
#endif
}


///////////////////////////////////////////////////////////////////////////
// CBaseConduitMonitor 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		LogBadXMap()      ** Protected **     
//
//	Description:    Adds message to the log when ApplyRemotePositionMap()
//					fails.
//
//	Parameters:		None
//		
//	Returns:		Nothing
//
///////////////////////////////////////////////////////////////////////////
void CBaseConduitMonitor::LogBadXMap(void)
{
	char  errBuff[MAX_LOG_STRING];

	LogBanner();

	// Load string from the resource file.
	::LoadString(m_DllInstance, IDSTR_BAD_XMAP, errBuff, sizeof(errBuff));     
	LogAddEntry((const char*)errBuff, slXMapFailed,FALSE);	         

	// May have lost connection message
	::LoadString(m_DllInstance, IDSTR_CONNECTION, errBuff, sizeof(errBuff));
	LogAddEntry((const char*)errBuff,slText,FALSE);
}


///////////////////////////////////////////////////////////////////////////
// CBaseConduitMonitor 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		LogBadArchiveErr()      ** Protected **     
//
//	Description:    Adds message to the log when ArchiveRecords() fails.
//
//	Parameters:		None
//		
//	Returns:		Nothing
//
///////////////////////////////////////////////////////////////////////////
void CBaseConduitMonitor::LogBadArchiveErr(void)
{
	char  errBuff[MAX_LOG_STRING];

	LogBanner();

	// Load string from the resource file.
	::LoadString(m_DllInstance, IDSTR_BAD_ARCHIVE_ERR, errBuff, sizeof(errBuff));     
	LogAddEntry((const char*)errBuff, slArchiveFailed,FALSE);	         

	// Desktop may be full message
	::LoadString(m_DllInstance, IDSTR_DESKTOP_FULL, errBuff, sizeof(errBuff));
	LogAddEntry((const char*)errBuff,slText,FALSE);
}


///////////////////////////////////////////////////////////////////////////
// CBaseConduitMonitor 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		LogBadLocalSave()      ** Protected **
//
//	Description:    Adds message to the log when SaveTo() fails.
//
//	Parameters:		None
//		
//	Returns:		Nothing
//
///////////////////////////////////////////////////////////////////////////
void CBaseConduitMonitor::LogBadLocalSave(void)
{
	char  errBuff[MAX_LOG_STRING];

	LogBanner();

	// Load string from the resource file.
	::LoadString(m_DllInstance, IDSTR_BAD_LOCAL_SAVE, errBuff, sizeof(errBuff));     
	LogAddEntry((const char*)errBuff, slLocalSaveFailed,FALSE);

	// Desktop may be full message
	::LoadString(m_DllInstance, IDSTR_DESKTOP_FULL, errBuff, sizeof(errBuff));
	LogAddEntry((const char*)errBuff,slText,FALSE);
}


///////////////////////////////////////////////////////////////////////////
// CBaseConduitMonitor 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		LogBadResetSyncFlags()      ** Protected **     
//
//	Description:    Adds message to the log when SyncResetSyncFlags() fails.
//
//	Parameters:		None
//		
//	Returns:		Nothing
//
///////////////////////////////////////////////////////////////////////////
void CBaseConduitMonitor::LogBadResetSyncFlags(void)
{
#ifdef _DEBUG

		char  errBuff[MAX_LOG_STRING];

		LogBanner();

		// Load string from the resource file.
		::LoadString(m_DllInstance, IDSTR_BAD_RESET_FLAGS, errBuff, sizeof(errBuff));     
		LogAddEntry((const char*)errBuff, slResetFlagsFailed,FALSE);	         

		// May have lost connection message
		::LoadString(m_DllInstance, IDSTR_CONNECTION, errBuff, sizeof(errBuff));
		LogAddEntry((const char*)errBuff,slText,FALSE);
#endif
}


///////////////////////////////////////////////////////////////////////////
// CBaseConduitMonitor 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		LogTooManyCats()      ** Protected **     
//
//	Description:	Adds message to the log there are already 15 existing
//					categories and another is trying to be added.
//
//	Parameters:		char* catName - category name
//		
//	Returns:		Nothing
//
///////////////////////////////////////////////////////////////////////////
void CBaseConduitMonitor::LogTooManyCats(char* catName)
{
	char errBuff[MAX_LOG_STRING];

	LogBanner();

	// Load string from the resource file.
	::LoadString(m_DllInstance, IDSTR_TOOMANY_CATS1, 
		                        errBuff, sizeof(errBuff));     
	strcat(errBuff, " '");
	strcat(errBuff, catName);
	strcat(errBuff, "'");

	LogAddEntry((const char*)errBuff, slTooManyCategories,FALSE);

	// Load string from the resource file.
	::LoadString(m_DllInstance, IDSTR_TOOMANY_CATS2,
		                        errBuff, sizeof(errBuff));     

	LogAddEntry((const char*)errBuff,slText,FALSE);	         
}


///////////////////////////////////////////////////////////////////////////
// CBaseConduitMonitor 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		LogCategoryDeleted()      ** Protected **     
//
//	Description:	When a category has been deleted from one side, it needs
//					to be deleted from the other side during a sync.  This
//					message tells the user what happened.
//
//	Parameters:		char* catName - category name
//					char* deviceName - Desktop / Pilot
//		
//	Returns:		Nothing
//
///////////////////////////////////////////////////////////////////////////
void CBaseConduitMonitor::LogCategoryDeleted(char* catName, char* deviceName)
{
    CString csError;

	LogBanner();

    csError.GetBuffer(1024);
    csError.Format(IDS_CATEGORY_DEL, deviceName, catName);
	LogAddEntry((const char*)csError.GetBuffer(0), slCategoryDeleted,FALSE);
}


///////////////////////////////////////////////////////////////////////////
// CBaseConduitMonitor 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		LogDoubleModify()      ** Protected **     
//
//	Description:	Adds message to the log when the records have been modified
//					on the Pilot and the Desktop and the changes are different.
//
//	Parameters:		CBaseRecord& rRec - Needed for LogRecordData
//		
//	Returns:		Nothing
//
///////////////////////////////////////////////////////////////////////////
void CBaseConduitMonitor::LogDoubleModify(CBaseRecord &rRec)
{
	char  fieldInfo[MAX_LOG_STRING];

	LogBanner();

    CString csLog;
    csLog.GetBuffer(1024);
	// Ask derivative to add field info..
	memset(fieldInfo, 0, sizeof(fieldInfo));
	LogRecordData(rRec, fieldInfo); 
    csLog.Format(IDS_DOUBLE_MODIFY, fieldInfo);

	LogAddEntry((const char*)csLog.GetBuffer(0), slDoubleModify,FALSE);
}

///////////////////////////////////////////////////////////////////////////
// CBaseConduitMonitor 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		LogSubscribeDoubleModify()      ** Protected **     
//
//	Description:	Adds message to the log when a subscription
//							 record has been modified on the Pilot/ Desktop.
//
//	Parameters:		CBaseRecord& rRec - Needed for LogRecordData
//		
//	Returns:		Nothing
//
///////////////////////////////////////////////////////////////////////////
void CBaseConduitMonitor::LogSubscribeDoubleModify(CBaseRecord &rRec)
{
}


///////////////////////////////////////////////////////////////////////////
// CBaseConduitMonitor 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		LogFileLinkCompleted()      ** Protected **     
//
//	Description:	Adds message to the log when a File Link
//							 has been updated (or failed to update)
//
//	Parameters:		CString& csCatName
//                          CString& csFileName
//                          BOOL bSuccess - TRUE if update was successful, 
//                                                        else FALSE
//		
//	Returns:		Nothing
//
///////////////////////////////////////////////////////////////////////////
void CBaseConduitMonitor::LogFileLinkCompleted(CString& csCatName, CString& csFilePath, BOOL bSuccess)
{
}


///////////////////////////////////////////////////////////////////////////
// CBaseConduitMonitor 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		LogFileLinkDeleted()      ** Protected **     
//
//	Description:	Adds message to the log when a File Link
//							 has been deleted
//
//	Parameters:		CString& csOldName
//                          CString& csNewName
//		
//	Returns:		Nothing
//
///////////////////////////////////////////////////////////////////////////
void CBaseConduitMonitor::LogFileLinkDeleted(CString& csOldName, CString& csNewName)
{
}

///////////////////////////////////////////////////////////////////////////
// CBaseConduitMonitor 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		LogArchiveDoubleModify()      ** Protected **     
//
//	Description:	Adds message to the log when the records have been modified
//					on the viewer and the PC and the changes are different.
//					Also, one of the records was marked as archive, but the record
//					did not get archived.
//
//	Parameters:		CBaseRecord& rRec - Needed for LogRecordData
//		
//	Returns:		Nothing
//
///////////////////////////////////////////////////////////////////////////
void CBaseConduitMonitor::LogArchiveDoubleModify(CBaseRecord &rRec)
{
	char  fieldInfo[MAX_LOG_STRING];

	LogBanner();
    CString csLog;
    csLog.GetBuffer(1024);
	// Ask derivative to add field info..
	memset(fieldInfo, 0, sizeof(fieldInfo));
	LogRecordData(rRec, fieldInfo); 
    csLog.Format(IDS_ARCH_DOUBLE_MOD, fieldInfo);

	LogAddEntry((const char*)csLog.GetBuffer(0), slDoubleModifyArchive,FALSE);
}


///////////////////////////////////////////////////////////////////////////
// CBaseConduitMonitor 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		LogReverseDelete()      ** Protected **     
//
//	Description:	Adds message to the log when one record was marked as 
//                  archive/delete and the other record was marked as 
//                  update - the result is that
//					the record is updated not archived/deleted.
//
//	Parameters:		CBaseRecord& rRec - Needed for LogRecordData
//		
//	Returns:		Nothing
//
///////////////////////////////////////////////////////////////////////////
void CBaseConduitMonitor::LogReverseDelete(CBaseRecord &rRec)
{
	char  fieldInfo[MAX_LOG_STRING];

	LogBanner();
    CString csLog;
    csLog.GetBuffer(1024);

    // Ask derivative to add field info..
	memset(fieldInfo, 0, sizeof(fieldInfo));
	LogRecordData(rRec, fieldInfo); 
    csLog.Format(IDS_REVERSE_DEL, fieldInfo);
	LogAddEntry((const char*)csLog.GetBuffer(0), slReverseDelete,FALSE);
}




///////////////////////////////////////////////////////////////////////////
// CBaseConduitMonitor 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		LogRecCountMismatch()      ** Protected **     
//
//	Description:	Called when desktop database is full and cannot accept
//                 anymore records from the pilot. 
//
//	Parameters:		
//					locCnt - record count of the Desktop dataBase
//					remCnt - record count of the Pilot dataBase
//		
//	Returns:		Nothing
//
///////////////////////////////////////////////////////////////////////////
void CBaseConduitMonitor::LogRecCountMismatch(long locCnt, long remCnt)
{
	char	errBuff[MAX_LOG_STRING];
	char	strData[MAX_LOG_STRING];

	LogBanner();
    CString csError;

    csError.LoadString(IDS_RECCOUNT_DESKTOP);
	LogAddEntry(csError, slRecCountMismatch,FALSE);

	// Load string from the resource file.
	::LoadString(m_DllInstance, IDSTR_DESKTOP, strData, sizeof(strData));
	sprintf(errBuff, "     %1s = %2ld, %3s = %4ld", strData, locCnt, HH_NAME, remCnt);
	LogAddEntry((const char*)errBuff,slText,FALSE);	         
}


///////////////////////////////////////////////////////////////////////////
// CBaseConduitMonitor 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		LogPilotFull()      ** Protected **     
//
//	Description:    Called when the Pilot cannot accept anymore database 
//                 records.	
//                 
//
//	Parameters:		
//					locCnt - record count of the Desktop dataBase
//					remCnt - record count of the Pilot dataBase
//		
//	Returns:		Nothing
//
///////////////////////////////////////////////////////////////////////////
void CBaseConduitMonitor::LogPilotFull(long locCnt, long remCnt)
{
	char	errBuff[MAX_LOG_STRING];
	char	strData[MAX_LOG_STRING];

	LogBanner();
    CString csError;


    csError.LoadString(IDS_RECCOUNT_PILOT);
	LogAddEntry(csError, slRecCountMismatch,FALSE);

    // Load string from the resource file.
	::LoadString(m_DllInstance, IDSTR_DESKTOP, strData, sizeof(strData));
	sprintf(errBuff, "     %1s = %2ld, %3s = %4ld", strData, locCnt, HH_NAME, remCnt);
	LogAddEntry((const char*)errBuff,slText,FALSE);	         
}


///////////////////////////////////////////////////////////////////////////
// CBaseConduitMonitor 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		LogRecordData()      ** Protected **     
//
//	Description:	By default this does nothing. It is expected that the
//					derived objects will have something to offer in their
//					version of this routine.   
//					
//	Parameters:		CBaseRecord& - record object causing possible error	
//					char* - buffer to place some field values into
//		
//	Returns:		Nothing
//
///////////////////////////////////////////////////////////////////////////
void CBaseConduitMonitor::LogRecordData(CBaseRecord&, char*) 
{
}


///////////////////////////////////////////////////////////////////////////
// CBaseConduitMonitor 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		LogApplicationName()      ** Protected **     
//
//	Description:	By default this does nothing. It is expected that the
//					derived objects will have something to offer in their
//					version of this routine.   
//					
//	Parameters:		char* - Application Name
//		
//	Returns:		Nothing
//
///////////////////////////////////////////////////////////////////////////
void CBaseConduitMonitor::LogApplicationName(char*, WORD) 
{
}


////////////////////////////////////////////////////////////////////////////
// CBaseConduitMonitor 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		ProcessSubscription()      ** Protected **     
//
//	Description:	Processes any pending subscription handling for the
//					conduit.  This method is called before a records are
//					synchronized.
//
//					** PLACEHOLDER ** - It is assumed that subscription
//										handling is unique to a given
//										conduit.   
//					
//	Parameters:		None.
//		
//	Returns:		None.
//
///////////////////////////////////////////////////////////////////////////
long CBaseConduitMonitor::ProcessSubscription()
{
	long retval = 0; 
	return(retval);
}

///////////////////////////////////////////////////////////////////////////
// CBaseConduitMonitor 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		GetFirstRecord()      ** Protected **     
//
//	Description:	Processes any pending subscription handling for the
//					conduit.  This method is called before a records are
//					synchronized.
//
//					** PLACEHOLDER ** - It is assumed that subscription
//										handling is unique to a given
//										conduit.   
//					
//	Parameters:		None.
//		
//	Returns:		None.
//
///////////////////////////////////////////////////////////////////////////
int CBaseConduitMonitor::GetFirstRecord (CStringArray*& pSubRecord )
{
	return( 0);
}

///////////////////////////////////////////////////////////////////////////
// CBaseConduitMonitor 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		GetNextRecord()      ** Protected **     
//
//	Description:	Processes any pending subscription handling for the
//					conduit.  This method is called before a records are
//					synchronized.
//
//					** PLACEHOLDER ** - It is assumed that subscription
//										handling is unique to a given
//										conduit.   
//					
//	Parameters:		None.
//		
//	Returns:		None.
//
///////////////////////////////////////////////////////////////////////////
int CBaseConduitMonitor::GetNextRecord (CStringArray*& pSubRecord )
{
	return(0);	
}

///////////////////////////////////////////////////////////////////////////
// CBaseConduitMonitor 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		GetSubData()      ** Protected **     
//
//	Description:	Processes any pending subscription handling for the
//					conduit.  This method is called before a records are
//					synchronized.
//
//					** PLACEHOLDER ** - It is assumed that subscription
//										handling is unique to a given
//										conduit.   
//					
//	Parameters:		None.
//		
//	Returns:		None.
//
///////////////////////////////////////////////////////////////////////////
int CBaseConduitMonitor::GetSubData (CString& csfilename, CString csFldOrder  )
{
	return(0);
	
}

///////////////////////////////////////////////////////////////////////////
// CBaseConduitMonitor 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		AddSubDataToTable()      ** Protected **     
//
//	Description:	Processes any pending subscription handling for the
//					conduit.  This method is called before a records are
//					synchronized.
//
//					** PLACEHOLDER ** - It is assumed that subscription
//										handling is unique to a given
//										conduit.   
//					
//	Parameters:		None.
//		
//	Returns:		None.
//
///////////////////////////////////////////////////////////////////////////
void CBaseConduitMonitor::AddSubDataToTable ( int subCatId )
{
	
}

/////////////////////////////////////////////////////////////////////////////
// CSubscribeDll
/////////////////////////////////////////////////////////////////////////////
//
//	 Method:		SubDeleteTableRecs()
//
//	 Description:	Removes unmodified records from
// 					the subscription category in the table.
//
//	 Parameters:	CSubInfo	- Subscription info.
//					CBaseTable*	- Table to modify
//
//	 Returns:		subNoError	- Success.
//					subFailed	- Unable to save the data file.
//
/////////////////////////////////////////////////////////////////////////////
int CBaseConduitMonitor::DeleteSubscTableRecs(CString& csCatName, CBaseTable* pTable, WORD wDeleteOption)
{
	int			errorRtn = 0;
	return(errorRtn);
}

/////////////////////////////////////////////////////////////////////////////
// CBaseConduitMonitor
/////////////////////////////////////////////////////////////////////////////
//
//	 Method:		AddCategory()
//
//	 Description:	Adds the category to the table 
//
//	 Parameters:	CString&	- Subscription category name from user.
//					
//
//	 Returns:		int			- Returns	(-1)			if cat could not be added.
//															index		of category
//
/////////////////////////////////////////////////////////////////////////////
int CBaseConduitMonitor::AddCategory(CString& csCatName, CBaseTable* pTable)
{
	int rtnIndex = -1;
	return (rtnIndex);
}

/////////////////////////////////////////////////////////////////////////////
// CBaseConduitMonitor
/////////////////////////////////////////////////////////////////////////////
//
//	 Method:		GetCategories()
//
//	 Description:	
//
//	 Parameters:	CString&	- Subscription category name from user.
//					
//
//	 Returns:		int			- Returns	(0)		if success
//
/////////////////////////////////////////////////////////////////////////////
int CBaseConduitMonitor::GetCategories(char  categoryNames[][CAT_NAME_LEN])
{
	int errRtn = -1;
	return(errRtn);
}

/////////////////////////////////////////////////////////////////////////////
// CBaseConduitMonitor
/////////////////////////////////////////////////////////////////////////////
//
//	 Method:		LogModifiedSubscRec()
//
//	 Description:	Update local data with subscription config 
//							changes such as renaming / deletion of categories
//
//	 Parameters:	
//					
//
//	 Returns:		long		0 - no error
//											!0 - error value
//
/////////////////////////////////////////////////////////////////////////////
long CBaseConduitMonitor::LogModifiedSubscRec(CBaseRecord* pRecord, BOOL blocalRec)
{
	long errRtn = -1;
		return (0);
}

/////////////////////////////////////////////////////////////////////////////
// CBaseConduitMonitor
/////////////////////////////////////////////////////////////////////////////
//
//	 Method:		UpdateTablesOnSubsc()
//
//	 Description:	Update local data with subscription config 
//							changes such as renaming / deletion of categories
//
//	 Parameters:	
//					
//
//	 Returns:		long		0 - no error
//											!0 - error value
//
/////////////////////////////////////////////////////////////////////////////
long CBaseConduitMonitor::UpdateTablesOnSubsc()
{
	long errRtn =CONDERR_BAD_LOCAL_TABLES;
	return (errRtn);
}

/////////////////////////////////////////////////////////////////////////////
// CBaseConduitMonitor
/////////////////////////////////////////////////////////////////////////////
//
//	 Method:		CheckFileName()
//
//	 Description:	Check if this is a DOS reserved
//                          word, and append  "_" if it is
//
//	 Parameters:	
//					
//
//	 Returns:		long		0 - no error
//											!0 - error value
//
/////////////////////////////////////////////////////////////////////////////
long CBaseConduitMonitor::CheckFileName(CString& csFileName)
{

	// make sure we don't have a reserved name...
	for (int i = 0; i < DOS_RESERVED_COUNT; i++)
	{
		if (csFileName.CompareNoCase(ReservedFileNames[i]) == 0)
		{
			csFileName += "_" ;
			break;
		}
	}
	return 0;
}

///////////////////////////////////////////////////////////////////////////
// CBaseConduitMonitor 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		virtual IsFatalConduitError()      
//
//	Description:	Some conduits may return an error that is
//                  non-fatal to the entire sync. Such as Todo and
//                  DateBook with records before 1970 return an
//                  error specifying the record date has moved.
//
//	Parameters:		long lError - specifies the error code.
//		
//	Returns:		TRUE fatal error.
//                  FALSe - non-fatal error
//
///////////////////////////////////////////////////////////////////////////
BOOL CBaseConduitMonitor::IsFatalConduitError(long lError, CBaseRecord *prRec)
{
    if (lError)
        return TRUE;
    return FALSE;
}


//-------------------------------------------------------------------------//
//-------------------------------------------------------------------------//
//--------------------------                      -------------------------//
//-------------------------- CBaseDTLinkConverter -------------------------//
//--------------------------                      -------------------------//
//-------------------------------------------------------------------------//
//-------------------------------------------------------------------------//

///////////////////////////////////////////////////////////////////////////
// CBaseDTLinkConverter 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		CBaseDTLinkConverter()      Public 
//
//	Description:	Constructor	
//
//	Parameters:		None
//
//	Returns:		n/a
//
///////////////////////////////////////////////////////////////////////////
CBaseDTLinkConverter::CBaseDTLinkConverter(HINSTANCE hInst)
					  : m_TransBuff(NULL)
{
	m_DllInstance = hInst;
	m_TransBuff = (TCHAR*) new char [TRANS_BUFFER_SIZE];
	if (m_TransBuff)
		memset(m_TransBuff, 0, TRANS_BUFFER_SIZE);
}


///////////////////////////////////////////////////////////////////////////
// CBaseDTLinkConverter 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		CBaseDTLinkConverter()      Public 
//
//	Description:	Destructor	
//
//	Parameters:		None
//
//	Returns:		n/a
//
///////////////////////////////////////////////////////////////////////////
CBaseDTLinkConverter::~CBaseDTLinkConverter()  
{
	if (m_TransBuff)
		delete m_TransBuff;
}


///////////////////////////////////////////////////////////////////////////
// CBaseDTLinkConverter 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		ConvertToRemote()        Public 
//
//	Description:	Reformat the local PC record into the format recognized
//					by the remote device, place the results into the second
//					parameter 'rInfo'.
//
//	Parameters:	
//       CBaseRecord &rRec  - PC Record object to read from
//       CRecordInfo &rInfo - Existing memory into which is placed the record
//                            data in a layout recognized by the remote device.
//
//	Returns:		long	error code
//
///////////////////////////////////////////////////////////////////////////
long CBaseDTLinkConverter::ConvertToRemote(CBaseRecord &rRec, 
                                           CRawRecordInfo &rInfo)
{
	long retval = -1;
	return(retval);
}


///////////////////////////////////////////////////////////////////////////
// CBaseDTLinkConverter 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		ConvertFromRemote()        Public 
//
//	Description:	Reformat the local PC record into the format recognized
//					by the remote device, place the results into the second
//					parameter 'rInfo'.
//
//	Parameters:	
//		CBaseRecord &rRec - PC Record object to write into 
//		CRecordInfo &rInfo - Existing memory into contains the raw record
//							 data fromt the remote device and must be parsed
//							 from in order to populate the PC record object.
//
//	Returns:		long	error code
//
///////////////////////////////////////////////////////////////////////////
long CBaseDTLinkConverter::ConvertFromRemote(CBaseRecord &rRec, 
                                             CRawRecordInfo &rInfo)
{
	long retval = -1;
	return(retval);
}


///////////////////////////////////////////////////////////////////////////
// CBaseDTLinkConverter 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		StripCRs()      ** Protected **     
//
//	Description:	Remove all the Carriage returns (if they exist) for the
//					passed in source byte stream.    
//
//	Parameters:		TCHAR* pDest - New string with CRs removed
//					TCHAR* pSrc - String to be striped
//					int len - length of pSrc
//
//	Returns:		int	- destination length
//
///////////////////////////////////////////////////////////////////////////
int CBaseDTLinkConverter::StripCRs(TCHAR* pDest, TCHAR* pSrc, int len)
{
	int  retval = 0;
	int  off    = 0;
	TCHAR* pCurr; 
	TCHAR* pStart = pDest;

	// See if any cr's are present in the first place.
	if ((pCurr = _tcspbrk(pSrc, STR_CRETURN)))
	{
		while (off < len && *pSrc)
		{
			if (*pSrc == CH_CRETURN)
				pSrc++;

			*pDest = *pSrc;

			off++;
			pDest++ ; pSrc++;
		}
		*pDest = 0;

		retval = strlen(pStart) + 1;
	}
	else
	{
		strncpy(pDest, pSrc, len);
		retval = len + 1;
	}	

	return(retval);
}


///////////////////////////////////////////////////////////////////////////
// CBaseDTLinkConverter 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		AddCRs()      ** Protected **     
//
//	Description:	Add in carriage returns (if newlines are detected).
//
//	Parameters:		Results are placed in data member m_TransBuffer. 
//
//	Returns:		long	error code
//
///////////////////////////////////////////////////////////////////////////
long CBaseDTLinkConverter::AddCRs(TCHAR* pSrc, int len)
{
	long retval = CONDERR_RAW_RECORD_ALLOCATE;
	int off=0;

	TCHAR* pCurr;

	if (m_TransBuff)
	{
		TCHAR* pDest = m_TransBuff;

		// Only add Cr's if newlines are present..
		if ((pCurr = _tcspbrk(pSrc, STR_NEWLINE)))
		{
			while (off < len && *pSrc)
			{
				if (*pSrc == CH_NEWLINE)
				{
					*pDest = CH_CRETURN;
					pDest++;
				}
				*pDest = *pSrc;

				pDest++ ; pSrc++;
				off++;
			}
			*pDest = 0;
		}
		else
		{
			strncpy(pDest, pSrc, len);
			pDest[len] = 0;
		}
		retval = 0;
	}
	return(retval);
}


///////////////////////////////////////////////////////////////////////////
// CBaseDTLinkConverter 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		ExtractCategories()        ** Protected ** 
//
//	Description:	The category labels are obtained through DT Link in a raw
//					format and passed in as the first parameter.  This routine
//					will add the category labels to the CategoryMgr passed as
//					the second paramenter.
//
//	Parameters:	
//					char* catLabelsPtr - Pointer to the category labels
//					short dirtyCats - Bits represent the dirty categories
//					CCategoryMgr* catMgr - Categories will be added to this class
//
//	Returns:		long	error code
//
///////////////////////////////////////////////////////////////////////////
long CBaseDTLinkConverter::ExtractCategories(char* catLabelsPtr, short dirtyCats, 
                                             CCategoryMgr* catMgr)
{
	long        retval = 0;
	CCategory	*category;
	CString		catName("");
	char		*catIdsPtr;
	BYTE		tempByte;
	short		dirtyBit = 1;

	// Start of category unique IDs
	catIdsPtr = catLabelsPtr + (MAX_CATEGORIES * CATEGORY_LENGTH);

	for (int i=0; i<MAX_CATEGORIES; i++)
	{
		if (strlen(&(catLabelsPtr[i*CATEGORY_LENGTH])) != 0)
		{
			category = new CCategory;
			if (category)
			{
				category->SetIndex(i);

				catName = (const char*)(&(catLabelsPtr[i*CATEGORY_LENGTH]));
				category->SetName(catName);

				tempByte = *(catIdsPtr + (i * sizeof(BYTE)));
				category->SetID((int)tempByte);

				if (dirtyCats & dirtyBit)
					category->SetNameDirty(TRUE);
				else
					category->SetNameDirty(FALSE);

				if (catMgr->Add(category) != noError)
					delete category;
			}
		}
		// Get the next bit which represents the next category
		dirtyBit = dirtyBit * 2;
	}
	return(retval);
}


///////////////////////////////////////////////////////////////////////////
// CBaseDTLinkConverter 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		ReplaceCategories()        ** Protected ** 
//
//	Description:	The categories are passed in through the CCategoryMgr class.
//					This routine will place those categories in the raw format
//					needed for DT Link.
//
//	Parameters:	
//		char* catLabelsPtr - Pointer to the category labels
//		CCategoryMgr* catMgr - Categories will be passed in from this class
//
//	Returns:		nothing
//
///////////////////////////////////////////////////////////////////////////
long CBaseDTLinkConverter::ReplaceCategories(char* catLabelsPtr, 
                                             CCategoryMgr* catMgr)
{
	long        retval = 0;
	CCategory	*category;
	CString		catName;
	char		*catIdsPtr;
	int			len;

	// Start of category unique IDs
	catIdsPtr = catLabelsPtr + (MAX_CATEGORIES * CATEGORY_LENGTH);

	for (int j=0; j<MAX_CATEGORIES; j++)
	{
		memset(&(catLabelsPtr[j*CATEGORY_LENGTH]), 0, CATEGORY_LENGTH);
		*((BYTE *)(catIdsPtr + (j * sizeof(BYTE)))) = 0;
	}

	for (int i=0; i<MAX_CATEGORIES; i++)
	{
		catMgr->FindIndex(i, category);
		if (category != NULL)
		{
			catName = category->GetName();
			LPTSTR pStr = catName.GetBuffer(CATEGORY_LENGTH);
			len = strlen(pStr);
			if (len >= CATEGORY_LENGTH)
				len = CATEGORY_LENGTH - 1;
			strncpy(&(catLabelsPtr[i*CATEGORY_LENGTH]), pStr, len);
			catName.ReleaseBuffer();

			*((BYTE *)(catIdsPtr + (i * sizeof(BYTE)))) = (BYTE)category->GetID();
		}
	}

	return(retval);
}


///////////////////////////////////////////////////////////////////////////
// CBaseDTLinkConverter 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		ConvertToRemoteCategories()			Public 
//
//	Description:	Virtual routine meant to be overridden by derivatives.
//                
//	Parameters:		CDbGenInfo&
//					CCategoryMgr*
//
//	Returns:		long	error code
//
///////////////////////////////////////////////////////////////////////////
long CBaseDTLinkConverter::ConvertToRemoteCategories(CDbGenInfo& dbInfo, 
                                                     CCategoryMgr* catMgr)
{
	return(-1);
}


///////////////////////////////////////////////////////////////////////////
// CBaseDTLinkConverter 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		ConvertFromRemoteCategories()          Public 
//
//	Description:	Virtual routine meant to be overridden by derivatives.
//                
//	Parameters:		CDbGenInfo&
//					CCategoryMgr*
//
//	Returns:		long	error code
//
///////////////////////////////////////////////////////////////////////////
long CBaseDTLinkConverter::ConvertFromRemoteCategories(CDbGenInfo& dbInfo, 
                                                       CCategoryMgr* catMgr)
{
	return(-1);
}


///////////////////////////////////////////////////////////////////////////
// CBaseDTLinkConverter 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		SynchronizeAppInfoBlock()          Public 
//
//	Description:	Virtual routine meant to be overridden by derivatives if
//					needed. 
//                
//	Parameters:		CDbGenInfo&
//					CBaseTable&
//					eSyncTypes
//
//	Returns:		Nothing
//
///////////////////////////////////////////////////////////////////////////
void CBaseDTLinkConverter::SynchronizeAppInfoBlock(CDbGenInfo& dbInfo,
												   CBaseTable& rTable,
												   eSyncTypes syncType,
                                                   eFirstSync firstSync)
{
}


///////////////////////////////////////////////////////////////////////////
// CBaseDTLinkConverter 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		ConvertToTdDate()
//
//	Description:	Convert the date from CTime format to the Pilot format.
//                
//	Parameters:		TdDateType& - Pilot Date format
//					TdTimeType& - Pilot Time format
//					long - Date & Time in Desktop format
//
//	Returns:		0 - Success
//
///////////////////////////////////////////////////////////////////////////
long CBaseDTLinkConverter::ConvertToTdDate(TdDateType& rTdDate, 
                                           TdTimeType& rTdTime, long lDate)
{
	long aYear, retval = 0;

	CTime tim(lDate);

	rTdTime.hours   = (unsigned char)tim.GetHour();
	rTdTime.minutes = (unsigned char)tim.GetMinute();

	rTdDate.day   = (unsigned short) tim.GetDay();
	rTdDate.month = (unsigned short) tim.GetMonth();
	aYear         = (unsigned short) tim.GetYear();
	aYear -= FIRST_YEAR;
	rTdDate.year  = (unsigned short) aYear;

	return(retval);
}


///////////////////////////////////////////////////////////////////////////
// CBaseDTLinkConverter 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		ConvertFromTdDate()
//
//	Description:	Convert from the Pilot Date & Time to the Desktop
//					Date & Time	
//                
//	Parameters:		TdDateType& - Pilot Date format
//					TdTimeType& - Pilot Time format
//					long& - results of conversion
// 
//	Returns:		0 - Success   
//
///////////////////////////////////////////////////////////////////////////
long CBaseDTLinkConverter::ConvertFromTdDate(TdDateType& rTdDate, 
                                             TdTimeType& rTdTime, long& rDate)
{
	long retval = 0;
	int   year, month, day, hour, minute, secs; 

	year   = rTdDate.year  ; 
	year  += FIRST_YEAR;
	month  = rTdDate.month ; 
	day    = rTdDate.day   ; 
	hour   = rTdTime.hours   ; 
	minute = rTdTime.minutes ;
	secs   = 0;

	if (year < 1970)
	{
		year = 1970;
		retval = CONDERR_DATE_MOVED;
	}
	if (!(month >= 1 && month <= 12))    // just in case funky months
		month = 12;

	CTime tim(year,month,day,hour,minute,secs);
	rDate = (long) tim.GetTime();
	
	return(retval);
}


/////////////////////////////////////////////////////////////////////////////
// CBaseDTLinkConverter
/////////////////////////////////////////////////////////////////////////////
//
//	Function:		SwapDWordToMotor()                 ** Protected **     
//
//	Description:	Converts an intel 4 byte DWORD to a 4 byte Motorola DWORD.
//					Grinds through and makes the least significant byte of 
//					the passed in subject the most significant byte of the
//					returned answer.
//                
//	Parameters:		DWORD value to swap   
//					
//	Returns:		Swapped DWORD
//
/////////////////////////////////////////////////////////////////////////////
unsigned long CBaseDTLinkConverter::SwapDWordToMotor(unsigned long dwSubj)
{
	DWORD  dwAnswer = 0; 
	BYTE   oneByte; 
	int    maxCars = sizeof(DWORD) - 1;    // 3

	for (int i=0; i <= maxCars; i++)
	{
		oneByte  = (BYTE)((dwSubj >> (i * 8)) & 0xFF); 
		dwAnswer |= oneByte << ((maxCars - i) * 8);
	}
	return(dwAnswer);
}


///////////////////////////////////////////////////////////////////////////
// CBaseDTLinkConverter
///////////////////////////////////////////////////////////////////////////
//
//	Function:	    SwapDWordToIntel()             ** Protected ** 
//
//	Description:   Converts a 4 byte Motorola DWORD to an intel 4 byte DWORD.
//					Grinds through and makes the least significant byte of 
//					the passed in subject the most significant byte of the
//					returned answer.
//
//					Motorola order:	Least to Most
//					Intel order: Most to Least
//
//	Parameters:	DWORD value to swap   
//					
//	Returns:		Swapped DWORD
//
///////////////////////////////////////////////////////////////////////////
unsigned long CBaseDTLinkConverter::SwapDWordToIntel(DWORD dwSubj)
{
	DWORD  dwAnswer = 0; 
	BYTE   oneByte; 
	int    maxCars = sizeof(DWORD) - 1;    // 3

	for (int i=0; i <= maxCars; i++)
	{
		oneByte = (BYTE)((dwSubj >> ((maxCars - i) * 8)) & 0xFF); 
		dwAnswer |= oneByte << (i * 8);
	}
	return(dwAnswer);
}


/////////////////////////////////////////////////////////////////////////////
// CBaseDTLinkConverter
/////////////////////////////////////////////////////////////////////////////
//
//	Function:		FlipWord()                 ** Protected **     
//
//	Description:	Flips a word form Intel to Motorola or from Motorola
//					to Intel.
//                
//	Parameters:		WORD to flip
//					
//	Returns:		fliped WORD
//
/////////////////////////////////////////////////////////////////////////////
WORD CBaseDTLinkConverter::FlipWord(WORD wSubj)
{
	WORD wAnswer;

	wAnswer = ((wSubj >> 8) & 0xff) | ((wSubj & 0xff) << 8);

	return(wAnswer);
}


/////////////////////////////////////////////////////////////////////////////
// CBaseDTLinkConverter
/////////////////////////////////////////////////////////////////////////////
//
//	Function:		ConvertPositionMap()		Public 
//
//	Description:	Spin through the motorola record Id's and flip them
//					to Intel format.  
//                
//	Parameters:		CPositionInfo&
//
//	Returns:		0
//
/////////////////////////////////////////////////////////////////////////////
long CBaseDTLinkConverter::ConvertPositionMap(CPositionInfo &rInfo)
{
	WORD  wOff;
	DWORD dwRecId;
	DWORD* pRecs;

	if (rInfo.m_pBytes && rInfo.m_NumReadIn > 0) 
	{
		pRecs = (DWORD*) rInfo.m_pBytes;

		for (wOff=0; wOff < rInfo.m_NumReadIn; wOff++)
		{
			dwRecId = pRecs[wOff]; 

			pRecs[wOff] = SwapDWordToIntel(dwRecId);
		}
	}
	return(0);
}



