/***********************************************************************
 *
 * FileName:
 *		ShrLib.h
 *
 * Description:
 *    This file contains the declarations of the API  published by the
 *    shared library.
 *
 * Copyright  1999, Bachmann Software and Services.
 * All rights reserved.
 *
 *******************************************************************/

#ifndef __SHRLIB_H__
#define __SHRLIB_H__

/*
   Check to see if we're compiling the actual implementation of
   the API, or if this header is being compiled into a source
   file of a project that uses the shared library.
*/
#ifdef BUILDING_LIBRARY
	#define SHRLIB_TRAP(trapNum)
#else
	#define SHRLIB_TRAP(trapNum) SYS_TRAP(trapNum)
#endif


/*
   Shared Library Identifiers:

   These identifiers are used by clients of the shared library to load
   the library:

      shrCreatorID:  Identifier for the library, registered with Palm Computing.
      shrTypeID:     Type of PRC.  For shared libraries, this is defined as 'libr'.
      shrName:       Textual name for library that can be passed to sysLibFind.
*/

#define  shrCreatorID   'SHRL'      // Shared Library database creator
#define  shrTypeID      'libr'      // Standard library database type
#define  shrName        "Shared Library"

/*
   Result Codes:

   Result codes are OR'd with the system constant appErrorClass,
   which is defined in SystemMgr.h.  appErrorClass is reserved
   for 3rd party applications and libraries.

   This library defines only a few:

      shrErrParam:   An invalid parameter was passed to a function.

      shrErrMemory:  An attempt to allocate memory failed.
*/

#define shrErrAlreadyOpen  (appErrorClass | 1)
#define shrErrParam        (appErrorClass | 2)
#define shrErrMemory       (appErrorClass | 3)

/*
   API Prototypes
  
   We declare these as "C" functions because we will need to 
   hard-code the function name into the dispatch table.  If we
   declare them as C++ functions, they will be name-mangled.
*/ 
#ifdef __cplusplus
extern "C" {
#endif

/* 
   Trap Identifiers:

   We declare a set of ids for our published API.  These traps are actually
   indices into the library's dispatch table.  The system defines traps for
   the required Open, Close, Sleep and Wake functions (sysLibTrapOpen,
   sysLibTrapClose, sysLibTrapSleep and sysLibWake, respectively).  We
   *must* start our identifiers with sysLibTrapCustom, and increment exactly
   one for each id - we can not skip.  We use an enum to ensure this.

   Also, the order of these traps *must* match the order of the associated 
   functions in the dispatch table.  

   See ShrDisp.c for explanations of the dispatch table.
*/   

/*
   ShrTrapIdEnum:

   We declare one trap identifier for each function we publish, plus one
   for "last".  The identifiers start with sysLibTrapCustom.
*/      
typedef enum {
   shrTrapAPIVersion = sysLibTrapCustom,
   shrTrapGetText,
   shrTrapSetText,
   shrTrapLast
} ShrTrapNumberEnum;

/*
   Function Declarations:

   Notice that the first parameter of all functions is a unsigned integer.
   This is the library reference number.  This value is returned when the
   application attaches to the library (either by loading it via SysLibLoad,
   or by finding an already loaded copy via SysLibFind).

   The operating system multiplexes calls to shared libraries by reference
   number and trap number.
*/

/*
   Standard Library Functions:

   Shared libraries are required to publish 4 functions:

   Open:    This function *must* be the first function called by
            client of the shared library.  It allows the shared
            library to initialize any resources it needs.

   Close:   This function *must* be the last function called by the
            user of the shared library.  It allows the shared library 
            to release any resources it owns.

   Sleep:   The operating system calls this function before the device
            enters sleep mode.  It allows system-level libraries to shut
            down hardware components to conserve power.

   Wake:    The operating system calls this function when the device
            wakes from sleep mable.  It allows system-level libraries
            to re-enable hardware components that were shut down when
            the device went to sleep.
*/
extern Err ShrOpen(UInt16 refNum)
				SHRLIB_TRAP(sysLibTrapOpen);
				
extern Err ShrClose(UInt16 refNum)
				SHRLIB_TRAP(sysLibTrapClose);

extern Err ShrSleep(UInt16 refNum)
				SHRLIB_TRAP(sysLibTrapSleep);

extern Err ShrWake(UInt16 refNum)
				SHRLIB_TRAP(sysLibTrapWake);

/*
   Custom Library Functions:

   These functions represent the services offered by the library:
   
   ShrGetVersion: Return the library version.  This is the only
                  method that can be called outside of an open
                  library session.

   ShrGetText:    Sample function that returns a string stored in
                  the library.

   ShrSetText:    Sample function that sets the string stored in 
                  the library.
*/

// Get our library API version
extern Err ShrAPIVersion(UInt16 refNum, UInt32 * dwVerP)
				SHRLIB_TRAP(shrTrapAPIVersion);
	
// Retrieve the text stored by the library.
extern Err ShrGetText(UInt16 refNum, char * string, UInt16 size)
				SHRLIB_TRAP(shrTrapGetText);

// Set the text stored by the library.
extern Err ShrSetText(UInt16 refNum, char * string)
				SHRLIB_TRAP(shrTrapSetText);

#ifdef __cplusplus 
}
#endif

#endif	// __SHRLIB_H__
