/***********************************************************************
 *
 * FileName:
 *		ShrLib.c
 *
 * Description:
 *		This file contains the implementation of the API published by the
 *    shared library.
 *
 * Note:
 *    Because this is example code, most of the error checking and safety
 *    measures (such as setting unlocked pointers to NULL) have been removed.
 *    This would need to be added back for production-ready code.
 *
 * Copyright  1999, Bachmann Software and Services.
 * All rights reserved.
 *
 **********************************************************************/

#include <PalmOS.h>			// Includes all PalmPIlot headers

// Include public declarations (API and constants).
#define BUILDING_LIBRARY
#include "ShrLib.h"

/*
   Globals:

   As a rule, globals or static variables are prohibited in shared
   library.  There are techniques to get around this prohibition,
   but they are best avoided.

   Instead, values that must persist between function calls are
   stored in a structure that is allocated off the heap.  We
   use movable memory to minimize dynamic heap fragmentation.
   This memory is locked only when needed and unlocked as soon
   as possible, to optimize system performance.
*/

/*
   Private Constants:

   Constants that are used internally by the shared library.

      shrMaxText:    Maximum length of text stored by the shared library.
*/
#define shrMaxText   (99)

/*
   ShrGlobalsType:

   Global values used in the library:

      thisLibRefNum:    Shared library reference number returned
                        by SysLibLoad() or SysLibFind() and passed
                        as a parameter to ShrOpen().
                        This value is stored in the global data
                        structure to aid in debugging.

      text:             The string stored in the library.  This is
                        initialized to an empty string, then accessed
                        via ShrGetText() and ShrSetText().
*/
typedef struct ShrGlobalsType {
   UInt16  refNum;
   Char  text [shrMaxText+1];
	} ShrGlobalsType;

typedef ShrGlobalsType*  ShrGlobalsPtr;

/*
   Library Version:

   The library version is determined according to the scheme defined
   in SystemMgr.h.  The version is stored in a long formatted as
   0xMMmfsbbb, where:

      MM:   Major version of library.
      m:    Minor version of library.
      f:    Is bug fix.
      s:    Stage of version.
      bbb:  Build number (for non-releases).

   Stages are defined as:
      0:    Development
      1:    Alpha
      2:    Beta
      3:    Release

   Examples:

   V1.12b3: 0x01122003
   V2.00a2: 0x02001002
   V1.01:   0x01013000

   See sysMakeROMVersion and sysGetROM... macros in SystemMgr.h
*/

#define prvShrVersion   sysMakeROMVersion(1, 0, 0, sysROMStageDevelopment, 1)


/*
   Function:      ShrOpen

   Description:   This function *must* be the first function called by
                  client of the shared library.  It allows the shared
                  library to initialize any resources it needs.

   Parameters:    refNum         Shared library reference number returned by
                                 SysLibLoad() or SysLibFind().

   Returns:       0              Success
                  shrErrMemory   Attempt to allocate resource failed.
*/
Err
ShrOpen(UInt16 refNum)
{
   // Declarations
   SysLibTblEntryPtr libEntryP = NULL;
   MemHandle globalsH = NULL;
   ShrGlobalsPtr globalsP = NULL;

   // Retrieve the system library table entry for this library.
   libEntryP = SysLibTblEntry(refNum); 

   // Make sure the globals have not been initialized.
   if (NULL != libEntryP->globalsP)
   {
      // Globals already initialized.
      // Library is open for another client.
      return shrErrAlreadyOpen;
   }

// Allocate and initialize the global data.

	// Allocate enough memory to hold data.
   // Note: We allocate memory handle rather than a (locked) pointer so that the 
   //       memory manager can move the block around.  Remember, this library may
   //       conceivably be around for a long time.
   globalsH = MemHandleNew (sizeof (ShrGlobalsType));

   // Set owner of global data memory block to "system" (zero).
   // Note: If the application were to remain the owner of the memory, the
   //       memory would be freed when the application exited.  For the
   //       library to maintain state between applications, this must not
   //       happen.  Any other allocated memory blocks should likewise be
   //       adopted by the system.
	MemHandleSetOwner(globalsH, 0);

	// Save the handle of our library globals in the system library table entry so we
	// can later retrieve it using SysLibTblEntry().
   // Note: lib entry structure expects a void pointer, so we must cast our
   //       handle.  See prev. note for explanation.
	libEntryP->globalsP = (void*) globalsH;

   // Lock the memory and cast it to a ShrGlobalsType pointer.
	globalsP = (ShrGlobalsPtr)MemHandleLock(globalsH);

   // Begin library-specific open functionality.

   // Initialize global data's reference number.
	globalsP->refNum = refNum;

   // Initialize global data's text buffer.
   StrCopy (globalsP->text, "Sample Shared Library text");

   // End library-specific open functionality.

   // Unlock globals.
   MemHandleUnlock (globalsH);

   // Return success.
	return 0;
}

/*
   Function:      ShrClose

   Description:   This function closes the shared library, and should
                  be the last function called by the client of the
                  shared library.  It allows the shared library to
                  release any resources it has allocated.

   Parameters:    refNum         Shared library reference number returned by
                                 SysLibLoad() or SysLibFind().

   Returns:       0              Success
*/
Err
ShrClose(UInt16 refNum)
{
   // Retrieve library system table entry.
	SysLibTblEntryPtr libEntryP = SysLibTblEntry(refNum); 

	// Retrieve the handle of our library globals from the library system
	// table entry.
	MemHandle globalsH = (MemHandle)(libEntryP->globalsP);

   // Begin library-specific close functionality.

   // End library-specific close functionality.

   // Clear library system table entry's global data.
	libEntryP->globalsP = NULL;

   // Free global data memory.
	MemHandleFree(globalsH);

   // Return success.
	return 0;
}

/*
   Function:      ShrSleep

   Description:   This function handles notification from the system that
                  it (the system) is about to shut down.  This notification
                  allows system-level libraries to conserve power while the
                  system is asleep by shutting down hardware components.

   Parameters:    refNum         Shared library reference number returned by
                                 SysLibLoad() or SysLibFind().

   Returns:       0              Success

   Note:          Because this function is invoked by a system interrupt,
                  it is only allowed to use interrupt-safe system services
                  (EvtEnqueueKey(), SysDisableInts() and SysRestoreStatus().
                  For the same reason, it must not take a long time.
*/
Err
ShrSleep(UInt16 /*refNum*/)
{
   // Return success.
	return 0;
}

/*
   Function:      ShrWake

   Description:   This function handles notification from the system that
                  it (the system) is about to wake up.  This notification
                  allows system-level libraries to re-enable hardware
                  components that were disabled when the system went to
                  sleep.

   Parameters:    refNum         Shared library reference number returned by
                                 SysLibLoad() or SysLibFind().

   Returns:       0              Success

   Note:          Because this function is invoked by a system interrupt,
                  it is only allowed to use interrupt-safe system services
                  (EvtEnqueueKey(), SysDisableInts() and SysRestoreStatus().
                  For the same reason, it must not take a long time.
*/
Err
ShrWake(UInt16 /*refNum*/)
{
   // Return success.
	return 0;
}

/*
   Function:      ShrAPIVersion

   Description:   Get the version of the library's API.

   Parameters:    refNum         Shared library reference number returned by
                                 SysLibLoad() or SysLibFind().
                  versionP       Output parameter for version number.

   Returns:       0              Success

   Note:          This is the only function which can be called whether
                  or not open has been called.

   Note:          This function returns the API version, which may or may not
                  be the same as the library version.  This allows client code
                  to determine if it is compatible with the API.
*/
Err ShrAPIVersion(UInt16 /*refNum*/, UInt32 * versionP)
{
   // Assign version constant to output parameter.
	(*versionP) = prvShrVersion;
	
   // Return success.
	return 0;
}

/*
   Function:      ShrGetText

   Description:   Sample function that retrieves the text stored in the
                  shared library.

   Parameters:    refNum         Shared library reference number returned by
                                 SysLibLoad() or SysLibFind().
                  string         Output parameter for stored text.
                  size           Size of string buffer.

   Returns:       0              Success
*/
Err
ShrGetText(UInt16 refNum, char * string, UInt16 size)
{
   // Retrieve library system table entry.
	SysLibTblEntryPtr libEntryP = SysLibTblEntry(refNum); 

	// Retrieve the handle of our library globals from the library system
	// table entry.
	MemHandle globalsH = (MemHandle)(libEntryP->globalsP);

   // Lock the memory and cast it to a ShrGlobalsType pointer.
	ShrGlobalsPtr globalsP = (ShrGlobalsPtr) MemHandleLock(globalsH);

   // Copy saved text to output parameter.
   StrNCopy (string, globalsP->text, (short)size-1);

   // Make sure string is null-terminated.
   // (StrNCopy() does *not* do this.)
   string [size-1] = '\0';
   
   // Unlock library globals.
   MemHandleUnlock (globalsH);

   // Return success.
	return 0;
}

/*
   Function:      ShrSetText

   Description:   Sample function that stores a string in the shared library.

   Parameters:    refNum         Shared library reference number returned by
                                 SysLibLoad() or SysLibFind().
                  string         New string to store.

   Returns:       0              Success
*/
Err
ShrSetText(UInt16 refNum, char * string)
{
   // Retrieve library system table entry.
	SysLibTblEntryPtr libEntryP = libEntryP = SysLibTblEntry(refNum); 

	// Retrieve the handle of our library globals from the library system
	// table entry.  Save this handle for later - we will free it.
   // Note: lib entry structure expects a void pointer, so we must cast the
   //       value back to a handle.  See notes in PrvShrMakeGlobals().
	MemHandle globalsH = (MemHandle)(libEntryP->globalsP);

   // Lock the memory and cast it to a ShrGlobalsType pointer.
	ShrGlobalsPtr globalsP = (ShrGlobalsPtr)MemHandleLock(globalsH);

   // Calc. max. size of stored text.
   UInt16 size = sizeof (globalsP->text);

   // Copy saved text to output parameter.
   StrNCopy(globalsP->text, string, (short)size-1);

   // Make sure string is null-terminated.
   // (StrNCopy() does *not* do this.)
   string [size-1] = '\0';
   
   // Unlock library globals.
   MemHandleUnlock (globalsH);

   // Return success.
	return 0;
}
