/***********************************************************************
 *
 * File Name:
 *		Sample.c
 *
 * Description:
 *		This file contains the implementation of the sample shared libary
 *    client application.
 *
 * Note:
 *    Because this is example code, most of the error checking and safety
 *    measures (such as setting unlocked pointers to NULL) have been removed.
 *    This would need to be added back for production-ready code.
 *
 * Copyright  1999, Bachmann Software and Services.
 * All rights reserved.
 *
 **********************************************************************/

#include <PalmOS.h>
#include <SysEvtMgr.h>
#include "Sample.h"
#include "Sample_Res.h"
#include <ShrLib.h>     // Shared library declarations.

// Private constants.
const UInt16 prvGetFieldSize = 125;
const UInt16 prvSetFieldSize = 125;

// Private data.
static UInt16 prvRefNum = 0;     // Shared library reference number.

// Private application functions.
static Err     AppStart (void);
static void    AppStop (void);
static void    AppEventLoop (void);
static Boolean AppHandleEvent (EventPtr);
static void    AppLoadForm (Int16);
static Err     AppCheckVersion (UInt32, Int16);

// Private main form functions.
static Boolean    MainFormHandleEvent (EventPtr pEvent);
static void       MainFormHandleOpen (FormPtr formP);
static void       MainFormHandleClose (FormPtr formP);
static Boolean    MainFormHandleButton (FormPtr formP, Int16 buttonID);
static void       MainFormHandleGetText (FormPtr formP);
static void       MainFormHandleSetText (FormPtr formP);

// Private utility functions.
static FieldPtr   UtlGetField (FormPtr formP, Int16 fieldID);
static Err        UtlFldInit (FieldPtr fieldP, UInt16 textSize);
static Err        UtlFldTerm (FieldPtr fieldP);
static Err        UtlFldSetText (FieldPtr fieldP, char * text);
static Err        UtlFldGetText (FieldPtr fieldP, char * text, UInt16 size);
static void       UtlErrMessage (char * msg);

/*
   PilotMain ()
   Main entry point into pilot application.
   Parms:   wCmd  - launch code.
            pInfo - struct associated w/ launch code.
            fuLaunch - extra info about launch.
*/
UInt32 
PilotMain (UInt16 wCmd, MemPtr /*pInfo*/, UInt16 fuLaunch)
{
   Err   err;

   // Check version compatibility.
   err = AppCheckVersion (PALMOS_VERSION, fuLaunch);
   if (err)
   {
      // Incompatible - exit.
      return err;
   }
   
   switch (wCmd)
   {
      case sysAppLaunchCmdNormalLaunch:
      {
         // Initialize application.
         err = AppStart ();
         if (!err)
         {
            // Show the main form.
            FrmGotoForm (MainForm);
            AppEventLoop ();
            AppStop ();
         }
         break;
      }
      default:
      {
         err = 0;
         break;
      }
   }
   return err;
}

/*
   AppStart ():
   Initialize the application.
   Parms:   none
   Return:  0 if success, or error code.
*/
Err
AppStart ()
{
   UInt16 refNum = 0;
   char msg [50];
   Err err = SysLibLoad (shrTypeID, shrCreatorID, &refNum);

   if (!err)
   {
      err = ShrOpen (refNum);
      if (!err)
      {
         prvRefNum = refNum;
      }
      else
      {
         StrPrintF (msg, "Error %i opening.", (int) err);
         UtlErrMessage (msg);
         SysLibRemove (refNum);
      }
   }
   else
   {
      StrPrintF (msg, "Error %i loading.", (int) err);
      UtlErrMessage (msg);
   }
   return err;
}

/*
   AppStop ():
   Save the current state of the application.
   Parms:   none
   Return:  none
*/
void
AppStop ()
{
   // Make sure library was opened.
   if (0 != prvRefNum)
   {
      // Close the library
      ShrClose (prvRefNum);

      // Remove the library from memory.
      SysLibRemove (prvRefNum);

      // Reference number is no longer valid.
      prvRefNum = 0;
   }
}
   
/*
   AppEventLoop (void)
   Process events coming to application.
   Parms:   none
   Return:  none
   Note:    Application is in this function for majority
            of its execution.
*/
void
AppEventLoop (void)
{
   EventType   event;
   UInt16        error;
   
   do
   {
      EvtGetEvent (&event, evtWaitForever);
      
      // Ask system to handle event.
      if (false == SysHandleEvent (&event))
      {
         // Ask Menu to handle event.
         if (false == MenuHandleEvent (0, &event, &error))
         {
            // Menu did not handle event.
            // Ask App (that is, this) to handle event.
            if (false == AppHandleEvent (&event))
            {
               // App did not handle event.
               // Send event to appropriate form.
               FrmDispatchEvent (&event);
            }  // end if (false == AppHandleEvent (&event))
         }     // end if (false == MenuHandleEvent (0, &event, &error))
      }        // end if (false == SysHandleEvent (&event))
   }
   while (event.eType != appStopEvent);
}

/*
   AppEventHandler (EventPtr)
   Handle application-level events.
   Parms:   pEvent   - event to handle.
   Return:  true  - handled (successfully or not)
            false - not handled
*/
Boolean
AppHandleEvent (EventPtr pEvent)
{
   Boolean bHandled = false;
   
   switch (pEvent->eType)
   {
      case frmLoadEvent:
      {
         // Load the form resource.
         Int16  wFormID = pEvent->data.frmLoad.formID;
         
         AppLoadForm (wFormID);
         bHandled = true;
         break;
      }

      default:
      {
         break;
      }
   }
   return bHandled;
}

/*
   AppLoadForm (Word)
   Load a form.
   Parms:   wFormID  - form to be loaded.
   Return:  none
*/
void
AppLoadForm (Int16 wFormID)
{
   FormPtr pForm = FrmInitForm (wFormID);
   
   FrmSetActiveForm (pForm);
   
   // Set the event handler for the form.
   //    The handler of the currently active form is called by
   //    FrmHandleEvent each time it receives an event.
   switch (wFormID)
   {
      case MainForm:
      {
         FrmSetEventHandler (pForm, MainFormHandleEvent);
         break;
      }

      default:
      {
         break;
      }
   }
}

/*
   AppCheckVersion ():
   Check hardware compatibility.
   Parms:   dwReqVersion   - reqUtlred ROM version.
            fuLaunch       - launch flags.
   Return:  0 if ROM is compatible or error code.
*/
Err
AppCheckVersion (UInt32 dwReqVersion, Int16 fuLaunch)
{
   // Assume the best.
   Err err = 0;
   
   // Get the ROM version.
   UInt32 dwRomVersion;
   
   FtrGet (sysFtrCreator, sysFtrNumROMVersion, &dwRomVersion);
   
   // Check if its compatible.
   if (dwRomVersion < dwReqVersion)
   {
      // Version is too low, see what launch flags are.
      Int16  wLaunchCheck = sysAppLaunchFlagNewGlobals | sysAppLaunchFlagUIApp;
      
      if (wLaunchCheck & fuLaunch == wLaunchCheck)
      {
         // Tell user the problem.
			FrmAlert (RomIncompatibleAlert);
		
			// Pilot 1.0 will continuously relaunch this app unless we switch to 
			// another safe one.
			if (dwRomVersion < PALMOS_VERSION_20)
			{
				AppLaunchWithCommand(sysFileCDefaultApp, sysAppLaunchCmdNormalLaunch, NULL);
			}
		}
      err = sysErrRomIncompatible;
   }
   return err;
}
   
/*
   MainFormHandleEvent:
   Parms:   pEvent   - event to be handled.
   Return:  true  - handled (successfully or not)
            false - not handled
*/
Boolean
MainFormHandleEvent (EventPtr eventP)
{
   Boolean  handled = false;
   
   switch (eventP->eType)
   {
      case ctlSelectEvent:
      {
      // A control button was pressed.
         FormPtr formP = FrmGetActiveForm ();
         handled = MainFormHandleButton (formP, eventP->data.ctlEnter.controlID);
         break;
      }
      
      case frmOpenEvent:
      {
         FormPtr formP = FrmGetActiveForm ();
         MainFormHandleOpen (formP);
         FrmDrawForm (formP);
         handled = true;
         break;
      }
      
      case frmCloseEvent:
      {
         FormPtr formP = FrmGetActiveForm ();
         MainFormHandleClose (formP);
         handled = true;
      }

      default:
      {
         break;
      }
   }
   return handled;
}

/*
   MainFormHandleOpen:
   Respond to opening of main form.
   Parms:   formP - pointer to main form.
   Return:  none
*/
void
MainFormHandleOpen (FormPtr formP)
{
   FieldPtr fieldP = NULL;
   Err err = 0;
      
   // Initialize "Get" field.
   fieldP = UtlGetField (formP, MainGetField);
   err = UtlFldInit (fieldP, prvGetFieldSize);

   if (!err)
   {
      // Initialize "Set" field.
      fieldP = UtlGetField (formP, MainSetField);
      err = UtlFldInit (fieldP, prvSetFieldSize);
   }
}

/*
   MainFormHandleClose:
   Respond to closing of main form.
   Parms:   formP - pointer to main form.
   Return:  none
*/
void
MainFormHandleClose (FormPtr formP)
{
   FieldPtr fieldP = NULL;
   
   // Terminate "Get" field.
   fieldP = UtlGetField (formP, MainGetField);
   UtlFldTerm (fieldP);

   // Terminate "Set" field.
   fieldP = UtlGetField (formP, MainGetField);
   UtlFldTerm (fieldP);
}

/*
   MainFormHandleButton:
   Handle a command sent to the main form.
   Parms:   formP    - form handling event.
            buttonID - identifier of selected button.
   Return:  true  - handled (successfully or not)
            false - not handled
*/
Boolean
MainFormHandleButton (FormPtr formP, Int16 buttonID)
{
   Boolean handled = false;
   
   switch (buttonID)
   {
      case MainGetButton:  // User selected "Get" button.
      {
         MainFormHandleGetText (formP);
         handled = true;
         break;
      }
      
      case MainSetButton:  // User selected "Set" button.
      {
         MainFormHandleSetText (formP);
         handled = true;
         break;
      }
   }
   return handled;
}

/*
   MainFormHandleGetText:
   Handle user pressing "get" button.
   Parms:   formP    - form handling event.
   Return:  true  - handled (successfully or not)
            false - not handled
*/
void
MainFormHandleGetText (FormPtr formP)
{
   static Char text [100];

   // Retrieve text from shared library.
   ShrGetText (prvRefNum, text, sizeof text);

   // Retrieve pointer to field.
   FieldPtr fieldP = UtlGetField (formP, MainGetField);

   // Set the field's text.
   UtlFldSetText (fieldP, text);
}

/*
   MainFormHandleSetText:
   Handle user pressing "set" button.
   Parms:   formP    - form handling event.
   Return:  true  - handled (successfully or not)
            false - not handled
*/
void
MainFormHandleSetText (FormPtr formP)
{
   Char text [100];

   // Retrieve pointer to field.
   FieldPtr fieldP = UtlGetField (formP, MainSetField);

   // Retrieve text from field.
   UtlFldGetText (fieldP, text, sizeof text);

   // Set the text in the shared library.
   ShrSetText (prvRefNum, text);
}

/*
   UtlGetField:
   Retrieve a field pointer given the form and field identifier.
   Parms:   formP    - form containing the field.
   Return:  true     - handled (successfully or not)
            false    - not handled
*/
FieldPtr
UtlGetField (FormPtr formP, Int16 fieldID)
{
   // Retrieve the field index.
   Int16 fieldIndex = FrmGetObjectIndex (formP, fieldID);

   // Retrieve the field pointer.
   FieldPtr fieldP = (FieldPtr) FrmGetObjectPtr (formP, fieldIndex);

   // Return field pointer.
   return fieldP;
}

/*
   UtlFldInit:
   Initialize a field.
   Parms:   fieldP   - field being initialized.
            textSize - size of text associated with field.
   Return:  0        - Success
            -1       - Error
*/
Err
UtlFldInit (FieldPtr fieldP, UInt16 textSize)
{
   // Allocate the block.
   MemHandle textH = (MemHandle) MemHandleNew (textSize);

   // Lock the block.
   char * textP = (char *) MemHandleLock (textH);

   // Set block to '\0'.
   MemSet (textP, textSize, '\0');

   // Unlock the block.
   MemHandleUnlock (textH);

   // Set handle in field.
   FldSetTextHandle (fieldP, textH);
   
   // Return success.
   return 0;
}

Err
UtlFldTerm (FieldPtr fieldP)
{
   // Get handle from field.
   MemHandle textH = FldGetTextHandle (fieldP);

   // Set field handle to NULL.
   FldSetTextHandle (fieldP, NULL);

   // Free the text block.
   MemHandleFree (textH);
   
   // Return success.
   return 0;
}
   
Err
UtlFldSetText (FieldPtr fieldP, char * text)
{
   // Get handle for field's text buffer.
   MemHandle textH = FldGetTextHandle (fieldP);

   // Get size of text buffer.
   UInt16 textSize = FldGetTextAllocatedSize (fieldP);

   // Declare pointer to text.
   char * textP = NULL;

   // Set field handle to NULL.
   FldSetTextHandle (fieldP, NULL);

   // Lock the text handle.
   textP = (char *) MemHandleLock (textH);

   // Copy the text to the field.
   StrNCopy (textP, text, textSize-1);

   // Unlock text buffer.
   MemHandleUnlock (textH);

   // Set field's text buffer back to modified buffer.
   FldSetTextHandle (fieldP, textH);

   // Redraw the field.
   FldDrawField (fieldP);

   // Return success.
   return 0;
}

Err
UtlFldGetText (FieldPtr fieldP, char * text, UInt16 size)
{
   // Get handle for field's text buffer.
   MemHandle textH = FldGetTextHandle (fieldP);

   // Lock text buffer.
   char * textP = (char *) MemHandleLock (textH);

   // Copy the text to the field.
   StrNCopy (text, textP, size);

   // Unlock text buffer.
   MemHandleUnlock (textH);
   
   // Return success.
   return 0;
}

void
UtlErrMessage (char * msg)
{
   FrmCustomAlert (ErrorAlert, msg, " ", " ");
}
