/*
   PLM_APP.CPP

   Main app entry point
   Copyright (c) 2002 Bachmann Software and Services
   Author: Glenn Bachmann
*/

#include <PalmOS.h>

#include "plm_main.h"   // MainFormEventHandler ()
#include "plm_res.h"


// Private constants.
const unsigned long  appFileCreator = PLM_FILE_CREATOR;
const unsigned short appFileVersion = PLM_FILE_VERSION;
const unsigned short appPrefID      = PLM_PREF_ID;
const unsigned short appPrefVersion = PLM_PREF_VERSION;
const unsigned long  appOSVersion   = PALMOS_VERSION;
const unsigned long  osVersion30    = PALMOS_VERSION_30;
 
// Private structures.

/*
   AppPreferences:
   Application preferences.
   Members: 
*/
struct AppPreferences
{
   Char byReplaceMe;
};

/*
   AppInformation:
   Application information.
   Members:
*/
struct AppInformation
{
   Char  byReplaceMe;
};

// Private functions.
static Err     AppStart ();
static void    AppStop ();
static void    AppEventLoop (void);
static Boolean AppEventHandler (EventPtr);
static void    AppLoadForm (Int16);
static Err     AppReadPreferences ();
static void    AppWritePreferences ();
static Err     AppCheckVersion (UInt32, Int16);

// Private data.
static AppPreferences   s_appPref;
static AppInformation   s_appInfo;

/*
   PilotMain ()
   Main entry point into pilot application.
   Parms:   wCmd  - launch code.
            pInfo - struct associated w/ launch code.
            fuLaunch - extra info about launch.
*/
UInt32	PilotMain (UInt16 wCmd, MemPtr cmdPBP, UInt16 fuLaunch)
{
   Err   err;

   // Check version compatibility.
   err = AppCheckVersion (appOSVersion, fuLaunch);
   if (err)
   {
      // Incompatible - exit.
      return err;
   }
   
   switch (wCmd)
   {
      case sysAppLaunchCmdNormalLaunch:
      {
         // Initialize application.
         err = AppStart ();
         if (!err)
         {
            // Show the main form.
            FrmGotoForm (MainForm);
            AppEventLoop ();
            AppStop ();
         }
         break;
      }
      default:
      {
         err = 0;
         break;
      }
   }

   return err;
}

/*
   AppStart ():
   Initialize the application.
   Parms:   none
   Return:  0 if success, or error code.
*/
Err
AppStart ()
{
   // Read preferences for the app.
   Err err = AppReadPreferences  ();
   
   return 0;
}

/*
   AppStop ():
   Save the current state of the application.
   Parms:   none
   Return:  none
*/
void
AppStop ()
{
}
   
/*
   AppEventLoop (void)
   Process events coming to application.
   Parms:   none
   Return:  none
   Note:    Application is in this function for majority
            of its execution.
*/
void
AppEventLoop (void)
{
   EventType   event;
   
   do
   {
      EvtGetEvent (&event, evtWaitForever);

      // Ask system to handle event.
      if (false == SysHandleEvent (&event))
      {
         // System did not handle event.
         
         UInt16        error;
         // Ask Menu to handle event.
         if (false == MenuHandleEvent (0, &event, &error))
         {
            // Menu did not handle event.
            // Ask App (that is, this) to handle event.
            if (false == AppEventHandler (&event))
            {
               // App did not handle event.
               // Send event to appropriate form.
               FrmDispatchEvent (&event);
            }  // end if (false == AppEventHandler (&event))
         }     // end if (false == MenuHandleEvent (0, &event, &error))
      }        // end if (false == SysHandleEvent (&event))
   }
   while (event.eType != appStopEvent);
}

/*
   AppEventHandler (EventPtr)
   Handle application-level events.
   Parms:   pEvent   - event to handle.
   Return:  true  - handled (successfully or not)
            false - not handled
*/
Boolean
AppEventHandler (EventPtr pEvent)
{
   Boolean bHandled = false;
   
   switch (pEvent->eType)
   {
      case frmLoadEvent:
      {
         // Load the form resource.
         Int16  wFormID = pEvent->data.frmLoad.formID;
         
         AppLoadForm (wFormID);
         bHandled = true;
      }
      
      default:
      {
         break;
      }
   }
   return bHandled;
}

/*
   AppLoadForm (Word)
   Load a form.
   Parms:   wFormID  - form to be loaded.
   Return:  none
*/
void
AppLoadForm (Int16 wFormID)
{
   FormPtr pForm = FrmInitForm (wFormID);
   
   FrmSetActiveForm (pForm);
   
   // Set the event handler for the form.
   //    The handler of the currently active form is called by
   //    FrmHandleEvent each time it receives an event.
   switch (wFormID)
   {
      case MainForm:
      {
         FrmSetEventHandler (pForm, MainFormEventHandler);
         break;
      }
      default:
      {
         break;
      }
   }
}

/*
   AppReadPreferences:

   Reads the app prefs to determine if the trial period has expired
   Parms: none
   Returns: none
*/

Err
AppReadPreferences ()
{
	return 0;
}

/*
   AppCheckVersion ():
   Check hardware compatibility.
   Parms:   dwReqVersion   - required ROM version.
            fuLaunch       - launch flags.
   Return:  0 if ROM is compatible or error code.
*/
Err
AppCheckVersion (UInt32 dwReqVersion, Int16 fuLaunch)
{
   // Assume the best.
   Err err = 0;
   
   // Get the ROM version.
   UInt32 dwRomVersion;
   
   FtrGet (sysFtrCreator, sysFtrNumROMVersion, &dwRomVersion);
   
   // Check if its compatible.
   if (dwRomVersion < dwReqVersion)
   {
      // Version is too low, see what launch flags are.
      Int16  wLaunchCheck = sysAppLaunchFlagNewGlobals | sysAppLaunchFlagUIApp;
      
      if (wLaunchCheck & fuLaunch == wLaunchCheck)
      {
         // Tell user the problem.
			FrmAlert (RomIncompatibleAlert);
		
			// Pilot 1.0 will continuously relaunch this app unless we switch to 
			// another safe one.
			if (dwRomVersion < osVersion30)
			{
				AppLaunchWithCommand(sysFileCDefaultApp, sysAppLaunchCmdNormalLaunch, NULL);
			}
		}
      err = sysErrRomIncompatible;
   }
   return err;
}
   
